/*-
 * ~~~~~~licensing~~~~~~
 * base-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.bundle;

import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.osgi.framework.wiring.BundleCapability;
import org.osgi.framework.wiring.BundleWiring;

public class BundleCapabilityHelper extends CapabilityHelper {
	
	protected Bundle bundle; 

	public BundleCapabilityHelper(Bundle bundle) {
		super();
		this.bundle = bundle;
		setMultipleNamespacesSupported(true);
		load();
	}
	
	@Override
	protected void load() {
	    BundleWiring bundleWiring = bundle.adapt(BundleWiring.class);
	    if (bundleWiring == null) 
	    	return;
	    List<BundleCapability> capabilities = bundleWiring.getCapabilities(null);
	    for (BundleCapability cap: capabilities) {
	    	CapabilityDescriptorImpl descriptor = new CapabilityDescriptorImpl();
	    	descriptor.namespace(cap.getNamespace());
	    	for (Map.Entry<String, Object> attr: cap.getAttributes().entrySet())
	    		descriptor.attribute(attr.getKey(), attr.getValue());
	    	addProvidedCapability(descriptor);
	    }
	}
	
	public CapabilityDescriptor findObjectDeclaration(String objectId, String objectType) {
		if (this.providedCapabilities == null)
			return null;
		if (this.providedCapabilities.get(objectType) == null)
			return null;
		for (CapabilityDescriptor desc: this.providedCapabilities.get(objectType))
			if (objectId.equals(desc.getAttributes().getOrDefault("objectId", "").toString()))
				return desc;
		return null;
				
	}
	
}
