/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.bundle;

import org.osgi.resource.Capability;
import java.util.Map;

public interface CapabilityDescriptor extends Capability {

	public static final String HEADER_PROVIDE_CAPABILITY = "Provide-Capability";
	public static final String HEADER_REQUIRE_CAPABILITY = "Require-Capability";
	
	public static interface ATTRIBUTE_TYPES {
		
		public static final String STRING = "String";
		public static final String VERSION = "Version";
		public static final String LONG = "Long";
		public static final String DOUBLE = "Double";
		public static final String LIST = "List";
		
		public static String TYPED_LIST(String itemType) {
			return LIST + "<" + itemType + ">";
		}

		public static boolean isList(String itemType) {
			return (itemType!=null) && (itemType.startsWith(LIST));
		}

		public static String itemType(String listType) {
			if (!isList(listType))
				return null;
			
			return listType.substring(listType.indexOf("<"), listType.indexOf(">"));
		}
		
	}
	
	public CapabilityDescriptor namespace(String namespace);
	public CapabilityDescriptor attributes(Map<String, Object> attributes);
	public CapabilityDescriptor attribute(String name, Object value);
	public CapabilityDescriptor attribute(String name, Object value, String type);
	
}
