/*-
 * ~~~~~~licensing~~~~~~
 * system-commons
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.service;

import org.osgi.framework.ServiceReference;

import java.util.ArrayList;
import java.util.List;

public class CommonReferenceListener<T> implements ReferenceListener<T> {

    protected List<T> registeredReferences = new ArrayList<T>();

    public void register(T service) {
        if (service instanceof ServiceReference)
            return;
        registeredReferences.add(service);
        doAfterRegister(service);
    }

    protected void doAfterRegister(T service) {
        // to override
    }

    protected void doBeforeUnregister(T service) {
        // to override
    }

    public void unregister(T service) {
        if (service instanceof ServiceReference)
            return;
        if (registeredReferences.contains(service)) {
            doBeforeUnregister(service);
            registeredReferences.remove(service);
        }
    }

}
