/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.tracker;

import java.util.LinkedList;
import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.FrameworkUtil;
import org.osgi.util.tracker.BundleTracker;

public class BundleTrackerUtils {

	public static class BundleTrackerBuilder<T> {
	
		protected UniformBundleTrackerCustomizer<T> customizer; 
		
		protected List<BundleFilter<?>> filters = new LinkedList<>();
		
		protected BundleContext bundleContext = FrameworkUtil.getBundle(BundleTrackerUtils.class).getBundleContext();
		
		protected int bundleState = Bundle.ACTIVE;
		
		public BundleTrackerBuilder<T> customizer(UniformBundleTrackerCustomizer<T> customizer){
			this.customizer = customizer;
			return this;
		} 
		
		public BundleTrackerBuilder<T> bundleContext(BundleContext bundleContext){
			this.bundleContext = bundleContext;
			return this;
		} 
		
		public BundleTrackerBuilder<T> addFilter(BundleFilter<?> bundleFilter){
			this.filters.add(bundleFilter);
			return this;
		}
		
		public BundleTrackerBuilder<T> bundleState(int state){
			this.bundleState = state;
			return this;
		} 
		
		public BundleTracker<T> get(){
			if (customizer == null)
				return null;
			if (customizer instanceof BundleContextAware)
				((BundleContextAware)customizer).setBundleContext(bundleContext);
			for (BundleFilter<?> filter: this.filters) {
				if (filter instanceof BundleContextAware)
					((BundleContextAware)filter).setBundleContext(bundleContext);
				customizer.addFilter(filter);
			}
			return new BundleTracker<T>(bundleContext, bundleState, customizer);
		}
	}
	
	public static <T> BundleTrackerBuilder<T> createBuilder(){
		return new BundleTrackerBuilder<T>(); 
	} 
	
}
