/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.tracker;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.osgi.util.tracker.BundleTrackerCustomizer;

public abstract class UniformBundleTrackerCustomizer<T> implements BundleTrackerCustomizer<T> {
	
	protected BundleTrackerCustomizerListener<T> listener;
	
	protected List<BundleFilter<?>> filters = new LinkedList<>();
	
	public UniformBundleTrackerCustomizer() {
		super();
	}
	
	public UniformBundleTrackerCustomizer(BundleTrackerCustomizerListener<T> listener) {
		super();
		setListener(listener);
	}

	public BundleTrackerCustomizerListener<T> getListener() {
		return listener;
	}

	public void setListener(BundleTrackerCustomizerListener<T> listener) {
		this.listener = listener;
	} 

	public void addFilter(BundleFilter<?> filter) {
		this.filters.add(filter);
	}
	
	public UniformBundleTrackerCustomizer<T> listener(BundleTrackerCustomizerListener<T> listener){
		setListener(listener);
		return this;
	}
	
	@Override
	public T addingBundle(Bundle bundle, BundleEvent event) {
		Map<String, List<?>> filterResults = new HashMap<>();
		for (BundleFilter<?> filter: filters) {
			List<?> result = filter.checkBundle(bundle, event);
			if (result == null)
				return null;
			filterResults.put(filter.getFilterId(), result);
		}
		T managedObject = createManagedObject(bundle, event, filterResults);
		enrichManagedObject(bundle, event, filterResults, managedObject);
		if (listener != null)
			listener.added(managedObject);
		return managedObject;
	}
	
	protected abstract T createManagedObject(Bundle bundle, BundleEvent event, Map<String, List<?>> filterResults);

	/*
	 * to override
	 */
	protected void enrichManagedObject(Bundle bundle, BundleEvent event, Map<String, List<?>> filterResults, T managedObject) {
		
	}
	
	@Override
	public void modifiedBundle(Bundle bundle, BundleEvent event, T object) {
		listener.modified(object);
	}
	
	@Override
	public void removedBundle(Bundle bundle, BundleEvent event, T object) {
		listener.removed(object);
	}
	
}
