/*-
 * ~~~~~~licensing~~~~~~
 * base-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.tracker.filter;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;

import ru.entaxy.platform.base.support.CommonUtils;

public class BundleHeaderFilter extends AbstractBundleFilter<String, BundleHeaderFilter> {

	public static final String DEFAULT_FILTER_ID = "bundle.header";
	
	protected String header;
	
	protected String value = null;
	
	@Override
	protected String getDefaultFilterId() {
		return DEFAULT_FILTER_ID;
	}

	@Override
	public List<String> checkBundle(Bundle bundle, BundleEvent event) {
		Map<String, String> headers = CommonUtils.getDictionaryAsMap(bundle.getHeaders());
		if (!headers.containsKey(header))
			return null;
		String headerValue = headers.get(header);
		if (value != null)
			if (!value.equals(headerValue))
				return null;
		return Arrays.asList(new String[] {headerValue});
	}
	
	public String getHeader() {
		return header;
	}

	public void setHeader(String header) {
		this.header = header;
	}

	public BundleHeaderFilter header(String headerValue) {
		this.setHeader(headerValue);
		return this;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	};
	
	public BundleHeaderFilter value(String valueValue) {
		this.setValue(valueValue);
		return this;
	};
}
