/*-
 * ~~~~~~licensing~~~~~~
 * configuration-test-1
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.config;

import java.util.Map;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DefaultPropertiesProviderWithImmutables extends DefaultPropertiesProvider implements Immutables {

	private static final Logger log = LoggerFactory.getLogger(DefaultPropertiesProviderWithImmutables.class);
	
	protected Map<String, Object> immutables;
	
	public DefaultPropertiesProviderWithImmutables(String objectId
			, String objectType
			, String configurationPid
			, BundleContext bundleContext
			, Map<String, Object> configurables) {
		super(objectId, objectType, configurationPid, bundleContext, configurables);
	}

	public DefaultPropertiesProviderWithImmutables(String configurationPid
			, BundleContext bundleContext
			, Map<String, Object> configurables) {
		super(configurationPid, bundleContext, configurables);
	}

	public DefaultPropertiesProviderWithImmutables(String configurationPid
			, BundleContext bundleContext
			, Map<String, Object> configurables
			, Map<String, Object> immutables) {
		super(configurationPid, bundleContext, configurables);
		this.immutables = immutables;
		registerImmutables();
	}

	public DefaultPropertiesProviderWithImmutables(String objectId
			, String objectType
			, String configurationPid
			, BundleContext bundleContext
			, Map<String, Object> configurables
			, Map<String, Object> immutables) {
		super(objectId, objectType, configurationPid, bundleContext, configurables);
		this.immutables = immutables;
		registerImmutables();
	}
	
	protected void registerImmutables() {
		if ((this.configurationPid != null) && (this.bundleContext != null)) {
			
			ServiceReference<ImmutablesCollector> collectorRef = this.bundleContext.getServiceReference(ImmutablesCollector.class);
			if (collectorRef != null) {
				ImmutablesCollector collector = this.bundleContext.getService(collectorRef);
				if (collector != null)
					collector.add(this);
				else
					log.error("registerImmutables :: COLLECTOR IS NULL");
				this.bundleContext.ungetService(collectorRef);
			} else {
				log.error("registerImmutables :: REF IS NULL");
			}
		}
		
	}

	@Override
	public String getPid() {
		return this.configurationPid;
	}

	@Override
	public Map<String, Object> getProperties() {
		return this.immutables;
	};

}
