/*-
 * ~~~~~~licensing~~~~~~
 * adapters-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.base.connecting.adapter.core.management;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;
import javax.management.StandardMBean;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ServiceScope;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.runtime.base.connecting.adapter.core.api.Adapter;

@Component(
		service = {AdaptersMBean.class, DynamicMBean.class, MBeanRegistration.class},
		// property = {"jmx.objectname=ru.entaxy.esb:group=platform,category=adapters"},
		property = {ManagementCore.ENTAXY_JMX_DOMAIN + "=" + AdaptersMBean.Q_ADAPTERS_S},
		scope = ServiceScope.SINGLETON,
		immediate = true
		)
public class AdaptersMBeanImpl extends StandardMBean implements AdaptersMBean {

	private static final Logger log = LoggerFactory.getLogger(AdaptersMBeanImpl.class);
	
	protected Map<Adapter, ServiceRegistration<AdapterMBean>> mbeans = new HashMap<>();
	
	// @Reference (service = BundleContext.class)
	protected BundleContext bundleContext;
	
	public AdaptersMBeanImpl() throws NotCompliantMBeanException {
		super(AdaptersMBean.class);
	}
	
	
	@Activate
	public void activate(ComponentContext componentContext) {
		log.debug(" >> ACTIVATE <<");
		this.bundleContext = componentContext.getBundleContext();
	}
	
	
	@Reference (service = Adapter.class, cardinality = ReferenceCardinality.MULTIPLE, 
			unbind = "unbindAdapter", collectionType = CollectionType.SERVICE, 
			policy = ReferencePolicy.DYNAMIC)
	public void bindAdapter(Adapter adapter) {
		
		try {
			AdapterMBeanImpl mbean = new AdapterMBeanImpl(adapter);
			Hashtable props = new Hashtable<>();
			// props.put("jmx.objectname", "ru.entaxy.esb:group=platform,category=adapters,id=" + adapter.getId());
			props.put(ManagementCore.JMX_OBJECTNAME, Q_ADAPTERS.qualifier("id", adapter.getId()).getValue());
			ServiceRegistration<AdapterMBean> reg = bundleContext.registerService(
					new String[] {
							AdapterMBean.class.getName(),
							DynamicMBean.class.getName(),
							MBeanRegistration.class.getName()
						}
					, mbean
					, props);
			this.mbeans.put(adapter, reg);
			
		} catch (NotCompliantMBeanException e) {
			log.error("Error creating MBean for adapter: ", e);
		}
		
	}
	
	public void unbindAdapter(Adapter adapter) {
		ServiceRegistration<AdapterMBean> reg = this.mbeans.get(adapter);
		if (reg != null)
			reg.unregister();
		this.mbeans.remove(adapter);
	}
	
	@Override
	public void execute(String value) {
		log.info(">> EXECUTED");
	}
	
}
