/*-
 * ~~~~~~licensing~~~~~~
 * adapters-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.base.connecting.adapter.core.tracker;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.framework.wiring.BundleWiring;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.platform.runtime.base.connecting.adapter.core.api.Adapter;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

public class AdapterHelper {
	
	protected static final Logger log = LoggerFactory.getLogger(AdapterHelper.class);
	
	protected BundleContext bundleContext;
	
	protected List<Object> objects = new ArrayList<>();
	protected List<ServiceRegistration> registrations = new ArrayList<>();
	
	public AdapterHelper(BundleContext bundleContext) {
		this.bundleContext = bundleContext;
	}
	
	public void create(Bundle targetBundle, String className) {
		log.info("Creating adapter in " + targetBundle.getSymbolicName()
			+ " of class " + targetBundle.getClass().getName()
			+ " having context of " + targetBundle.getBundleContext().getClass().getName());
		BundleWiring wiring = targetBundle.adapt(BundleWiring.class);
		ClassLoader cl = wiring.getClassLoader();
		try {
			Class<?> clazz = cl.loadClass(className);
			Constructor<?> constructor = clazz.getConstructor(BundleContext.class);
			Object obj = constructor.newInstance(targetBundle.getBundleContext());
			log.debug("Created object of class {}", obj.getClass().getName());

			Adapter adapter = (Adapter)obj;
			if (!adapter.isInited()) {
				log.warn("Adapter initialization failed.");
				return;
			}
			
			Hashtable<String, String> properties = new Hashtable<>();
			properties.put("adapter.id", adapter.getId());
			properties.put("adapter.name", adapter.getName());
			properties.put("adapter.description", adapter.getDescription());
			
			ServiceRegistration sr = targetBundle.getBundleContext().registerService(new String[] {Adapter.class.getName()}, obj, properties);

//			// print out adapter info
//			String props = "\n";
//			for (Map.Entry<String, String> entry: adapter.getProperties().entrySet())
//				props += entry.getKey() + ": " + entry.getValue() + "\n";
//			String opts = "\n";
//			for (Map.Entry<String, String> entry: adapter.getOptions().entrySet())
//				opts += entry.getKey() + ": " + entry.getValue() + "\n";
//
//			log.info("\n\n\tFINAL ADAPTER INFO: \nID: {}\nDescription: {}\nProperties:{}Options:{}"
//					, adapter.getId()
//					, adapter.getDescription()
//					, props
//					, opts);
			
		} catch (Exception e) {
			log.error("Can't create adapter:\n", e);
		}
	}
}
