/*-
 * ~~~~~~licensing~~~~~~
 * generator-api
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.base.generator.template.impl;

import java.net.URL;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;

import ru.entaxy.base.generator.template.TemplateService;
import ru.entaxy.esb.system.core.template.Template;

@Component(immediate = true)
public class LegacyTemplateAdapter {

	BundleContext bundleContext = null;
	
	List<Template> delayedTemplates = new ArrayList<>();
	
	Map<Template, ServiceRegistration<ru.entaxy.base.generator.template.Template>> services = new HashMap<>();
	
	protected class TemplateWrapper implements ru.entaxy.base.generator.template.Template {
		
		Template template;
		
		public TemplateWrapper(Template template) {
			this.template = template;
		}

		@Override
		public URL getTemplateLocation() {
			return template.getTemplateLocation();
		}

		@Override
		public String getId() {
			return template.getTemplateName();
		}

		@Override
		public String getType() {
			return "ftl";
		}

		@Override
		public String getTemplateName() {
			return template.getTemplateName();
		}

		@Override
		public String getTemplateFileName() {
			return template.getTemplateFileName();
		}

		@Override
		public String getTemplateFullName() {
			return template.getTemplateFileName() + ".ftl";
		}

		@Override
		public Map<String, String> getAdditionalProperties() {
			return null;
		}
		
	}
	
	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, unbind = "removeTemplate")
	public void addTemplate(Template template) {
		if (bundleContext == null)
			delayedTemplates.add(template);
		else
			wrapTemplate(template);
	}
	
	public void removeTemplate(Template template) {
		this.services.remove(template);
	}
	
	@Activate
	public void activate(BundleContext context) {
		this.bundleContext = context;
		processDelayed();
	}
	
	protected void processDelayed() {
		for (Template t: delayedTemplates)
			wrapTemplate(t);
		delayedTemplates.clear();
	}
	
	protected void wrapTemplate(Template template) {
		URL url = template.getTemplateLocation();
		Dictionary<String, String> props = new Hashtable<>();
		props.put(TemplateService.PROP_ID, template.getTemplateName());
		props.put(TemplateService.PROP_NAME, template.getTemplateName());
		props.put(TemplateService.PROP_DESCRIPTION, template.getTemplateName());
		props.put(TemplateService.PROP_TYPE, "ftl");
		props.put("origin.location", url.toString());
		props.put("origin.filename", template.getTemplateFileName());
		ServiceRegistration<ru.entaxy.base.generator.template.Template> service = 
				this.bundleContext.registerService(ru.entaxy.base.generator.template.Template.class
					, new TemplateWrapper(template)
					, props);
		this.services.put(template, service);
	}
}
