/*-
 * ~~~~~~licensing~~~~~~
 * generator-api
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.base.generator.template.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.base.generator.template.Template;
import ru.entaxy.base.generator.template.TemplateService;

@Component(service = TemplateService.class, immediate = true)
public class TemplateServiceImpl implements TemplateService {

	private static final Logger log = LoggerFactory.getLogger(TemplateServiceImpl.class);
	
	private BundleContext bundleContext = null;
	
	protected final Map<String, Template> templates = new HashMap<>();
	
	protected List<ServiceReference<Template>> delayed = new ArrayList<>();
	
	@Reference(cardinality = ReferenceCardinality.MULTIPLE, collectionType = CollectionType.REFERENCE
	, policy = ReferencePolicy.DYNAMIC, unbind = "removeTemplate")
	public void addTemplate(ServiceReference<Template> templateService) {
		log.info("Added template service with id [" + templateService.getProperty(PROP_ID) + "]");
		if (this.bundleContext == null)
			this.delayed.add(templateService);
		else
			processTemplate(templateService);
	}
	
	public void removeTemplate(ServiceReference<Template> templateService) {
		String id = templateService.getProperty(PROP_ID).toString();
		this.bundleContext.ungetService(templateService);
		this.templates.remove(id);
	}

	@Activate
	public void activate(BundleContext context) {
		this.bundleContext = context;
		processDelayed();
	}
	
	protected void processDelayed() {
		for (ServiceReference<Template> ref: delayed)
			processTemplate(ref);
		delayed.clear();
	}
	
	protected void processTemplate(ServiceReference<Template> reference) {
		Template t = this.bundleContext.getService(reference);
		this.templates.put(t.getId(), t);
	}
	
	@Override
	public List<Template> getAllTemplates() {
		return new ArrayList<>(this.templates.values());
	}
	
	@Override
	public Template getTemplateById(String id) {
		return this.templates.get(id);
	}
}
