/*-
 * ~~~~~~licensing~~~~~~
 * generator-api
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.base.generator.template.impl;

import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.url.AbstractURLStreamHandlerService;
import org.osgi.service.url.URLStreamHandlerService;

import ru.entaxy.base.generator.template.Template;
import ru.entaxy.base.generator.template.TemplateService;

@Component(service = URLStreamHandlerService.class, property = "url.handler.protocol=templates", immediate = true)
public class TemplateServiceURLResolver extends AbstractURLStreamHandlerService {

	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	TemplateService templateService;
	
	@Override
	public URLConnection openConnection(URL url) throws IOException {
		String templateId = url.toString();
		templateId = templateId.replace("templates:", "");
		templateId = templateId.replace("/", ".");
		
		Template template = templateService.getTemplateById(templateId);
		if (template == null) {
			// we'll try without file extension
			int index = templateId.lastIndexOf(".");
			if (index>0)
				templateId = templateId.substring(0, index);
			template = templateService.getTemplateById(templateId);
		}

		if (template == null)
			return null;
		
		String templateUrl = template.getTemplateLocation().toString() + template.getTemplateFullName();
		URL resultUrl = new URL(templateUrl);
		return resultUrl.openConnection();
	}

}
