/*-
 * ~~~~~~licensing~~~~~~
 * generator-api
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.base.connecting.generator;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractSelfPublishGenerator<T extends AbstractSelfPublishGenerator<T>> implements Generator, GeneratorService {

	private static final Logger log = LoggerFactory.getLogger(AbstractSelfPublishGenerator.class);
	
	protected Map<String, Object> initialProperties = new HashMap<>();
	
	@Override
	public Generator getGenerator(Map<String, Object> initialProperties) {
		String errorClass = "CLASS=[" + this.getClass().getName() + "]";
		try {
			Constructor<T> constructor = ((Class<T>)this.getClass()).getConstructor();
			T instance = constructor.newInstance();
			instance.init(initialProperties);
			return (Generator) instance;
		} catch (NoSuchMethodException | SecurityException e) {
			log.error(errorClass, e);
		} catch (InstantiationException e) {
			log.error(errorClass, e);
		} catch (IllegalAccessException e) {
			log.error(errorClass, e);
		} catch (IllegalArgumentException e) {
			log.error(errorClass, e);
		} catch (InvocationTargetException e) {
			log.error(errorClass, e);
		}
		return null;
	}

	@Override
	public String getGeneratorId() {
		return this.getClass().getName();
	}
	
	protected void init(Map<String, Object> initialProperties) {
		this.initialProperties.putAll(initialProperties);
	}
	
}
