/*-
 * ~~~~~~licensing~~~~~~
 * template-service-test
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.base.generator.template.shell;

import java.util.HashMap;
import java.util.Map;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Argument;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;

import ru.entaxy.base.generator.template.Template;
import ru.entaxy.base.generator.template.TemplateAwareGenerator;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generator;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.factory.GeneratorFactory;
import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.base.support.JSONUtils;

@Service
@Command(scope = "templates", name = "generate")
public class Generate extends TemplateServiceSupport implements Action {

	@Argument(index = 0, name = "template-id", required = true)
	String templateId;
	
	@Argument(index = 1, name = "parameters", required = false)
	String params;
	
	@Override
	public Object execute() throws Exception {
		Template t = templateService.getTemplateById(templateId);
		if (t == null) {
			System.out.println("Template [" + templateId + "] not found");
			return null;
		}
		TemplateAwareGenerator g = GeneratorFactory.createGenerator(t);
		if (g == null) {
			System.out.println("Generator for template [" + templateId + "] not found");
			return null;
		}
		Generated result = null;
		if (CommonUtils.isValid(params)) {
			GsonBuilder builder = new GsonBuilder();
			Gson gson = builder.create();

			JsonElement je = (new JsonParser()).parse(params);
			Map<String, Object> map = JSONUtils.element2map(je);

			result = g.generate(t, map);
		} else {
			result = g.generate(t, new HashMap<>()); 
		}
		if (result == null) {
			System.out.println("Generated is empty, template: [" + templateId + "] not found");
			return null;
		} else {
			System.out.println("Generated: ");
			System.out.println("\ttype: " + result.getType());
			System.out.println("\tcontent: " + (result.getObject()==null?"null":"[==\n" + result.getObject().toString() + "\n==]"));
		}
		return null;
	}
	
}
