/*-
 * ~~~~~~licensing~~~~~~
 * template-service-test
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.base.generator.template.shell;

import java.util.Comparator;
import java.util.stream.Collectors;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;
import org.apache.karaf.shell.support.table.ShellTable;

import ru.entaxy.base.generator.template.Template;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generator;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.factory.GeneratorFactory;

@Service
@Command(scope = "templates", name = "list")
public class ListTemplates extends TemplateServiceSupport implements Action {

	@Override
	public Object execute() throws Exception {
		
		ShellTable table = new ShellTable();
		table.column("id");
		table.column("File");
		table.column("Location");
		table.column("Generator");
		
		for (Template t: templateService.getAllTemplates().stream().sorted(new Comparator<Template>() {

			@Override
			public int compare(Template o1, Template o2) {
				return o1.getId().compareToIgnoreCase(o2.getId());
			}
			
		}).collect(Collectors.toList()) ) {
			Generator g = (Generator)GeneratorFactory.createGenerator(t);
			table.addRow().addContent(
					t.getId(),
					t.getTemplateFullName(),
					t.getTemplateLocation()==null?"":t.getTemplateLocation().toString(), 
					(g==null?"NOT FOUND":g.getGeneratorId())
				);
		}
		
		table.print(System.out);
		
		return null;
	}

}
