/*-
 * ~~~~~~licensing~~~~~~
 * management-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.base.management.core.api;

import javax.management.Descriptor;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanOperationInfo;
import javax.management.NotCompliantMBeanException;
import javax.management.StandardMBean;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.utils.MBeanInfoHelper;

public class AnnotatedMBean<I> extends StandardMBean {

	private static final Logger log = LoggerFactory.getLogger(AnnotatedMBean.class);
	
	protected Class<I> interfaceClass;
	
	protected AnnotatedMBean(Class<I> mbeanInterface) throws NotCompliantMBeanException {
		super(mbeanInterface);
		this.interfaceClass = mbeanInterface;
	}

	@Override
	public MBeanInfo getMBeanInfo() {
		log.debug(this.interfaceClass==null?">> INTERFACE IS NULL":">> " + this.interfaceClass.getName());
		MBeanExportPolicy policy = MBeanExportPolicy.IGNORE_ANNOTATIONS;
		MBeanAnnotated anno = this.interfaceClass.getAnnotation(MBeanAnnotated.class);
		if (anno != null)
			policy = anno.policy();
		
		MBeanInfo beanInfo = super.getMBeanInfo();
		if (MBeanExportPolicy.IGNORE_ANNOTATIONS.equals(policy))
			return beanInfo; 
		
		String className = beanInfo.getClassName();
		String description = beanInfo.getDescription();
		MBeanAttributeInfo[] attributes = /* beanInfo.getAttributes() */ getMBeanAttributeInfo(policy);
		MBeanConstructorInfo[] constructors = beanInfo.getConstructors();
		MBeanOperationInfo[] operations = /* beanInfo.getOperations() */ getMBeanOperationInfo(policy);
		MBeanNotificationInfo[] notifications = beanInfo.getNotifications();
		Descriptor descriptor = beanInfo.getDescriptor();
		
		return new MBeanInfo(className, description, attributes, constructors, operations, notifications, descriptor); 
	}
	
	protected MBeanAttributeInfo[] getMBeanAttributeInfo(MBeanExportPolicy policy) {
		return MBeanInfoHelper.getMBeanAttributesInfo(this.interfaceClass, policy);
	} 
	
	protected MBeanOperationInfo[] getMBeanOperationInfo(MBeanExportPolicy policy) {
		return MBeanInfoHelper.getMBeanOperationsInfo(this.interfaceClass, policy);
	}
	
}
