/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.factory.configuration;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory.CONFIGURATION.FIELDS;

public class FieldsElement extends AbstractElement<FieldsElement> {
	
	private static final Logger log = LoggerFactory.getLogger(FieldsElement.class);
	
	public static final String ELEMENT_NAME = EntaxyFactory.CONFIGURATION.FIELDS_SECTION_NAME;

	protected Map<String, FieldElement> fields = new LinkedHashMap<>();
	
	@Override
	public FieldsElement fromJson(JsonElement jsonElement) {
		Gson gson = new Gson(); 
		if (jsonElement.isJsonObject()) {
			for (Entry<String, JsonElement> entry : jsonElement.getAsJsonObject().entrySet()) {
				
				Class<? extends FieldElement> feClass = FieldElement.class;
				
				if (entry.getValue().isJsonObject())
					if (entry.getValue().getAsJsonObject().has(FIELDS.ATTRIBUTES.IS_REF)) {
						JsonElement val = entry.getValue().getAsJsonObject().get(FIELDS.ATTRIBUTES.IS_REF);
						if (val.isJsonPrimitive() && val.getAsJsonPrimitive().isBoolean())
							if (val.getAsJsonPrimitive().getAsBoolean()) {
								feClass = RefFieldElement.class;
							}
					}
				
				FieldElement fe = gson.fromJson(entry.getValue(), feClass);
				fe.setName(entry.getKey());
				fe.setJsonOrigin(entry.getValue().isJsonObject()?entry.getValue().getAsJsonObject():new JsonObject());
				addField(fe);
			}
		}
		/*
		 * this.fields = gson.fromJson( jsonElement,
		 * TypeToken.getParameterized(ArrayList.class, FieldElement.class).getType() );
		 */
		// processOutputs();
		return this;
	}

	protected void addField(FieldElement fieldElement) {
		this.fields.put(fieldElement.getName(), fieldElement);
	}
	
	public Map<String, FieldElement> getFieldsMap(){
		return this.fields;
	}
	
	public List<FieldElement> getFields() {
		return new ArrayList<>(fields.values());
	}	
	
}
