/*-
 * ~~~~~~licensing~~~~~~
 * object-factory
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.factory.tracker;

import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.support.JSONUtils;

public class TrackedFactoryContainer {

	private static final Logger log = LoggerFactory.getLogger(TrackedFactoryContainer.class);

	private static final String FACTORY_ROOT_PATH = "/ru/entaxy/factory/";
	
	Bundle bundle;
	Map<String, TrackedFactory> factories = new LinkedHashMap<>();
	
	public TrackedFactoryContainer(Bundle bundle) {
		super();
		this.bundle = bundle;
	}
	
	public void reload() {
		Enumeration<URL> entries = bundle.findEntries(FACTORY_ROOT_PATH, "*.json", true);
		while (entries.hasMoreElements()) {
			URL entry = entries.nextElement();
			String urlString = entry.toString();
			log.debug("Found path: " + urlString);
			if (urlString.endsWith("/"))
				continue;
			String id = urlString.substring(urlString.lastIndexOf("/") + 1);
			id = id.substring(0, id.lastIndexOf("."));
			log.debug("Found id: " + id);
			
			try {
/*				String config = new BufferedReader (
						  new InputStreamReader(
								  entry.openStream(), StandardCharsets.UTF_8))
							.lines()
							.collect(Collectors.joining("\n"));
*/							
				String config = JSONUtils.getJsonRootObjectString(entry);
				if (factories.containsKey(id)) {
					factories.get(id).setConfigString(config);
				} else {
					TrackedFactory tf = new TrackedFactory();
					tf.setId(id);
					tf.setBundle(bundle);
					tf.setConfigString(config);
					factories.put(id, tf);
				}
			} catch (Exception e) {
				log.error("Error reading url: " + urlString, e);
			}
		}
		
	}
	
	public List<TrackedFactory> getFactoryList(){
		return new ArrayList<>(this.factories.values());
	}
}
