/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.factory.tracker;

import java.util.Dictionary;
import java.util.Hashtable;
import java.util.List;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.base.objects.factory.impl.DefaultFactory;
import ru.entaxy.platform.base.support.osgi.tracker.BundleTrackerCustomizerListener;

public class TrackedFactoryCustomizerListener implements BundleTrackerCustomizerListener<List<TrackedFactory>> {
	
	private static final Logger log = LoggerFactory.getLogger(TrackedFactoryCustomizerListener.class);
	
	protected BundleContext bundleContext;
	
	public TrackedFactoryCustomizerListener(BundleContext bundleContext) {
		this.bundleContext = bundleContext;
	}

	@Override
	public void added(List<TrackedFactory> managedObject) {
		if (managedObject == null) {
			log.debug("managedObject is null");
			return;
		}
		for (TrackedFactory tf: managedObject) {
			log.debug("Added factory: " + tf.getId());
			if (TrackedFactory.trackedFactoriesMap.containsKey(tf.getId())) {
				TrackedFactory.trackedFactoriesMap.get(tf.getId()).unregister();
			}
			DefaultFactory defaultFactory = new DefaultFactory();
			defaultFactory.setFactoryId(tf.getId());
			defaultFactory.configure(tf.getConfigString());
			if (defaultFactory.isValid()) {

				tf.setId(defaultFactory.getFactoryId());
				
				Dictionary<String, String> props = new Hashtable<String, String>();
				props.put(EntaxyFactory.SERVICE.PROP_ID, defaultFactory.getFactoryId());
				props.put(EntaxyFactory.SERVICE.PROP_TYPE, defaultFactory.getFactoryType());
				props.put(EntaxyFactory.SERVICE.PROP_ORIGIN_BUNDLE, tf.getBundle().getBundleId()+"");				
				
				tf.setServiceRegistration(
						this.bundleContext.registerService(EntaxyFactory.class, defaultFactory, props)
				);
				
				TrackedFactory.trackedFactoriesMap.put(tf.getId(), tf);
			}
		}
	}

	@Override
	public void modified(List<TrackedFactory> managedObject) {
	}

	@Override
	public void removed(List<TrackedFactory> managedObject) {
		if (managedObject == null)
			return;
		for (TrackedFactory tf: managedObject) {
			try {
				tf.getServiceRegistration().unregister();
			} catch (Exception e) {
				// do nothing
			}
			TrackedFactory.trackedFactoriesMap.remove(tf.getId());
		}
		
	}

	
	
}
