/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.shell;

import java.util.List;
import java.util.stream.Collectors;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Argument;
import org.apache.karaf.shell.api.action.Completion;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.shell.completers.ObjectCompleter;

public abstract class ObjectAwareCommand extends EntaxyObjectServiceSupport implements Action {
	
	@Argument (index = 0, name = "objectFullId", required = true)
	@Completion(caseSensitive = false, value = ObjectCompleter.class)
	String objectFullId;

	EntaxyObject entaxyObject;
	
	@Override
	public Object execute() throws Exception {
		
		List<EntaxyObject> candidates = objectService.getObjects()
				.stream().filter(obj->obj.getObjectFullId().equals(objectFullId))
				.collect(Collectors.toList());
		if (candidates.size()>0) {
			entaxyObject = candidates.get(0);
			return doExecute();
		} else {
			throw new IllegalArgumentException("Object not found by full Id: [" + objectFullId + "]");
		}
	}

	public abstract Object doExecute() throws Exception;
	
}
