/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.storage.impl;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.base.objects.EntaxyObjectStorage;
import ru.entaxy.platform.base.objects.EntaxyObjectStorageService;

@Component(service = EntaxyObjectStorageService.class, immediate = true)
public class EntaxyObjectStorageServiceImpl implements EntaxyObjectStorageService {

	protected Map<String, EntaxyObjectStorage> objectStorages = new HashMap<>();
	
	@Override
	public EntaxyObjectStorage getObjectStorage(String objectType) {
		return this.objectStorages.get(objectType);
	}

	@Reference(bind = "addObjectStorage", unbind = "removeObjectStorage", cardinality = ReferenceCardinality.MULTIPLE
			, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY, collectionType = CollectionType.SERVICE)
	public void addObjectStorage(EntaxyObjectStorage objectStorage) {
		String[] types = objectStorage.getObjectTypes();
		if (types == null)
			return;
		for (int i=0; i<types.length; i++)
			this.objectStorages.put(types[i], objectStorage);
	}
	
	public void removeObjectStorage(EntaxyObjectStorage objectStorage) {
		String[] types = objectStorage.getObjectTypes();
		if (types == null)
			return;
		for (int i=0; i<types.length; i++)
			this.objectStorages.remove(types[i]);
	}
	
}
