/*-
 * ~~~~~~licensing~~~~~~
 * platform-manager-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.manager.impl;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Collection;
import java.util.List;
import java.util.function.Function;

import org.apache.felix.hc.api.Result;
import org.apache.felix.hc.api.Result.Status;
import org.apache.felix.hc.api.execution.HealthCheckExecutionOptions;
import org.apache.felix.hc.api.execution.HealthCheckExecutionResult;
import org.apache.felix.hc.api.execution.HealthCheckExecutor;
import org.apache.felix.hc.api.execution.HealthCheckSelector;
import org.apache.karaf.features.BootFinished;
import org.apache.karaf.shell.api.console.Session;
import org.apache.karaf.shell.api.console.SessionFactory;
import org.apache.karaf.shell.support.ansi.SimpleAnsi;
import org.apache.karaf.shell.support.table.ShellTable;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.manager.EntaxyPlatformManager;
import ru.entaxy.platform.manager.shell.Utils;

@Component(service = EntaxyPlatformManager.class, immediate = true)
public class EntaxyPlatformManagerImpl implements EntaxyPlatformManager {

	private static final Logger log = LoggerFactory.getLogger(EntaxyPlatformManagerImpl.class);
	
	@Reference(cardinality = ReferenceCardinality.MANDATORY, policyOption = ReferencePolicyOption.GREEDY)
	BootFinished bootFinished;
	
	@Reference(cardinality = ReferenceCardinality.MANDATORY, policyOption = ReferencePolicyOption.GREEDY)
	HealthCheckExecutor healthCheckExecutor;

	@Reference(cardinality = ReferenceCardinality.MANDATORY, policyOption = ReferencePolicyOption.GREEDY)
	SessionFactory sessionFactory;
	
	
	@Activate
	public void activate(ComponentContext componentContext) {
		Thread thread = new Thread(new Runnable() {
			
			@Override
			public void run() {
				
				doOutput();
			}
		});
		thread.setContextClassLoader(Thread.currentThread().getContextClassLoader()); 
		thread.run();
		// doOutput();
	}
	
	public void doOutput() {
		System.out.println("\n\t******************");
		System.out.println("\t* " + Utils.WLC + SimpleAnsi.INTENSITY_BOLD + "Entaxy started " + Utils.NOR + SimpleAnsi.INTENSITY_NORMAL + "*");
		System.out.println("\t******************\n\n");

/*		
	
		try (	final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
				final PrintStream printStream = new PrintStream(byteArrayOutputStream);
				) {
			Session session = sessionFactory.create(System.in, printStream, System.err);
			session.execute("bundle:list");
			System.out.println(byteArrayOutputStream.toString());
			System.out.println("================");
		} catch (Exception e) {
			e.printStackTrace();
			log.error("ERROR:", e);
		}
  		HealthCheckExecutionOptions options = new HealthCheckExecutionOptions();
        options.setCombineTagsWithOr(true);
        options.setForceInstantExecution(true);
*/        
		
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
			// NOOP;
		}
		
		HealthCheckSelector selector = HealthCheckSelector.tags(new String[] {""}); 
        
        Collection<HealthCheckExecutionResult> results = healthCheckExecutor.execute(selector);//, options);
        
        printHealthCheckResults(results);
		
	}
	
    protected void printHealthCheckResults(Collection<HealthCheckExecutionResult> results) {
    	
    	System.out.println("Overall HealthCheck status: " + Utils.colored(Utils.getOverallStatus(results)));
		try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream(); PrintStream printStream = new PrintStream(outputStream)) {

			ShellTable table = Utils.createTable(results);
			
			table.print(printStream, StandardCharsets.ISO_8859_1, true);
			System.out.println(outputStream.toString());
			
			System.out.println("\n\t ***" + SimpleAnsi.INTENSITY_BOLD + "NOTICE" + SimpleAnsi.INTENSITY_NORMAL + "***");
			System.out.println("Some bundles may start slowly");
			System.out.println("\tUse " + SimpleAnsi.COLOR_BLUE + SimpleAnsi.INTENSITY_BOLD
					+ "entaxy:platform-health" + SimpleAnsi.RESET + SimpleAnsi.INTENSITY_NORMAL
					+ " command to check platform health");
			System.out.println("\tUse " + SimpleAnsi.COLOR_BLUE + SimpleAnsi.INTENSITY_BOLD
					+ "entaxy:inactive" + SimpleAnsi.RESET + SimpleAnsi.INTENSITY_NORMAL
					+ " command to view inactive bundles");
			System.out.println();
	    } catch (IOException e) {
			// NOOP
		}

    	
    }
    
}
