/*-
 * ~~~~~~licensing~~~~~~
 * resources-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.management.impl;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.utils.ManagedObjectDescriptor;
import ru.entaxy.esb.resources.EntaxyResourceProvider;
import ru.entaxy.esb.resources.management.EntaxyResourceProviderMBean;
import ru.entaxy.esb.resources.management.EntaxyResourceServiceMBean;

@Component(service = EntaxyResourceProviderTracker.class, immediate = true)
public class EntaxyResourceProviderTracker {

	private static final Logger log = LoggerFactory.getLogger(EntaxyResourceProviderTracker.class);
	
	protected Map<String, ManagedObjectDescriptor<EntaxyResourceProvider, EntaxyResourceProviderMBean, EntaxyResourceProviderMBean>>
			resourceProviders = new HashMap<>();
	
	protected BundleContext bundleContext;
	
	protected Object lock = new Object();
	
	@Activate
	public EntaxyResourceProviderTracker(BundleContext context) {
		bundleContext = context;
	}
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, policyOption = ReferencePolicyOption.GREEDY
			, unbind = "removeProvider", policy = ReferencePolicy.DYNAMIC)
	public void addProvider(EntaxyResourceProvider provider) {
		String protocol = provider.getProtocol();
		if (resourceProviders.containsKey(protocol)) {
			removeProvider(resourceProviders.get(protocol).object);
		}
		synchronized (lock) {
			try {
				ManagedObjectDescriptor<EntaxyResourceProvider, EntaxyResourceProviderMBean, EntaxyResourceProviderMBean> descriptor = 
						new ManagedObjectDescriptor<>();
				descriptor.object = provider;
				descriptor.service = new EntaxyResourceProviderMBeanImpl(provider);
				Hashtable props = new Hashtable<>();
				props.put(ManagementCore.JMX_OBJECTNAME, 
						ManagementCore.Q_RUNTIME_S + "," + 
						EntaxyResourceServiceMBean.RESOURCE_SERVICE_KEY 
						+ "=" + EntaxyResourceServiceMBean.RESOURCE_SERVICE_KEY_VALUE
						+ "," + 
						"provider="+protocol);
				ServiceRegistration<EntaxyResourceProviderMBean> sr = bundleContext.registerService(EntaxyResourceProviderMBean.class
						, descriptor.service, props);
				descriptor.registration = sr;
				this.resourceProviders.put(protocol, descriptor);
				
			} catch (Exception e) {
				log.error("Error creating mbean for provider [" + protocol + "]", e);
			}
		}
	}
	
	public void removeProvider(EntaxyResourceProvider provider) {
		synchronized (lock) {
			try {
				ManagedObjectDescriptor<EntaxyResourceProvider, EntaxyResourceProviderMBean, EntaxyResourceProviderMBean> descriptor = 
						resourceProviders.get(provider.getProtocol());
				resourceProviders.remove(provider.getProtocol());
				if (descriptor!=null)
					descriptor.registration.unregister();
			} catch (Exception e) {
				log.warn("Error removing mbean for provider [" + provider.getProtocol() + "]", e);
			}
		}
	}
	
}
