/*-
 * ~~~~~~licensing~~~~~~
 * resources-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.management.impl;

import org.osgi.framework.BundleContext;
import org.osgi.service.component.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.esb.resources.EntaxyResourceMetadata;
import ru.entaxy.esb.resources.EntaxyResourceService;
import ru.entaxy.esb.resources.management.EntaxyResourceServiceMBean;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;
import java.util.HashMap;
import java.util.Map;

@Component(service = { EntaxyResourceServiceMBean.class, DynamicMBean.class, MBeanRegistration.class }, property = {
        ManagementCore.JMX_OBJECTNAME + "=" + ManagementCore.Q_RUNTIME_S + "," + EntaxyResourceServiceMBean.RESOURCE_SERVICE_KEY + "="
                + EntaxyResourceServiceMBean.RESOURCE_SERVICE_KEY_VALUE }, scope = ServiceScope.SINGLETON, immediate = true)
public class EntaxyResourceServiceMBeanImpl extends AnnotatedMBean<EntaxyResourceServiceMBean>
        implements EntaxyResourceServiceMBean {

    private static final Logger log = LoggerFactory.getLogger(EntaxyResourceServiceMBeanImpl.class);

    protected BundleContext bundleContext;

    @Reference(cardinality = ReferenceCardinality.MANDATORY
            , service = EntaxyResourceService.class
            , collectionType = CollectionType.SERVICE)
    EntaxyResourceService entaxyResourceService;

    public EntaxyResourceServiceMBeanImpl() throws NotCompliantMBeanException {
        super(EntaxyResourceServiceMBean.class);
    }

    @Override
    public String getResource(String location) throws Exception {
        return entaxyResourceService.getResource(location).getAsString();
    }

    @Override
    public Map<String, String> getResourceMetadata(String location) throws Exception {
        EntaxyResource resource = entaxyResourceService.getResource(location);

        if (resource.hasMetadata()) {
            Map<String, String> metadata = new HashMap<>();
            for (EntaxyResourceMetadata.MetadataSection section : resource.getMetadata().getSections()) {
                metadata.put(section.getName(), section.getContent().toString());
            }
            return metadata;
        } else {
            throw new Exception("Metadata is not present");
        }
    }
}
