/*-
 * ~~~~~~licensing~~~~~~
 * resources-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.impl;

import ru.entaxy.esb.resources.EntaxyResourceMetadata;
import ru.entaxy.esb.resources.EntaxyResource;

public abstract class AbstractResource<T extends EntaxyResourceMetadata> implements EntaxyResource {

	protected boolean metadataInited = false;
	
	protected T metadata = null;
	
	@Override
	public boolean hasMetadata() {
		if (!metadataInited)
			initMetadata();
		return this.metadata != null;
	}
	
	protected void initMetadata() {
		doInitMetadata();
		metadataInited = true;
	};
	
	abstract protected void doInitMetadata();
	
	abstract protected T createMetadata();
	
	@Override
	public EntaxyResourceMetadata getMetadata() {
		return getMetadata(false);
	}
	
	
	@Override
	public EntaxyResourceMetadata getMetadata(boolean create) {
		if (!metadataInited)
			initMetadata();
		if (metadata != null)
			return metadata;
		if (!create)
			return null;
		return createMetadata();
	}
	
	@Override
	public final String getTimestamp() {
		EntaxyResourceMetadata.MetadataSection resourceSection = getResourceSection();
		if (resourceSection == null)
			return DEFAULT_TIMESTAMP;
		try {
			return resourceSection.getContent().get(EntaxyResourceMetadata.SECTION_RESOURCE.ATTRIBUTES.TIMESTAMP).getAsString();
		} catch (Exception e) {
			// NOT FOUND
		}
		return DEFAULT_TIMESTAMP;
	}

	@Override
	public final RESOURCE_SCOPE getScope() {
		EntaxyResourceMetadata.MetadataSection resourceSection = getResourceSection();
		if (resourceSection == null)
			return DEFAULT_SCOPE;
		try {
			String value = resourceSection.getContent().get(EntaxyResourceMetadata.SECTION_RESOURCE.ATTRIBUTES.TIMESTAMP).getAsString();
			return RESOURCE_SCOPE.valueOf(value.toUpperCase());
		} catch (Exception e) {
			// NOT FOUND OR INCORRECT
		}
		return DEFAULT_SCOPE;
	}
	
	protected EntaxyResourceMetadata.MetadataSection getResourceSection(){
		if (!hasMetadata())
			return null;
		return getMetadata().getSection(EntaxyResourceMetadata.SECTION_RESOURCE.NAME);
	}
}
