/*-
 * ~~~~~~licensing~~~~~~
 * resources-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.impl;

import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.esb.resources.EntaxyResourceMetadata;
import ru.entaxy.esb.resources.EntaxyResourceProcessor;

public class ResourceWrapper implements EntaxyResource {

	protected EntaxyResource origin;
	protected List<EntaxyResourceProcessor> processors;
	protected String originProtocol;
	protected String originLocation;
	
	public ResourceWrapper(EntaxyResource originResource
			, String originProtocol
			, String originLocation
			, List<EntaxyResourceProcessor> resourceProcessors) {
		this.origin = originResource;
		this.originProtocol = originProtocol;
		this.originLocation = originLocation;
		this.processors = resourceProcessors==null
				?Collections.emptyList()
				:new ArrayList<>(resourceProcessors);
	}
	
	@Override
	public InputStream getInputStream() {
		return origin.getInputStream();
	}

	@Override
	public String getAsString() {
		return origin.getAsString();
	}

	@Override
	public void save(InputStream inputStream) {
		InputStream current = inputStream;
		for (EntaxyResourceProcessor processor: processors) {
			InputStream prev = current;
			current = processor.preProcess(current, this);
			if (current == null)
				current = prev;
		}
		origin.save(current);
		for (EntaxyResourceProcessor processor: processors)
			processor.postProcess(this);
	}

	@Override
	public boolean exists() {
		return origin.exists();
	}

	@Override
	public boolean isFolder() {
		return origin.isFolder();
	}

	@Override
	public String getName() {
		return origin.getName();
	}

	@Override
	public long getSize() {
		return origin.getSize();
	}

	@Override
	public URL getLocation() {
		return origin.getLocation();
	}

	@Override
	public boolean hasMetadata() {
		return origin.hasMetadata();
	}

	@Override
	public EntaxyResourceMetadata getMetadata() {
		return origin.getMetadata();
	}

	@Override
	public EntaxyResourceMetadata getMetadata(boolean create) {
		return origin.getMetadata(create);
	}

	@Override
	public String getURL() {
		return this.originProtocol + ":" + originLocation;
	}
	
	@Override
	public String getTimestamp() {
		return origin.getTimestamp();
	}
	
	@Override
	public RESOURCE_SCOPE getScope() {
		return origin.getScope();
	}
}
