/*-
 * ~~~~~~licensing~~~~~~
 * resources-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.shell;

import java.text.DecimalFormat;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Argument;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Reference;
import org.apache.karaf.shell.api.action.lifecycle.Service;
import org.apache.karaf.shell.support.table.ShellTable;

import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.esb.resources.EntaxyResourceService;

@Service
@Command (scope = "resources", name = "list")
public class ResourceList implements Action {

	@Argument(index = 0, required = true)
	String path;

	@Reference(optional = false)
	EntaxyResourceService entaxyResourceService;
	
	@Override
	public Object execute() throws Exception {
		
		List<EntaxyResource> resources = entaxyResourceService.getResources(path);
		
		ShellTable table = new ShellTable();
		table.column("Name");
		table.column("Size");
		table.column("Has metadata");
		table.column("URL");
		
		if (!resources.isEmpty()) {
			
			Collections.sort(resources, new Comparator<EntaxyResource>() {

				@Override
				public int compare(EntaxyResource o1, EntaxyResource o2) {
					if (o1.isFolder() && !o2.isFolder())
						return -1;
					if (!o1.isFolder() && o2.isFolder())
						return 1;
					return o1.getName().compareTo(o2.getName());
				}
				
			});
			
			for (EntaxyResource r: resources) {
				table.addRow().addContent(r.getName()
						, r.isFolder()
							?""
							:(new DecimalFormat("###,###,###,###")).format(r.getSize())
								.replace(",", " ")
						, r.isFolder()
							?""
							:r.hasMetadata()?"true":""
						, r.isFolder()
							?""
							:r.getURL()==null
								?""
								:r.getURL().toString()
						);
			}
			
		}
		
		table.print(System.out);
		return null;
	}
	
	
}
