/*-
 * ~~~~~~licensing~~~~~~
 * schema-impl
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.support;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.url.URLStreamHandlerService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.platform.base.support.CommonUtils;

import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.URIResolver;
import javax.xml.transform.stream.StreamSource;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Path;
import java.nio.file.Paths;

@Component(service = URIResolver.class, property = "type=xslt", immediate = true)
public class XslUrlResolver implements URIResolver {

    private static final Logger log = LoggerFactory.getLogger(XslUrlResolver.class);

    @Reference(cardinality = ReferenceCardinality.MANDATORY, service = URLStreamHandlerService.class,
            target = "(url.handler.protocol=entaxy-resource)")
    protected URLStreamHandlerService entaxyResourcesURLHandler;


    @Override
    public Source resolve(String href, String base) throws TransformerException {
        if (CommonUtils.isValid(href) && !CommonUtils.isValid(base)) {
            return resolve(href);
        } else if (CommonUtils.isValid(href) && CommonUtils.isValid(base)) {
            Path hrefPath = Paths.get(href);
            Path resolveBasePath = hrefPath.resolve(Paths.get(base));
            return resolve(resolveBasePath.toString());
        }
        return null;
    }

    protected Source resolve(String href) throws TransformerException {
        try {
            URL u = new URL(href);
            return new StreamSource(entaxyResourcesURLHandler.openConnection(u).getInputStream());
        } catch (IOException e) {
            log.error("Can't find resource [" + href + "]", e);
        }
        return null;
    }
}
