/*-
 * ~~~~~~licensing~~~~~~
 * resources-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.tracker;

import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.support.CommonUtils;

public class ResourceProvider {

	private static final Logger log = LoggerFactory.getLogger(ResourceProvider.class);
	
	public static final String RESOURCE_PATH = "/ru/entaxy/resource";
	
	protected Bundle bundle;
	
	protected Map<String, List<ResourceDescriptor>> resources = new HashMap<>();
	
	public ResourceProvider(Bundle bundle) {
		this.bundle = bundle;
	}
	
	public Map<String, List<ResourceDescriptor>> getResources(){
		if (!resources.isEmpty())
			return resources;
		
		Enumeration<URL> foundEntries = bundle.findEntries(ResourceProvider.RESOURCE_PATH, "*.*", true);
		while (foundEntries.hasMoreElements()) {
			URL entry = foundEntries.nextElement();
			log.debug("FOUND :: " + entry.toString());
			if (entry.toString().endsWith("/")) {
				log.debug(":: .. is folder");
				continue;
			}
			String localPath = entry.toString();
			localPath = localPath.substring(localPath.indexOf(ResourceProvider.RESOURCE_PATH) + ResourceProvider.RESOURCE_PATH.length() + 1);
			
			String resourceName = localPath.substring(localPath.lastIndexOf("/")+1);
			
			String path = localPath.lastIndexOf("/")>=0
					?localPath.substring(0, localPath.lastIndexOf("/"))
					:"";
			int splitIndex = path.indexOf("/");
			String protocol = splitIndex>0
					?path.substring(0, splitIndex)
					:"";
			
			if (splitIndex>0)
				path = path.substring(splitIndex+1);
			
			if (!CommonUtils.isValid(protocol) || !CommonUtils.isValid(resourceName))
				continue;
			
			if (!resources.containsKey(protocol))
				resources.put(protocol, new ArrayList<>());
			
			resources.get(protocol).add(new ResourceDescriptor(entry, path + "/" + resourceName));
			
		}
		
		return resources;
		
	}
	
	public static class ResourceDescriptor {
		
		public URL url;
		public String location;
		
		public ResourceDescriptor(URL url, String location) {
			this.url = url;
			this.location = location;
		}
		
	}
	
}
