/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact;

import java.io.File;
import java.util.Map;

import ru.entaxy.platform.base.support.osgi.bundle.CapabilityDescriptor;

public interface Artifact {

	public static final String ARTIFACT_CATEGORY_UNKNOWN = "unknown";
	
	public static final String ARTIFACT_CATEGORY_BLUEPRINT = "blueprint";
	public static final String ARTIFACT_CATEGORY_BUNDLE = "bundle";
	public static final String ARTIFACT_CATEGORY_JAR = "jar";
	
	public static final String ARTIFACT_CATEGORY_CONFIG = "config";
	public static final String ARTIFACT_CATEGORY_FEATURES = "features";
	
	public static final String DEFAULT_RUNTIME_GROUP_ID = "entaxy.runtime";
	
	public String getCategory();
	
	public ArtifactCoordinates getCoordinates();

	public Object getContent();
	
	// TODO add 'throws' declaration if needed  
	public void setContent(Object content);
	
	public void toFile(File destination);
	
	public byte[] asByteArray();
	
	public Map<String, Object> getProperties();
	
	// capabilities support
	
	@Deprecated(forRemoval = true, since = "1.8.2")
	public CapabilityDescriptor provideCapability(String namespace);
	
	@Deprecated(forRemoval = true, since = "1.8.2")
	public CapabilityDescriptor requireCapability(String namespace);
	
	public default boolean isManifested() {
		return Manifested.class.isAssignableFrom(this.getClass());
	}
}
