/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.core.artifact.annotation.ArtifactSupport;

public class Artifacts {

	private static final Logger log = LoggerFactory.getLogger(Artifacts.class);
	
	protected static final Map<String, Class<?>> supportClasses = new HashMap<>();

	static {
		Artifacts.registerSupport(Blueprint.class);
	}
	
	
	public static final void registerSupport(Class<?> supportClass) {
		if (supportClass.isAnnotationPresent(ArtifactSupport.class)) {
			String artifactType = supportClass.getAnnotation(ArtifactSupport.class).supportedCategory();
			Artifacts.supportClasses.put(artifactType, supportClass);
		}
	}
	
	public static final DefaultArtifact create(String category) {
		if (supportClasses.containsKey(category)) {
			Class<?> supportClass = supportClasses.get(category);
			try {
				Constructor<?> c = supportClass.getConstructor();
				Object instance = c.newInstance();
				if (DefaultArtifact.class.isAssignableFrom(instance.getClass()))
					return (DefaultArtifact)instance;
			} catch (NoSuchMethodException e) {
				log.error("Error creating artifact", e);
			} catch (SecurityException e) {
				log.error("Error creating artifact", e);
			} catch (InstantiationException e) {
				log.error("Error creating artifact", e);
			} catch (IllegalAccessException e) {
				log.error("Error creating artifact", e);
			} catch (IllegalArgumentException e) {
				log.error("Error creating artifact", e);
			} catch (InvocationTargetException e) {
				log.error("Error creating artifact", e);
			}
		}
		return new DefaultArtifact(); 
	}
	
	public static final DefaultArtifact fromGenerated(Generated generated) {
		DefaultArtifact result = Artifacts.create(generated.getType());
		if (result != null)
			result.setContent(generated.getObject());
		return result;
	}
	
}
