/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.capabilities;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.base.support.osgi.bundle.CapabilityDescriptor;
import ru.entaxy.platform.base.support.osgi.bundle.CapabilityDescriptorImpl;
import ru.entaxy.platform.base.support.osgi.bundle.CapabilityHelper;
import ru.entaxy.platform.core.artifact.ArtifactManifest;

public class ManifestCapabilityHelper extends CapabilityHelper {

	protected ArtifactManifest manifest;

	
	public ManifestCapabilityHelper(ArtifactManifest manifest) {
		super();
		this.manifest = manifest;
		this.load();
	}

	@Override
	protected void load() {
		String existing = this.manifest.getCustomAttributes().get(CapabilityDescriptor.HEADER_PROVIDE_CAPABILITY);
		if (CommonUtils.isValid(existing)) {
			List<CapabilityDescriptorImpl> list = parse(existing);
			for (CapabilityDescriptorImpl c: list)
				addProvidedCapability(c);
		}
		existing = this.manifest.getCustomAttributes().get(CapabilityDescriptor.HEADER_REQUIRE_CAPABILITY);
		if (CommonUtils.isValid(existing)) {
			List<CapabilityDescriptorImpl> list = parse(existing);
			for (CapabilityDescriptorImpl c: list)
				addRequiredCapability(c);
		}
	}
	
	public void save() {
		this.manifest.getCustomAttributes().put(CapabilityDescriptor.HEADER_PROVIDE_CAPABILITY, 
			this.providedCapabilities.values().stream()
				.flatMap(List::stream)
				.map(
					cap -> (
						cap.getNamespace()
						+ ";"
						+ cap.getAttributesAsString()
					)
				)
				.collect(Collectors.joining(","))
				);
	}
	
}
