/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.installer.builder.impl;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import ru.entaxy.platform.core.artifact.installer.builder.InstallationResult;

public class InstallationResultImpl implements InstallationResult {

	public static InstallationResultImpl create() {
		return new InstallationResultImpl();
	}
	
	protected Result result = null;
	
	protected final Map<String, Object> properties = new HashMap<>(); 
	
	protected Object object = null;
	
	protected final List<InstallationResult> subResults = new LinkedList<>();
	
	protected Throwable error = null;
	
	protected String message = null;
	
	@Override
	public Result getResult() {
		return this.result;
	}

	@Override
	public Map<String, Object> getProperties() {
		return this.properties;
	}

	@Override
	public Object getObject() {
		return this.object;
	}

	@Override
	public List<InstallationResult> getSubResults() {
		return this.subResults;
	}

	@Override
	public Throwable getError() {
		return this.error;
	}

	@Override
	public String getMessage() {
		return this.message;
	}
	
	public InstallationResultImpl result(Result result) {
		this.result = result;
		return this;
	}
	
	public InstallationResultImpl object(Object object) {
		this.object = object;
		return this;
	}
	
	public InstallationResultImpl property(String key, Object value) {
		this.properties.put(key, value);
		return this;
	}
	
	public InstallationResultImpl error(Throwable error) {
		this.error = error;
		return this;
	}
	
	public InstallationResultImpl subResult(InstallationResult subresult) {
		this.subResults.add(subresult);
		return this;
	}
	
	public InstallationResultImpl message(String message) {
		this.message = message;
		return this;
	}
	
	public InstallationResultImpl notChanged() {
		return this.result(Result.NOT_CHANGED);
	}
	
	public InstallationResultImpl installed() {
		return this.result(Result.INSTALLED);
	}

	public InstallationResultImpl installed(Object object) {
		object(object);
		return this.result(Result.INSTALLED);
	}

	public InstallationResultImpl uninstalled() {
		return this.result(Result.UNINSTALLED);
	}

	public InstallationResultImpl uninstalled(Object object) {
		object(object);
		return this.result(Result.UNINSTALLED);
	}

	public InstallationResultImpl updated() {
		return this.result(Result.UPDATED);
	}

	public InstallationResultImpl updated(Object object) {
		object(object);
		return this.result(Result.UPDATED);
	}
	
	public InstallationResultImpl started() {
		return this.result(Result.STARTED);
	}

	public InstallationResultImpl started(Object object) {
		object(object);
		return this.result(Result.STARTED);
	}
	
	public InstallationResultImpl stopped() {
		return this.result(Result.STOPPED );
	}

	public InstallationResultImpl stopped(Object object) {
		object(object);
		return this.result(Result.STOPPED);
	}
	
	public InstallationResultImpl failed() {
		return this.result(Result.FAILED);
	}

	public InstallationResultImpl failed(String message) {
		message(message);
		return this.result(Result.FAILED);
	}
	
	
}
