/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.installer.builder.impl;

import java.util.HashMap;
import java.util.Map;

import ru.entaxy.platform.core.artifact.installer.ArtifactInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.ClusterInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.Installer;
import ru.entaxy.platform.core.artifact.installer.builder.Installers;
import ru.entaxy.platform.core.artifact.installer.builder.InstallersWithDefaults;
import ru.entaxy.platform.core.artifact.installer.builder.LocalInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.impl.cluster.ClusterInstallerWrapperImpl;

public class InstallersWithDefaultsImpl implements InstallersWithDefaults {

	protected InstallersImpl defaults = null;
	
	protected Map<String, ArtifactInstaller<?>> installers = new HashMap<>();
	
	protected ArtifactInstaller<LocalInstaller> local = null;
	
	protected ArtifactInstaller<ClusterInstaller> cluster = null;
	
	protected boolean localOnlyMode = false;
	
	@Override
	public LocalInstaller local() {
		if (this.local == null)
			if (defaults != null)
				return defaults.local();
		return this.local.createInstaller();
	}

	@Override
	public ClusterInstaller cluster(String... groupName) {
		if (this.localOnlyMode)
			return new ClusterInstallerWrapperImpl(this.local());
		if (this.cluster == null)
			if (defaults != null)
				return defaults.cluster(groupName);
		return this.cluster.createInstaller().groups(groupName);
	}

	@Override
	public Installer<?> byName(String name) {
		if (ArtifactInstaller.ARTIFACT_INSTALLER_LOCAL.equals(name))
			return local();
		if (ArtifactInstaller.ARTIFACT_INSTALLER_CLUSTER.equals(name))
			return cluster();
		if (!this.installers.containsKey(name))
			return null;
		return this.installers.get(name).createInstaller();
	}

	@SuppressWarnings("unchecked")
	public void addInstaller(ArtifactInstaller<?> installer) {
		installers.put(installer.getName(), installer);
		if (ArtifactInstaller.ARTIFACT_INSTALLER_LOCAL.equalsIgnoreCase(installer.getName()))
			this.local = (ArtifactInstaller<LocalInstaller>)installer; 
		if (ArtifactInstaller.ARTIFACT_INSTALLER_CLUSTER.equalsIgnoreCase(installer.getName()))
			this.cluster = (ArtifactInstaller<ClusterInstaller>)installer; 
	}
	
	@Override
	public Installers useDefaults() {
		if (defaults == null)
			return this;
		return defaults;
	}

	public void setDefaults(InstallersImpl defaults) {
		this.defaults = defaults;
	}

	public boolean isLocalOnlyMode() {
		return localOnlyMode;
	}

	public void setLocalOnlyMode(boolean localOnlyMode) {
		this.localOnlyMode = localOnlyMode;
	}
}

