/*-
 * ~~~~~~licensing~~~~~~
 * core-support-runtime
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.support.runtime;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.osgi.framework.BundleContext;
import org.osgi.framework.wiring.BundleWiring;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class EntaxyResourceURLClassLoader extends URLClassLoader {
	
	private static final Logger log = LoggerFactory.getLogger(EntaxyResourceURLClassLoader.class);
	
	static final String ENTAXY_RESOURCE_PROTOCOL = "entaxy-resource";
	
	protected List<String> resourcePath = new ArrayList<>();
	
	public EntaxyResourceURLClassLoader() {
		super(new URL[] {});
	}

	public EntaxyResourceURLClassLoader(ClassLoader parent) {
		super(new URL[] {}, parent);
	}

	public EntaxyResourceURLClassLoader(BundleContext bundleContext) {
		super(new URL[] {}, bundleContext.getBundle().adapt(BundleWiring.class).getClassLoader());
	}

	protected URL processPath(String path) throws MalformedURLException {
		String pathToAnalyze = path;
		if (path.startsWith(ENTAXY_RESOURCE_PROTOCOL)) {
			pathToAnalyze = pathToAnalyze.substring(ENTAXY_RESOURCE_PROTOCOL.length() + 1);
		}
		pathToAnalyze = StringUtils.stripStart(pathToAnalyze, "/:");
		pathToAnalyze = pathToAnalyze.replace(':', '/');
		return new URL(ENTAXY_RESOURCE_PROTOCOL + "://" + pathToAnalyze);
	} 
	
	public List<String> getResourcePath() {
		return resourcePath;
	}

	public void setResourcePath(List<String> resourcePath) {
		this.resourcePath = resourcePath;
		
		for (String s: resourcePath) {
			try {
				addURL(processPath(s));
			} catch (MalformedURLException e) {
				log.error("MalformedURLException for [" + s + "]" , e);
			}
		}

	}

	@Override
	public Class<?> loadClass(String name) throws ClassNotFoundException {
		Class<?> result = super.loadClass(name); 
		return result;
	}
	
	@Override
	public URL getResource(String name) {
		return super.getResource(name);
	}
	
	@Override
	public InputStream getResourceAsStream(String name) {
		return super.getResourceAsStream(name);
	}
	
	@Override
	public Enumeration<URL> getResources(String name) throws IOException {
		return super.getResources(name);
	}
	@Override
	public URL findResource(String name) {
		return super.findResource(name);
	}
	@Override
	public Enumeration<URL> findResources(String name) throws IOException {
		return super.findResources(name);
	}
	@Override
	public URL[] getURLs() {
		return super.getURLs();
	}
}
