/*-
 * ~~~~~~licensing~~~~~~
 * core-support-runtime
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.support.runtime.cxf;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.support.CommonUtils;

public abstract class EntaxyCxfDelegatingClassUnwrapper extends EntaxyCxfClassUnwrapper {

	private static final Logger log = LoggerFactory.getLogger(EntaxyCxfDelegatingClassUnwrapper.class);
	
	protected ClassLoader delegate;
	protected String wrappedClass;
	
	public void activate() {
		super.activate();
		if (bus!=null) {
			if (delegate == null)
				delegate = createClassLoader();
			if (delegate != null)
				bus.setExtension(delegate, ClassLoader.class);
		}
	}
	
	protected abstract ClassLoader createClassLoader();
	
	@Override
	protected Class<?> doGetRealClassFromClassName(String className) {
		if (!CommonUtils.isValid(className))
			if (CommonUtils.isValid(wrappedClass))
				return doGetRealClassFromClassName(wrappedClass);
			else
				return null;

		List<ClassLoader> loaders = getClassLoaders();
		
		for (ClassLoader cl: loaders) {
			if (cl==null)
				continue;
			try {
				return cl.loadClass(className);
			} catch (Exception e) {
				continue;
			}
		}
		log.warn("Class not loaded: [%]", className);
		return null;
		
	}
	
	@Override
	protected Class<?> doGetRealClassFromClass(Class<?> clazz) {
		return doGetRealClassFromClassName(wrappedClass);
	}

	protected List<ClassLoader> getClassLoaders() {
		List<ClassLoader> result = new ArrayList<>();
		if (delegate != null)
			result.add(delegate);
		result.add(this.getClass().getClassLoader());
		result.add(Thread.currentThread().getContextClassLoader());
		return result;
	}
	
	public String getWrappedClass() {
		return wrappedClass;
	}

	public void setWrappedClass(String wrappedClass) {
		this.wrappedClass = wrappedClass;
	}
}
