/*-
 * ~~~~~~licensing~~~~~~
 * schema-impl
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.core.infrastructure.schema.impl;

import java.util.ArrayList;
import java.util.List;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;

import ru.entaxy.platform.core.producer.api.EntaxyProducerService;

@Component (service = ServiceHelper.class, immediate = true)
public class ServiceHelper {

	public static interface DelayedExecution {
		void execute();
	}

	protected static ServiceHelper INSTANCE = null; 
	
	protected static Object syncLock = new Object();
	
	protected static List<DelayedExecution> executions = new ArrayList<>();
	
	public static ServiceHelper getInstance() {
		return INSTANCE;
	}

	@Reference (cardinality = ReferenceCardinality.MANDATORY)
	EntaxyProducerService entaxyProducerService;

	@Activate
	public void activate(ComponentContext componentContext) {
		synchronized (syncLock) {
			ServiceHelper.INSTANCE = this;
			for (DelayedExecution exec: executions)
				exec.execute();
			executions.clear();
		}
	}

	public EntaxyProducerService getEntaxyProducerService() {
		return entaxyProducerService;
	}
	
	
	public static void addExecution(DelayedExecution execution) {
		synchronized (syncLock) {
			if (getInstance()==null)
				executions.add(execution);
			else
				execution.execute();
		}
	}
	
}
