/*-
 * ~~~~~~licensing~~~~~~
 * datasources-initializer
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.core.initializer.datasources;

import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.platform.runtime.core.initializer.api.AbstractInitializer;
import ru.entaxy.esb.platform.runtime.core.initializer.api.Initializer;
import ru.entaxy.esb.platform.runtime.core.initializer.api.InitializerException;

import javax.sql.DataSource;
import java.util.Collection;

public class DataSourcesInitializer extends AbstractInitializer {

	protected static final Logger log = LoggerFactory.getLogger(DataSourcesInitializer.class);
	
	protected static Initializer.Callback callback = null;
	
	public static void collectorCompleted() {
		if (DataSourcesInitializer.callback != null)
			DataSourcesInitializer.callback.inited(new DataSourcesInitializer());
	}
	
	@Override
	public void init() throws InitializerException {
		log.info("Init in DataSourcesInitializer is called");
		try {
			Collection<ServiceReference<DataSource>> dataSources = 
					this.bundleContext.getServiceReferences(
					javax.sql.DataSource.class, 
					"(|(osgi.jndi.service.name=entaxy.esb.storage)(osgi.jndi.service.name=entaxy.esb.cache))");
			if (dataSources.size()!=2)
				throw new InitializerException(this, "Datasources initialization error", "Datasources count = " + dataSources.size(), null);
		} catch (InvalidSyntaxException e) {
			e.printStackTrace();
			log.error("Initializer error: " + e.getMessage());
			throw new InitializerException(this, "Initializer exception", "Initializer exception", e);
		}
	}

	@Override
	public void reinit() throws InitializerException {
		log.info("ReInit in CoreInitializer is called");
	}
	
	@Override
	public void setCallback(Callback callback) {
		DataSourcesInitializer.callback = callback;
	}
}
