/*-
 * ~~~~~~licensing~~~~~~
 * liquibase-updater
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.core.initializer.storage.initializer.liquibase.updater;

import liquibase.Contexts;
import liquibase.Liquibase;
import liquibase.database.Database;
import liquibase.database.DatabaseFactory;
import liquibase.database.jvm.JdbcConnection;
import liquibase.osgi.OSGiResourceAccessor;
import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.SQLException;

import javax.sql.DataSource;

public class LiquibaseUpdater {

	private static final Logger log = LoggerFactory.getLogger(LiquibaseUpdater.class);
	
    private static final String CHANGELOG_PATH = "db/changelog.xml";

    public LiquibaseUpdater(DataSource dataSource, BundleContext bundleContext) {
    	java.sql.Connection connection = null;
        try {
            connection = dataSource.getConnection();

            Database database = DatabaseFactory.getInstance().findCorrectDatabaseImplementation(new JdbcConnection(connection));
            String config = bundleContext.getBundle().getEntry(CHANGELOG_PATH).getPath();

            Liquibase liquibase = new liquibase.Liquibase(config,
                    new OSGiResourceAccessor(bundleContext.getBundle()), database);

            liquibase.update(new Contexts());
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        } finally {
			if (connection != null)
				try {
					connection.close();
				} catch (SQLException e) {
					log.error("LiquibaseUpdater", e);
				}
		}
    }
}
