/*-
 * ~~~~~~licensing~~~~~~
 * cluster-node-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.cluster.node.impl;

import java.util.Collections;
import java.util.Map;

import org.apache.karaf.cellar.core.ClusterManager;
import org.apache.karaf.cellar.core.Node;
import org.apache.karaf.cellar.core.control.ConsumerSwitchCommand;
import org.apache.karaf.cellar.core.control.ConsumerSwitchResult;
import org.apache.karaf.cellar.core.control.ProducerSwitchCommand;
import org.apache.karaf.cellar.core.control.ProducerSwitchResult;

import com.hazelcast.core.Member;


import ru.entaxy.esb.platform.core.management.cluster.node.Attributes;
import ru.entaxy.esb.platform.core.management.cluster.node.ClusterUtils;
import ru.entaxy.esb.platform.core.management.node.NodeDataProvider;

public class MemberAwareDataProvider implements NodeDataProvider {

	protected Member member;
	
	protected ServiceHelper serviceHelper;
	
	public MemberAwareDataProvider(Member member, ServiceHelper serviceHelper) {
		super();
		this.member = member;
		this.serviceHelper = serviceHelper;
	}

	@Override
	public String getEntaxyPlatformVersion() {
		return member.getStringAttribute(Attributes.ENTAXY_PLATFORM_VERSION);
	}

	@Override
	public String getHttpPort() {
		return member.getStringAttribute(Attributes.HTTP_PORT);
	}

	@Override
	public String getSshPort() {
		return member.getStringAttribute(Attributes.SSH_PORT);
	}

	@Override
	public boolean isUIEnabled() {
		return member.getBooleanAttribute(Attributes.UI_ENABLED);
	}

	@Override
	public boolean isClusterEnabled() {
		return true;
	}

	@Override
	public String getClusterIP() {
		return member.getAddress().getHost();
	}

	@Override
	public String getClusterPort() {
		return member.getAddress().getPort()+"";
	}
	
	@Override
	public String getNodeId() {
		return ClusterUtils.getCellarNodeId(member);
	}
	
	@Override
	public boolean isConsumerStarted() {
		try {
			Node node = serviceHelper.clusterManager.findNodeById(getNodeId());
			ConsumerSwitchCommand command = new ConsumerSwitchCommand(serviceHelper.clusterManager.generateId());
			command.setDestination(Collections.singleton(node));
			command.setTimeout(1000);
			Map<Node, ConsumerSwitchResult> result = serviceHelper.executionContext.execute(command);
			if (result == null)
				return false;
			for (Node n: result.keySet()) {
				if (n.getId().equals(node.getId())) {
					return result.get(n).getStatus();
				}
			}
			return false;
		} catch (Exception e) {
			// ignore
		}
		return false;
	}
	
	@Override
	public boolean isProducerStarted() {
		try {
			Node node = serviceHelper.clusterManager.findNodeById(getNodeId());
			ProducerSwitchCommand command = new ProducerSwitchCommand(serviceHelper.clusterManager.generateId());
			command.setDestination(Collections.singleton(node));
			command.setTimeout(1000);
			Map<Node, ProducerSwitchResult> result = serviceHelper.executionContext.execute(command);
			if (result == null)
				return false;
			for (Node n: result.keySet()) {
				if (n.getId().equals(node.getId())) {
					return result.get(n).getStatus();
				}
			}
			return false;
		} catch (Exception e) {
			// ignore
		}
		return false;
	}
}
