/*-
 * ~~~~~~licensing~~~~~~
 * cluster-node-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.cluster.node.impl;

import java.util.Iterator;
import java.util.List;

import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.hazelcast.core.Member;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.utils.ManagedObjectsListener;
import ru.entaxy.esb.platform.core.management.node.ClusterNodeMBean;

public class MemberMBeanManager extends ManagedObjectsListener<List<Member>, Member, ClusterNodeMBean, ClusterNodeMBeanImpl> {

	private static final Logger log = LoggerFactory.getLogger(MemberMBeanManager.class);
	
	protected ServiceHelper serviceHelper;
	
	public MemberMBeanManager(BundleContext bundleContext, ServiceHelper serviceHelper) {
		super(bundleContext, ClusterNodeMBean.class);
		this.serviceHelper = serviceHelper;
	}

	@Override
	protected Iterator<Member> getIterator(List<Member> managedObject) {
		return managedObject.iterator();
	}

	@Override
	protected String getKeyValue(Member object) {
		return object.getUuid();
	}

	@Override
	protected ClusterNodeMBeanImpl createService(Member managedObject) {
		try {
			return new ClusterNodeMBeanImpl(managedObject, serviceHelper);
		} catch (NotCompliantMBeanException e) {
			log.error("Error creating [ClusterNodeMBeanImpl]", e);
			return null;
		}
	}

	@Override
	protected String getJmxObjectName(Member managedObject) {
		return ManagementCore.ENTAXY_JMX_DOMAIN + ":" + ManagementCore.REMOTE_NODE_KEY + "=" 
				+ managedObject.getAddress().getHost()+"-"+managedObject.getAddress().getPort();
	}

}
