/*-
 * ~~~~~~licensing~~~~~~
 * connection-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.connection.impl;

import javax.management.NotCompliantMBeanException;
import javax.management.StandardMBean;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

import ru.entaxy.esb.platform.base.management.core.api.EntaxyRuntimeTyped;
import ru.entaxy.esb.platform.core.management.connection.ConnectionMBean;

//@TODO move string to constant
@EntaxyRuntimeTyped(name = "entaxy.runtime.connection")
public class ConnectionMBeanImpl extends StandardMBean implements ConnectionMBean {

	protected ManagedConnection connection;
	
	protected BundleContext bundleContext;
	
	public ConnectionMBeanImpl(BundleContext bundleContext, ManagedConnection connection) throws NotCompliantMBeanException {
		super(ConnectionMBean.class);
		this.bundleContext = bundleContext;
		this.connection = connection;
	}

	@Override
	public String getName() {
		return connection.getName();
	}

	@Override
	public boolean isPlatform() {
		return connection.isPlatform();
	}

	@Override
	public String getStatus() {
		int state = this.bundleContext.getBundle(connection.getBundleId()).getState();
		switch (state) {
		case Bundle.ACTIVE:
			return "Active";
		case Bundle.INSTALLED:
			return "Installed";
		case Bundle.RESOLVED:
			return "Resolved";
		case Bundle.STARTING:
			return "Starting";
		case Bundle.STOPPING:
			return "Stopping";
		default:
			return "Unknown";
		}
	}

	@Override
	public boolean isLocal() {
		return connection.isLocal;
	}

	@Override
	public boolean isShared() {
		return connection.isShared();
	}
	
	@Override
	public boolean isDeployed() {
		return connection.isDeployed;
	}

	@Override
	public String getBundleId() {
		return connection.getBundleId() + "";
	}

	@Override
	public String getLabel() {
		return connection.getLabel();
	}
	
}
