/*-
 * ~~~~~~licensing~~~~~~
 * connection-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.connection.impl;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.Qualifier;
import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.esb.platform.core.management.connection.ConnectionMBean;
import ru.entaxy.esb.platform.core.management.connection.ConnectionsMBean;
import ru.entaxy.esb.platform.core.management.connection.tracker.DeployedConnectionTracker;
import ru.entaxy.platform.base.objects.EntaxyObjectService;

@Component(
		service = {ConnectionsMBean.class, DynamicMBean.class, MBeanRegistration.class},
		property = {ManagementCore.JMX_OBJECTNAME 
				+ "=" + ManagementCore.Q_RUNTIME_S  
				+ "," + ConnectionsMBean.CONNECTIONS_KEY
				+ "=" + ConnectionsMBean.CONNECTIONS_KEY_VALUE},
		scope = ServiceScope.SINGLETON,
		immediate = true		
		)
public class ConnectionsMBeanImpl extends AnnotatedMBean<ConnectionsMBean>
		implements ConnectionsMBean, ManagedConnectionsListener {

	private static final Logger log = LoggerFactory.getLogger(ConnectionsMBeanImpl.class);
	
	public static final Qualifier Q_CONNECTIONS = ManagementCore.Q_RUNTIME
			.qualifier(ConnectionsMBean.CONNECTIONS_KEY, ConnectionsMBean.CONNECTIONS_KEY_VALUE); 
	
	protected BundleContext bundleContext;
	
	protected DeployedConnectionTracker tracker;

	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	EntaxyObjectService entaxyObjectService;

	protected static class ManagedConnectionDescriptor {
		
		ManagedConnection connection;
		ServiceRegistration<ConnectionMBean> registration;
		
	}
	
	protected Map<String, ManagedConnectionDescriptor> managed = new HashMap<>();
	
	public ConnectionsMBeanImpl() throws NotCompliantMBeanException {
		super(ConnectionsMBean.class);
	}
	
	@Activate
	public void activate(ComponentContext componentContext) {
		this.bundleContext = componentContext.getBundleContext();
		tracker = new DeployedConnectionTracker(bundleContext, this);
		tracker.open();
	}

	@Deactivate
	public void deactivate() {
		tracker.close();
	}
	
	@Override
	public void added(ManagedConnections connections) {
		Hashtable props = new Hashtable<>();
		// String jmxObjectName = "jmx.objectname";
		// String objectName = "ru.entaxy.esb:group=platform,category=connections,connection=";
		for (ManagedConnection connection: connections.managedConnections) {
			if (managed.containsKey(connection.getName()))
				this.managed.get(connection.getName()).registration.unregister();
			try {
				ConnectionMBeanImpl service = new ConnectionMBeanImpl(bundleContext, connection);
				ManagedConnectionDescriptor descriptor = new ManagedConnectionDescriptor();
				descriptor.connection = connection;
				props.put(ManagementCore.JMX_OBJECTNAME
						, Q_CONNECTIONS.qualifier(ConnectionMBean.CONNECTION_KEY
								, connection.getName()).getValue());
				descriptor.registration = bundleContext.registerService(ConnectionMBean.class
						, service
						, props);
				managed.put(connection.getName(), descriptor);
			} catch (Exception e) {
				log.error("Error adding connection [" + connection.getName() + "]", e);
			}
		}
	}
	
	@Override
	public void removed(ManagedConnections connections) {
		for (ManagedConnection connection: connections.managedConnections) {
			if (managed.containsKey(connection.getName())) {
				this.managed.get(connection.getName()).registration.unregister();
				this.managed.remove(connection.getName());
			}
		}
	}

    @Override
    public String getConnectionConfig(String connectionName) throws Exception {
        return entaxyObjectService.findObject(connectionName, "entaxy.runtime.connection").getConfiguration();
    }
}
