/*-
 * ~~~~~~licensing~~~~~~
 * connection-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.connection.tracker;

import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.wiring.BundleCapability;
import org.osgi.framework.wiring.BundleRevision;
import org.osgi.util.tracker.BundleTrackerCustomizer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.core.management.connection.ConnectionsMBean;
import ru.entaxy.esb.platform.core.management.connection.impl.ManagedConnection;
import ru.entaxy.esb.platform.core.management.connection.impl.ManagedConnections;
import ru.entaxy.esb.platform.core.management.connection.impl.ManagedConnectionsListener;
import ru.entaxy.platform.base.support.CommonUtils;

public class DeployedConnectionCustomizer implements BundleTrackerCustomizer<ManagedConnections> {

	private static final Logger log = LoggerFactory.getLogger(DeployedConnectionCustomizer.class);
	
	protected ManagedConnectionsListener listener;
	
	public DeployedConnectionCustomizer(ManagedConnectionsListener listener) {
		this.listener = listener;
	}
	
	@Override
	public ManagedConnections addingBundle(Bundle bundle, BundleEvent event) {
		log.debug("INSPECTING: " + bundle.getBundleId());
		BundleRevision revision = bundle.adapt(BundleRevision.class);
		if (revision == null)
			return null;
		log.debug("REVISION FOUND: " + bundle.getBundleId());
		List<BundleCapability> capabilities = revision.getDeclaredCapabilities(ConnectionsMBean.CAPABILITY_NAMESPACE);
		if ((capabilities==null) || (capabilities.size()==0))
			return null;
		log.debug("CAPABILITIES FOUND: " + bundle.getBundleId());
		ManagedConnections result = new ManagedConnections();
		for (BundleCapability capability: capabilities) {
			Object val = capability.getAttributes().get("name");
			String name = val==null?"":val.toString();
			log.debug("CAPABILITIES/NAME [" + name + "] : " + bundle.getBundleId());
			if (!CommonUtils.isValid(name))
				continue;
			val = capability.getAttributes().get("platform");
			boolean isPlatform = val==null?false:"true".equals(val.toString());
			val = capability.getAttributes().get("label");
			String label = val == null ? "" : val.toString();
			ManagedConnection mc = (new ManagedConnection())
					.name(name)
					.platform(isPlatform)
					.deployed(true)
					.local(isPlatform)
					.shared(true)
					.bundleId(bundle.getBundleId())
					.label(label);
			result.managedConnections.add(mc);
		}
		listener.added(result);
		return result;
	}

	@Override
	public void modifiedBundle(Bundle bundle, BundleEvent event, ManagedConnections object) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void removedBundle(Bundle bundle, BundleEvent event, ManagedConnections object) {
		log.debug("BUNDLE [{}] REMOVING", bundle.getBundleId());
		listener.removed(object);
	}

}
