/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile;

import ru.entaxy.esb.platform.base.management.core.Qualifier;
import ru.entaxy.esb.platform.base.management.core.api.Attribute;
import ru.entaxy.esb.platform.base.management.core.api.MBeanAnnotated;
import ru.entaxy.esb.platform.base.management.core.api.MBeanExportPolicy;
import ru.entaxy.esb.platform.base.management.core.api.Operation;
import ru.entaxy.esb.platform.base.management.core.api.RuntimeTypedMBean;
import ru.entaxy.esb.platform.base.management.core.utils.BundleAwareMBean;

@MBeanAnnotated(policy = MBeanExportPolicy.ANNOTATION_ENRICH)
public interface ConnectorMBean extends BundleAwareMBean, RuntimeTypedMBean {

	public static interface Helper {

		public static Qualifier getQualifier(String profileName, String connectorName) {
			return ConnectorsMBean.Helper.getQualifier(profileName).qualifier(CONNECTOR_KEY, connectorName);
		}
		
	}

	public static final String CAPABILITY_NAMESPACE = "entaxy.runtime.connector";
	
	public static final String CONNECTOR_KEY = "connector";
	
	public static final String CONNECTOR_KEY_VALUE_DEFAULT = "default";

	// TODO implement if needed
	// public boolean isDefault();

	@Attribute(desc = "Factory ID used to create the connector")
	public String getFactoryId();
	
	@Attribute(desc = "Connector name")
	public String getName();
	
	@Attribute(desc = "Connector registered name used in registry")
	public String getRegisteredName();

	@Attribute(desc = "Connector system")
	public String getSystem();

	@Attribute(desc = "Connector classifier")
	public String getClassifier();

	@Attribute(desc = "Connector display name")
	public String getDisplayName();

	@Attribute(desc = "Connector direction")
	public String getDirection();

	@Operation(desc = "Starts the connector")
	public void start() throws Exception;
	
	@Operation(desc = "Stops the connector")
	public void stop() throws Exception;
	
}
