/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl;

import javax.naming.OperationNotSupportedException;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.core.management.profile.api.ProfileManager;

@Component(service = {ProfileHelper.class},
		immediate = true)
public class ProfileHelperImpl implements ProfileHelper {

	private static final Logger log = LoggerFactory.getLogger(ProfileHelperImpl.class);
	
	protected static final String EXPLANATION = "ProfileManager not bound";
	
	protected ProfileManager profileManager = null;
	
	protected ProfileManager nullProfileManager = null;
	
	@Activate
	public void activate(ComponentContext componentContext) {
		
		this.nullProfileManager = new ProfileManager() {
			
			@Override
			public void uninstallProfile(String idOrName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void stopProfile(String idOrName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void startProfile(String idOrName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void createProfile(String id, String name, String description) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}

			@Override
			public String getProfileConfig(String idOrName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
		};
		
	}
	
	
	
	public ProfileManager getEffectiveProfileManager() {
		return profileManager!=null?profileManager:nullProfileManager;
	}

	@Reference (collectionType = CollectionType.SERVICE
			, cardinality = ReferenceCardinality.OPTIONAL
			, policyOption = ReferencePolicyOption.GREEDY
			, service = ProfileManager.class
			, unbind = "unsetProfileManager")
	public void setProfileManager(ProfileManager profileManager) {
		this.profileManager = profileManager;
		log.debug(">> ProfileManager is bound");
	}

	public void unsetProfileManager(ProfileManager profileManager) {
		this.profileManager = null;
		log.debug(">> ProfileManager is unbound");
	}



	@Override
	public void createProfile(String id, String name, String description) throws Exception {
		getEffectiveProfileManager().createProfile(id, name, description);
	}

	@Override
	public String getProfileConfig(String idOrName) throws Exception {
		return getEffectiveProfileManager().getProfileConfig(idOrName);
	}

	@Override
	public void startProfile(String idOrName) throws Exception {
		getEffectiveProfileManager().startProfile(idOrName);
	}

	@Override
	public void stopProfile(String idOrName) throws Exception {
		getEffectiveProfileManager().stopProfile(idOrName);
	}

	@Override
	public void uninstallProfile(String idOrName) throws Exception {
		getEffectiveProfileManager().uninstallProfile(idOrName);
	}

}
