/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.api.EntaxyRuntimeTyped;
import ru.entaxy.esb.platform.base.management.core.utils.BundleAwareMBeanImpl;
import ru.entaxy.esb.platform.core.management.profile.ProfileMBean;
import ru.entaxy.esb.platform.core.management.profile.impl.helper.Helpers;
import ru.entaxy.esb.system.auth.basic.jpa.api.entity.BasicAuthAccount;
import ru.entaxy.esb.system.jpa.entity.System;
import ru.entaxy.platform.base.support.CommonUtils;

// @TODO move string to constant
@EntaxyRuntimeTyped(name = "entaxy.runtime.profile")
public class ProfileMBeanImpl extends BundleAwareMBeanImpl<ManagedProfile, ProfileMBean> implements ProfileMBean {

	private static final Logger log = LoggerFactory.getLogger(ProfileMBeanImpl.class);
	
	Helpers helpers;
	
	public ProfileMBeanImpl(BundleContext bundleContext, ManagedProfile managedProfile
				, Helpers helpers) 
			throws NotCompliantMBeanException {
		super(ProfileMBean.class, managedProfile);
		this.bundleContext = bundleContext;
		this.helpers = helpers;
	}
	
	@Override
	public String getName() {
		return managedObject.getName();
	}

	@Override
	public String getDescription() {
		return managedObject.getDescription();
	}
	
	@Override
	public String getUuid() {
		System system = helpers.systemService.getByName(getName());
		if (system == null)
			return "N/A";
		return system.getUuid();
	}
	
	
	
	@Override
	public String getBasicAuthAccount() {
		Optional<BasicAuthAccount> bas = this.helpers.basicAuthService.getBySystem(getUuid());
		if (bas.isPresent())
			return bas.get().getLogin();
		return "";
	}
	
	@Override
	public void createAccount(String login, String password) throws Exception {
		if (CommonUtils.isValid(getBasicAuthAccount()))
			throw new IllegalArgumentException("Account already exists");
		// FIXME remove hardcoded
		this.helpers.basicAuthService.saveCommon(login
				, password
				, getUuid()
				, "admin"
				, "admin");
		
	}
	
	@Override
	public void addConnector(String connectorFactoryId, Map<String, String> parameters) throws Exception {
		this.helpers.connectorHelper.addConnector(getName(), connectorFactoryId, parameters);
	}

	@Override
	public void removeConnector(String connectorName) throws Exception {
		this.helpers.connectorHelper.removeConnector(getName(), connectorName);
	}

	@Override
	public void startConnector(String connectorName) throws Exception {
		this.helpers.connectorHelper.startConnector(getName(), connectorName);
	}

	@Override
	public void stopConnector(String connectorName) throws Exception {
		this.helpers.connectorHelper.stopConnector(getName(), connectorName);
	}

	@Override
	public String getConnectorConfig(String connectorName) throws Exception {
		return helpers.entaxyObjectService.findObject(connectorName, "entaxy.runtime.connector").getConfiguration();
	}

	@Override
	public List<String> getAllowedTargets() {
		try {
			return helpers.permissionHelper.getObjectPermissions(getUuid());
		} catch (Exception e) {
			log.warn("getAllowedTargets not succeded", e);
		}
		return new ArrayList<>();
	}

	@Override
	public List<String> getAllowedSources() {
		try {
			return helpers.permissionHelper.getSubjectPermissions(getUuid());
		} catch (Exception e) {
			log.warn("getAllowedSources not succeded", e);
		}
		return new ArrayList<>();
	}
	
	@Override
	public void allowSource(String idOrName) throws Exception {
		helpers.permissionHelper.createPermission(idOrName, getUuid());
	}

	@Override
	public void allowTarget(String idOrName) throws Exception {
		helpers.permissionHelper.createPermission(getUuid(), idOrName);
		
	}
	
}
