/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl.connector;

import java.util.Map;

import javax.naming.OperationNotSupportedException;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.core.management.profile.api.ConnectorManager;
import ru.entaxy.esb.platform.core.management.profile.impl.ProfileHelperImpl;

@Component(service = ConnectorHelper.class, immediate = true)
public class ConnectorHelperImpl implements ConnectorHelper {

	private static final Logger log = LoggerFactory.getLogger(ProfileHelperImpl.class);
	
	protected static final String EXPLANATION = "ConnectorManager not bound";
	
	protected ConnectorManager connectorManager = null;
	
	protected ConnectorManager nullConnectorManager = null;

	@Activate
	public void activate(ComponentContext componentContext) {
		this.nullConnectorManager = new ConnectorManager() {
			
			@Override
			public void stopConnector(String idOrName, String connectorName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void startConnector(String idOrName, String connectorName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void removeConnector(String idOrName, String connectorName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void addConnector(String idOrName, String connectorFactoryId, Map<String, String> parameters)
					throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
		};
	}

	public ConnectorManager getEffectiveConnectorManager() {
		return connectorManager!=null?connectorManager:nullConnectorManager;
	}

	@Reference (collectionType = CollectionType.SERVICE
			, cardinality = ReferenceCardinality.OPTIONAL
			, policyOption = ReferencePolicyOption.GREEDY
			, service = ConnectorManager.class
			, unbind = "unsetConnectorManager")
	public void setConnectorManager(ConnectorManager connectorManager) {
		this.connectorManager = connectorManager;
		log.debug(">> ConnectorManager is bound");
	}

	public void unsetConnectorManager(ConnectorManager connectorManager) {
		this.connectorManager = null;
		log.debug(">> ConnectorManager is unbound");
	}
	
	
	@Override
	public void addConnector(String idOrName, String connectorFactoryId, Map<String, String> parameters)
			throws Exception {
		getEffectiveConnectorManager().addConnector(idOrName, connectorFactoryId, parameters);
	}

	@Override
	public void removeConnector(String idOrName, String connectorName) throws Exception {
		getEffectiveConnectorManager().removeConnector(idOrName, connectorName);
	}

	@Override
	public void startConnector(String idOrName, String connectorName) throws Exception {
		getEffectiveConnectorManager().startConnector(idOrName, connectorName);
	}

	@Override
	public void stopConnector(String idOrName, String connectorName) throws Exception {
		getEffectiveConnectorManager().stopConnector(idOrName, connectorName);
	}

}
