/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl.connector;

import java.util.Iterator;
import java.util.List;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.utils.ManagedObjectsListener;
import ru.entaxy.esb.platform.core.management.profile.ConnectorMBean;
import ru.entaxy.esb.platform.core.management.profile.impl.helper.Helpers;

public class ManagedConnectorsListener extends ManagedObjectsListener<List<ManagedConnector>, ManagedConnector, ConnectorMBean, ConnectorMBeanImpl> { 

	private static final Logger log = LoggerFactory.getLogger(ManagedConnectorsListener.class);
	
	protected Helpers helpers;
	
	public ManagedConnectorsListener(BundleContext bundleContext, Helpers helpers) {
		super(bundleContext, ConnectorMBean.class);
		this.helpers = helpers;
	}

	@Override
	protected Iterator<ManagedConnector> getIterator(List<ManagedConnector> managedObject) {
		return managedObject.iterator();
	}

	@Override
	protected String getKeyValue(ManagedConnector object) {
		return object.getName();
	}

	@Override
	protected ConnectorMBeanImpl createService(ManagedConnector managedObject) {
		try {
			return new ConnectorMBeanImpl(bundleContext, managedObject, helpers);
		} catch (Exception e) {
			log.error("Error creating service", e);
			return null;
		}
	}

	@Override
	protected String getJmxObjectName(ManagedConnector managedObject) {
		return ConnectorMBean.Helper.getQualifier(
				managedObject.getSystem(), 
				managedObject.getName()
			).getValue();
	}

}
