/*-
 * ~~~~~~licensing~~~~~~
 * service-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.service;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.Qualifier;
import ru.entaxy.esb.platform.base.management.core.api.MBeanAnnotated;
import ru.entaxy.esb.platform.base.management.core.api.MBeanExportPolicy;
import ru.entaxy.esb.platform.base.management.core.api.Operation;
import ru.entaxy.esb.platform.base.management.core.api.Parameter;

@MBeanAnnotated(policy = MBeanExportPolicy.ANNOTATION_ENRICH)
public interface ServicesMBean {
	
	public static final String CAPABILITY_NAMESPACE = "entaxy.runtime.service";
	
	public static final String SERVICES_KEY = "category";
	
	public static final String SERVICES_KEY_VALUE = "services";

	public static final Qualifier Q_SERVICES = ManagementCore.Q_RUNTIME
				.qualifier(ServicesMBean.SERVICES_KEY, ServicesMBean.SERVICES_KEY_VALUE);

	@Operation(desc = "Gets service's configuration")
	public String getServiceConfig(
			@Parameter(name = "idOrName", desc = "Service id or name") String idOrName) throws Exception;

	@Operation(desc = "Start service")
	public void startService(
			@Parameter(name = "idOrName", desc = "Service id or name") String idOrName) throws Exception;

	@Operation(desc = "Stop service")
	public void stopService(
			@Parameter(name = "idOrName", desc = "Service id or name") String idOrName) throws Exception;

	@Operation(desc = "Uninstall service")
	public void uninstallService(
			@Parameter(name = "idOrName", desc = "Service id or name") String idOrName) throws Exception;

}
