/*-
 * ~~~~~~licensing~~~~~~
 * service-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.*;
import org.osgi.util.tracker.BundleTracker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.esb.platform.core.management.service.ServiceMBean;
import ru.entaxy.esb.platform.core.management.service.ServicesMBean;
import ru.entaxy.esb.platform.core.management.service.impl.tracker.DeployedServiceCustomizer;
import ru.entaxy.esb.platform.core.management.service.impl.tracker.ManagedService;
import ru.entaxy.esb.platform.core.management.service.impl.tracker.ManagedServiceListener;
import ru.entaxy.esb.system.management.bundle.manager.BundleManager;
import ru.entaxy.platform.base.objects.EntaxyObjectService;
import ru.entaxy.platform.base.support.osgi.tracker.BundleTrackerUtils;
import ru.entaxy.platform.base.support.osgi.tracker.filter.BundleCapabilityFilter;

@Component(service = { ServicesMBean.class, DynamicMBean.class, MBeanRegistration.class }, property = {
		ManagementCore.JMX_OBJECTNAME + "=" + ManagementCore.Q_RUNTIME_S + "," + ServicesMBean.SERVICES_KEY + "="
				+ ServicesMBean.SERVICES_KEY_VALUE }, scope = ServiceScope.SINGLETON, immediate = true)
public class ServicesMBeanImpl extends AnnotatedMBean<ServicesMBean> implements ServicesMBean {

	private static final Logger log = LoggerFactory.getLogger(ServicesMBeanImpl.class);

	protected BundleContext bundleContext;
	
	protected BundleTracker<List<ManagedService>> tracker;

	private String SERVICE_RUNTIME_TYPE = "entaxy.runtime.service";

	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	public EntaxyObjectService entaxyObjectService;

	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	private BundleManager bundleManager;

	protected static class ManagedServiceDescriptor {

		ManagedService profile;
		ServiceRegistration<ServiceMBean> registration;

	}

	protected Map<String, ManagedServiceDescriptor> managed = new HashMap<>();

	public ServicesMBeanImpl() throws NotCompliantMBeanException {
		super(ServicesMBean.class);
	}

	@Activate
	public void activate(ComponentContext componentContext) {
		this.bundleContext = componentContext.getBundleContext();

		//tracker = new DeployedProfileTrackerOld(bundleContext, this);
		tracker = BundleTrackerUtils.<List<ManagedService>>createBuilder()
				.customizer(
						(new DeployedServiceCustomizer())
						.listener(new ManagedServiceListener(bundleContext))
					)
				.addFilter(
						(new BundleCapabilityFilter()).namespace(ServiceMBean.CAPABILITY_NAMESPACE)
					)
				.bundleContext(bundleContext)
				.bundleState(Bundle.ACTIVE | Bundle.INSTALLED | Bundle.RESOLVED | Bundle.STARTING | Bundle.STOPPING)
				.get();
		tracker.open();
	}
	
	@Deactivate
	public void deactivate() {
		tracker.close();
	}

	@Override
	public String getServiceConfig(String idOrName) {
		return entaxyObjectService.findObject(idOrName, SERVICE_RUNTIME_TYPE).getConfiguration();
	}

	@Override
	public void startService(String idOrName) throws Exception {
		long bundleId = entaxyObjectService.findObject(idOrName, SERVICE_RUNTIME_TYPE).getBundleInfo().getBundleId();
		bundleManager.startBundle(bundleContext.getBundle(bundleId).getSymbolicName());
	}

	@Override
	public void stopService(String idOrName) throws Exception {
		long bundleId = entaxyObjectService.findObject(idOrName, SERVICE_RUNTIME_TYPE).getBundleInfo().getBundleId();
		bundleManager.stopBundle(bundleContext.getBundle(bundleId).getSymbolicName());
	}

	@Override
	public void uninstallService(String idOrName) throws Exception {
		long bundleId = entaxyObjectService.findObject(idOrName, SERVICE_RUNTIME_TYPE).getBundleInfo().getBundleId();
		Bundle bundle = bundleContext.getBundle(bundleId);
		bundleManager.uninstallBundleByName(bundle.getSymbolicName(), "service", bundle.getVersion().toString());
		// todo remove all connected resources
	}
}
