/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor;

import java.util.HashMap;
import java.util.Map;

public class CommandInstructions {

	protected Map<String, Object> instructionsMap = new HashMap<>();
	
	public CommandInstructions() {
		super();
	}
	
	public CommandInstructions(Map<String, Object> instructionsMap) {
		super();
		load(instructionsMap);
	}
	
	public void load(Map<String, Object> instructionsMap) {
		this.instructionsMap = instructionsMap;
	}
	
	public boolean has(String name) {
		return this.instructionsMap.containsKey(name);
	}
	
	public Boolean getBoolean(String name) {
		if (!this.has(name))
			return null;
		Object value = this.instructionsMap.get(name);
		
		if (value instanceof Boolean)
			return (Boolean)value;
		
		if (value instanceof String)
			return "true".equals(((String)value).toLowerCase());
		
		if (value instanceof Number)
			return ((Number)value).longValue() > 0;
			
		if (value instanceof Long)
			return ((Long)value).longValue() > 0;
		
		return value!=null;
	}
	
	public Object get(String name) {
		return this.instructionsMap.get(name);
	}
	
	public String getString(String name) {
		if (!has(name))
			return null;
		Object value = instructionsMap.get(name);
		return value!=null?value.toString():"";
	}
	
	public Long getLong(String name) {
		if (!has(name))
			return null;
		Object value = instructionsMap.get(name);
		if (value instanceof Number)
			return ((Number)value).longValue();
		return (long)-1;
	}
	
}
