/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.Map;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.generationmodel.AbstractTask;
import ru.entaxy.platform.core.producer.executor.generationmodel.GeneratedList;
import ru.entaxy.platform.core.producer.executor.generationmodel.GenerationModel;

@CommandExecutor(id = "generate", predecessors = {"validate", "pre-generate"})
public class Generate extends AbstractCommandExecutor {

	public Generate(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
		// TODO Auto-generated constructor stub
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {
		
		GenerationModel generationModel = currentResult.findResultObject(GenerationModel.class);

		// generate
		this.printOutput("\n\t == Execution ==\n");
		for (AbstractTask at: generationModel.getTasks()) {
			at.setPrintOutput(isPrintOutput);
			this.printOutput("\tExecuting: " + at.getInfo());
			at.execute(currentResult, commandResult, entaxyProducerService, instructions);
			this.printOutput("\n :: Result ::\n");
			if (at.getGeneratedResult()==null)
				this.printOutput("-> NOTHING");
			else {
				Generated g = at.getGeneratedResult();
				this.printOutput("\tTYPE :: [" + g.getType() + "]");
				this.printOutput("\tCONTENT :: \n\n");
				if (g.getObject() == null)
					this.printOutput("-> EMPTY\n\n");
				else
					this.printOutput(g.getObject().toString());
			}
			this.printOutput("\n ::\n");
		}
		this.printOutput("\n\t ==\n");
		
		// collect result
		GeneratedList resultObject = new GeneratedList();
		for (AbstractTask at: generationModel.getTasks())
			resultObject.add(at.getGeneratedResult());
		commandResult.resultObject(resultObject);	
			
		return true;
	}

}
