/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObject;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObjectList;
import ru.entaxy.platform.core.producer.executor.installer.InstalledObject;
import ru.entaxy.platform.core.producer.executor.installer.InstalledObjectList;
import ru.entaxy.platform.core.producer.executor.installer.ObjectInstaller;
import ru.entaxy.platform.core.producer.executor.support.ObjectSupportRegistry;

@CommandExecutor(id = "install", predecessors = "deploy")
public class Install extends AbstractCommandExecutor {

	private static final Logger log = LoggerFactory.getLogger(Install.class);
	
	public static final String INSTALL_LOCAL_INSTRUCTION = "installLocal";
	public static final String UPDATE_INSTRUCTION = "update";
	public static final String INSTALL_ONLY_IF_MISSING_INSTRUCTION = "installOnlyIfMissing";
	public static final String START_LEVEL_INSTRUCTION = "startLevel";

	public Install(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {

		DeployedObjectList objectList = currentResult.findResultObject(DeployedObjectList.class);
		if (objectList == null) {
			log.info("Nothing to install");
			return true;
		}
		
		InstalledObjectList resultList = new InstalledObjectList();
		
		for (DeployedObject obj: objectList) {
			
			if (obj == null)
				continue;
			
			printOutput(" = Installing for deployed of type [" + obj.getObject().getClass().getName() + "]" );
			
			
			ObjectInstaller installer = ObjectSupportRegistry.getInstance().findInstaller(obj);
			if (installer == null) {
				log.warn("ObjectInstaller not found for deployed object of type [{}]", obj.getObject().getClass().getName());
				printOutput(" :: ObjectInstaller not found");
				continue;
			}
			
			printOutput(" :: ObjectInstaller is: " + installer.getClass().getName());
			InstalledObject result = installer.install(obj, instructions);
			if (result != null) {
				resultList.add(result);
				printOutput(" :: Result is: " + result.getClass().getName());
			} else 
				printOutput(" :: Result is: NULL");			
		}
		
		commandResult.resultObject(resultList);
		
		return true;
	}

}
