/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.EntaxyObjectStorage;
import ru.entaxy.platform.base.objects.EntaxyObjectStorageService;
import ru.entaxy.platform.base.support.osgi.OSGIUtils;
import ru.entaxy.platform.core.artifact.DeployedArtifact;
import ru.entaxy.platform.core.artifact.installer.builder.InstallationResult;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.installer.InstalledObject;
import ru.entaxy.platform.core.producer.executor.installer.InstalledObjectList;
import ru.entaxy.platform.core.producer.executor.support.EntaxyObjectPropertiesHelper;

@CommandExecutor(id = "store", predecessors = "install")
public class Store extends AbstractCommandExecutor {

	private static final Logger log = LoggerFactory.getLogger(Store.class);
	
	protected EntaxyObjectStorageService objectStorageService; 
	
	public Store(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
		try {
			objectStorageService = OSGIUtils.services().ofClass(EntaxyObjectStorageService.class).get();
		} catch (Exception e) {
			log.warn("EntaxyObjectStorageService not found yet", e);
		}
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {
		
		InstalledObjectList objectList = currentResult.findResultObject(InstalledObjectList.class);
		
		if (objectStorageService == null)
			objectStorageService = OSGIUtils.services().ofClass(EntaxyObjectStorageService.class).get();
		
		if (objectList == null)
			return true;
		
		for (InstalledObject obj: objectList) {
			if (!(obj.getObject() instanceof InstallationResult))
				continue;
			InstallationResult installationResult = (InstallationResult)obj.getObject();
			printOutput("\n -- INSTALLATION RESULT ::"
					+ "\n\t" + installationResult.getResult().toString()
					+ "\n\tproperties:"
					+ (installationResult.getProperties()==null
						?"\n\t\t<EMPTY>"
						:installationResult.getProperties().entrySet().stream()
							.map(entry->
							"\n\t\t[" + entry.getKey() + "] = ["
							+ (entry.getValue()==null
								?""
								:(entry.getValue() instanceof DeployedArtifact 
									?"DeployedArtifact :: " + (
										((DeployedArtifact)entry.getValue()).getArtifact()
											.getProperties().entrySet().stream()
											.map(e->
												"\n\t\t\t[" + e.getKey() + "] = ["
												+ (e.getValue()==null
													?""
													:e.getValue().toString()) + "]"
											).collect(Collectors.joining())
									)
									:entry.getValue().toString()) + "]"
								)
							).collect(Collectors.joining())
						)
					);
			
			Map<String, Object> objectProperties = new HashMap<>();
			objectProperties.putAll(installationResult.getProperties());
			if (EntaxyObjectPropertiesHelper.isMerged(objectProperties))
				objectProperties.putAll(EntaxyObjectPropertiesHelper.getMainObjectProperties(objectProperties));
			
			EntaxyObjectStorage objectStorage = objectStorageService.getObjectStorage(
					objectProperties
					.getOrDefault(EntaxyObject.FIELDS.OBJECT_TYPE, "").toString()
				);
			
			if (objectStorage == null) {
				log.warn("Storage not found for type: [" + installationResult.getProperties()
					.getOrDefault(EntaxyObject.FIELDS.OBJECT_TYPE, "").toString() + "]");
				continue;
			}
			
			objectStorage.store(objectProperties);
			
		}
		
		return true;
	}

}
