/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.generationmodel;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;

import ru.entaxy.platform.base.support.JSONUtils;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.objectmodel.Calculation;
import ru.entaxy.platform.core.producer.executor.objectmodel.FactoredObject;
import ru.entaxy.platform.core.producer.executor.objectmodel.ObjectModel;

public class CalculateTask extends AbstractObjectTask {

	private static final Logger log = LoggerFactory.getLogger(CalculateTask.class);

	private static Gson GSON = new Gson();
	
	public static boolean hasCalculations(FactoredObject factoredObject) {
		return factoredObject.getInternals().stream().filter(obj -> (obj instanceof Calculation))
				.count() > 0;
	}
	
	public CalculateTask(FactoredObject factoredObject) {
		super(factoredObject);
	}

	@Override
	public String getInfo() {
		return "CALCULATE FOR " + "[" + origin.getObjectId() + "]";
	}

	@Override
	public boolean execute(ProducerResult currentResult, CommandResult currentCommand,
			EntaxyProducerService producerService, Map<String, Object> instructions) throws Exception {
		
		ObjectModel objectModel = currentResult.findResultObject(ObjectModel.class);
		
		List<Calculation> calculations = this.origin.getInternals().stream()
				.filter(obj -> (obj instanceof Calculation))
				.map(obj -> (Calculation)obj)
				.collect(Collectors.toList());
		
		for (Calculation calculation: calculations) {
			Object result = calculation.calculate(objectModel, origin);
			printOutput("CALCULATION RESOLVED TO :: " + result);
			printOutput("\n REPLACE"
					+ "\n\t" + calculation.origin
					+ "\n\tWITH :: " + GSON.toJsonTree(result)
					+ "\n\tAT :: " + calculation.relativePath);
			String currentPath = calculation.relativePath;
			String targetPath = calculation.getTargetPath();
			boolean remove = (!currentPath.equals(targetPath) && calculation.isRemoveResolved());
			targetPath = targetPath.replace("$.", "");
			JSONUtils.replaceValue(origin.origin, targetPath, GSON.toJsonTree(result));
			if (remove) {
				JSONUtils.replaceValue(origin.origin, currentPath, GSON.toJsonTree("TO_REMOVE"));
			}

		}
		
		return true;
	}

}
