/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.generationmodel;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.EntaxyObjectService;

@Component(service = ObjectResolveHelper.class, immediate = true)
public class ObjectResolveHelper {

	public static ObjectResolveHelper _INSTANCE;
	
	public static ObjectResolveHelper getInstance(){
		return _INSTANCE;
	}
	
	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	EntaxyObjectService entaxyObjectService;
	
	protected Map<String, ObjectDataResolver> externalResolvers = new HashMap<>();
	
	DefaultObjectDataResolver defaultObjectDataResolver = new DefaultObjectDataResolver();
	
	@Activate
	public void activate() {
		_INSTANCE = this;
	}
	
	@Reference(unbind = "removeResolver", cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC,
			policyOption = ReferencePolicyOption.GREEDY, collectionType = CollectionType.SERVICEOBJECTS, service = ObjectDataResolver.class)
	public void addResolver(ObjectDataResolver resolver) {
		synchronized (externalResolvers) {
			externalResolvers.put(resolver.getSupportedObjectType(), resolver);
		}
	}

	public void removeResolver(ObjectDataResolver resolver) {
		synchronized (externalResolvers) {
			externalResolvers.remove(resolver.getSupportedObjectType());
		}
	}
	
	public EntaxyObject findObject(String objectId, String objectType) {
		return entaxyObjectService.getObjects().stream()
				.filter(obj -> (obj.getObjectId().equals(objectId) && obj.getObjectType().equals(objectType)))
				.findFirst().orElse(null);
	}
	
	public ObjectDataResolver getResolver(EntaxyObject object) {
		return externalResolvers.getOrDefault(object.getObjectType(), defaultObjectDataResolver);
	}
	
	public Object resolve(EntaxyObject entaxyObject, String propertyName) {
		return getResolver(entaxyObject).resolve(entaxyObject, propertyName);
	}
}
