/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.objectmodel;

import java.util.LinkedList;
import java.util.Map;
import java.util.UUID;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import ru.entaxy.platform.base.support.JSONUtils.ObjectWrapper;

public abstract class AbstractItem extends ObjectWrapper {

	public static final String OBJECT_UUID = "__uuid__";
	
	public static final String FACTORED_CONTEXT_KEY = "factoredObjects";
	
	public String jsonPath = "";
	public String relativePath = "";
	public JsonObject origin;
	public FactoredObject definedIn = null;
	
	protected String uuid = null;
	
	protected ObjectModel objectModel;

	protected AbstractItem(ObjectModel objectModel) {
		this.objectModel = objectModel;
	}	
	
	public boolean isDefinedIn(FactoredObject fo) {
		if (fo == null)
			return false;
		FactoredObject test = this.definedIn;
		while (test!=null && test!=fo)
			test = test.definedIn;
		return test == fo;
	}

	protected void computeRelativePath(String parentPath) {
		this.relativePath = this.jsonPath.replace(parentPath, "");
		if (this.relativePath.startsWith("."))
			this.relativePath = this.relativePath.substring(1);
	}

	public static void initContext(Map<String, Object> context) {
		if (!context.containsKey(FACTORED_CONTEXT_KEY))
			context.put(FACTORED_CONTEXT_KEY, new LinkedList<>());
	}

	public void wrap(JsonObject object, Map<String, Object> context, String path) {
		this.origin = object;
		this.jsonPath = path;
		if (this.origin.has(OBJECT_UUID)) {
			JsonElement je = this.origin.get(OBJECT_UUID);
			if (je.isJsonPrimitive())
				if (je.getAsJsonPrimitive().isString())
					this.uuid = je.getAsJsonPrimitive().getAsString(); 
		}
		if (this.uuid == null) {
			this.uuid = UUID.randomUUID().toString();
			this.origin.addProperty(OBJECT_UUID, this.uuid);
		}
	}
	
	public String getUuid() {
		return uuid;
	}
	
	public void updateFromOrigin() {
		
	} 
}
