/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.support;

import java.util.List;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.core.producer.executor.builder.BuiltObject;
import ru.entaxy.platform.core.producer.executor.builder.ObjectBuilder;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObject;
import ru.entaxy.platform.core.producer.executor.deployer.ObjectDeployer;
import ru.entaxy.platform.core.producer.executor.installer.ObjectInstaller;

@Component (service = ObjectSupportRegistry.class, immediate = true)
public class ObjectSupportRegistry {

	private static ObjectSupportRegistry INSTANCE = null;
	
	public static ObjectSupportRegistry getInstance() {
		return INSTANCE;
	}
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	volatile List<ObjectBuilder> objectBuilders;

	@Reference (cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	volatile List<ObjectDeployer> objectDeployers;
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	volatile List<ObjectInstaller> objectInstallers;
	
	@Activate
	public void activate() {
		INSTANCE = this;
	}
	
	public ObjectBuilder findBuilder(Generated generated) {
		synchronized (this.objectBuilders) {
			for (ObjectBuilder ob: this.objectBuilders)
				if (ob.isBuildable(generated))
					return ob;
		}
		return null;
	}

	public ObjectDeployer findDeployer(BuiltObject object) {
		synchronized (this.objectDeployers) {
			for (ObjectDeployer od: this.objectDeployers)
				if (od.isDeployable(object))
					return od;
		}
		return null;
	}
	
	public ObjectInstaller findInstaller(DeployedObject object) {
		synchronized (this.objectInstallers) {
			for (ObjectInstaller oi: this.objectInstallers)
				if (oi.isInstallable(object))
					return oi;
		}
		return null;
	}
	
}
