/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-config-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.config;

import org.osgi.service.component.annotations.Component;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.EntaxyObject.FIELDS;
import ru.entaxy.platform.core.producer.executor.generationmodel.DefaultObjectDataResolver;
import ru.entaxy.platform.core.producer.executor.generationmodel.ObjectDataResolver;

@Component(service = ObjectDataResolver.class, immediate = true)
public class ConfigObjectDataResolver extends DefaultObjectDataResolver {

	@Override
	public String getSupportedObjectType() {
		return ObjectConfig.CONFIG_OBJECT_TYPE;
	}
	
	@Override
	protected Object resolveNotRoot(EntaxyObject sourceObject, String propertyName, JsonObject jsonRoot) {
		/* System.out.println("\n\n\tRESOLVE - ConfigObjectDataResolver :: " + propertyName
				+ "\n" + jsonRoot); */
		if (jsonRoot.has(FIELDS.PROPERTIES)) {
			JsonElement propertiesElement = jsonRoot.get(FIELDS.PROPERTIES);
			if (propertiesElement.isJsonObject()) {
				JsonObject properties = propertiesElement.getAsJsonObject();
				/* System.out.println("\n\n\tCONFIGURED DATA:"
						+ "\nconfigured: " + properties.get(ObjectConfig.CONFIG_ID_FIELD_NAME)
						+ "\nconfiguredType: " + properties.get(ObjectConfig.CONFIG_TYPE_FIELD_NAME)
						+ "\nconfiguredProperties: " + properties.get(ObjectConfig.CONFIG_PROPERTIES_FIELD_NAME)
						); */
				if (properties.has(ObjectConfig.CONFIG_ID_FIELD_NAME) 
						&& properties.has(ObjectConfig.CONFIG_TYPE_FIELD_NAME) 
						&& properties.has(ObjectConfig.CONFIG_PROPERTIES_FIELD_NAME)) {
					
					String targetId = properties.get(ObjectConfig.CONFIG_ID_FIELD_NAME).getAsString();
					String targetType = properties.get(ObjectConfig.CONFIG_TYPE_FIELD_NAME).getAsString();
					JsonElement configuredPropertiesElement = properties.get(ObjectConfig.CONFIG_PROPERTIES_FIELD_NAME);
					if (configuredPropertiesElement.isJsonObject()) {
						JsonObject configuredProperties = configuredPropertiesElement.getAsJsonObject();
						String configPrefix = ObjectConfig.getConfigurationPrefix(targetId, targetType);
						
						if (configuredProperties.has(propertyName))
							return configPrefix + propertyName + "}";
						
					}
					
				}
				
			}
			
		}
		return super.resolveNotRoot(sourceObject, propertyName, jsonRoot);
	}
}
