/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.management.impl;

import java.util.HashMap;
import java.util.Map;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.osgi.service.component.annotations.ServiceScope;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.core.producer.api.EntaxyProducer;
import ru.entaxy.platform.core.producer.management.FactoriesMBean;
import ru.entaxy.platform.core.producer.management.ProducerMBean;

@Component(service = { FactoriesMBean.class, DynamicMBean.class, MBeanRegistration.class }, property = {
		ManagementCore.JMX_OBJECTNAME + "=" + ManagementCore.Q_PLATFORM_S + "," + FactoriesMBean.FACTORIES_KEY + "="
				+ FactoriesMBean.FACTORIES_KEY_VALUE }, scope = ServiceScope.SINGLETON, immediate = true)
public class FactoriesMBeanImpl extends AnnotatedMBean<FactoriesMBean> implements FactoriesMBean {

	private static final Logger log = LoggerFactory.getLogger(FactoriesMBeanImpl.class);
	
	protected Map<EntaxyProducer, ManagedTypedProducers> producersMap = new HashMap<>(); 
	
	protected Map<EntaxyProducer, ManagedProducer> managedProducersMap = new HashMap<>(); 
	
	protected Object managedProducersMapLock = new Object();
	
	protected BundleContext bundleContext;
	
	protected TypedProducerListener producerListener = null;
	
	// to ensure FactoryMBeanPublisher is started 
	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	FactoryMBeanPublisher factoryMBeanPublisher;
	
	@Activate
	public FactoriesMBeanImpl(BundleContext bundleContext) throws NotCompliantMBeanException {
		super(FactoriesMBean.class);
		this.bundleContext = bundleContext;
		log.debug("\n\tCONSTRUCTOR, bundleContext: [{}]\n"
				, this.bundleContext==null?"null":this.bundleContext.getBundle().getBundleId());
		this.producerListener = new TypedProducerListener(bundleContext, ProducerMBean.class);
	}

	
	@Reference(bind="bindProducer", unbind = "unbindProducer", cardinality = ReferenceCardinality.MULTIPLE
			, policy = ReferencePolicy.DYNAMIC
			, policyOption = ReferencePolicyOption.GREEDY, collectionType = CollectionType.SERVICE
			, service = EntaxyProducer.class)
	public void bindProducer(EntaxyProducer producer) {
		log.debug("\n\tBinding producer: [{}]\n", producer.getProducerId());
		synchronized (managedProducersMapLock) {
			if (managedProducersMap.containsKey(producer)) {
				managedProducersMap.get(producer).beforeRemove();
			}
			// ManagedTypedProducers mtp = new ManagedTypedProducers(producer);
			ManagedProducer mp = new ManagedProducer(bundleContext, producer);
			managedProducersMap.put(producer, mp);
		}
	}

	public void unbindProducer(EntaxyProducer producer) {
		synchronized (managedProducersMapLock) {
			if (managedProducersMap.containsKey(producer)) {
				managedProducersMap.get(producer).beforeRemove();
				managedProducersMap.remove(producer);
			}
		}
	}
/*
	public void bindProducer(EntaxyProducer producer) {
		log.debug("\n\tBinding producer: [{}]\n", producer.getProducerId());
		synchronized (producersMap) {
			if (producersMap.containsKey(producer)) {
				producerListener.removed(producersMap.get(producer));
			}
			ManagedTypedProducers mtp = new ManagedTypedProducers(producer);
			producersMap.put(producer, mtp);
			if (producerListener != null)
				producerListener.added(mtp);
		}
	}

	public void unbindProducer(EntaxyProducer producer) {
		synchronized (producersMap) {
			if (producersMap.containsKey(producer)) {
				if (producerListener != null)
					producerListener.removed(producersMap.get(producer));
				producersMap.remove(producer);
			}
		}
	}
*/	
	
	public void bindFactory(EntaxyFactory entaxyFactory) {
		
	}
	
}
