/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.management.impl;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.core.producer.management.FactoryMBean;

@Component(service = FactoryMBeanPublisher.class, immediate = true)
public class FactoryMBeanPublisher {

	protected static FactoryMBeanPublisher _INSTANCE = null;
	
	public static FactoryMBeanPublisher getInstance() {
		return _INSTANCE;
	};
	
	protected BundleContext bundleContext;
	
	@Activate
	public void activate(ComponentContext componentContext) {
		this.bundleContext = componentContext.getBundleContext();
		_INSTANCE = this;
	}
	
	public List<ServiceRegistration<FactoryMBean>> publish(FactoryMBeanImpl service, EntaxyFactory factory){
		
		List<ServiceRegistration<FactoryMBean>> result = new ArrayList<>();
		
		Hashtable<String, String> props = new Hashtable<>();
		props.put(ManagementCore.JMX_OBJECTNAME
			, FactoryMBean.getQualifier(factory.getId(), factory.getType()).getValue());
		
		// catch and ignore exception typical for bundle refreshing process
		try {
			result.add(this.bundleContext.registerService(FactoryMBean.class, service, props));
		} catch (IllegalStateException e) {
			if (!CommonUtils.isValid(e.getMessage()) || !e.getMessage().contains("Invalid BundleContext"))
				throw e;
		}
		
		if (CommonUtils.isValid(factory.getCategory())) {
			props.put(ManagementCore.JMX_OBJECTNAME
					, ManagementCore.Q_PLATFORM
						.qualifier(FactoryMBean.CATEGORY_KEY, factory.getCategory())
						.qualifier(FactoryMBean.FACTORY_KEY, factory.getId()).getValue());
			
			result.add(this.bundleContext.registerService(FactoryMBean.class, service, props));
			
		}
		
		return result;
		
	}
	
}
