/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.management.impl;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.core.producer.api.EntaxyProducer;
import ru.entaxy.platform.core.producer.management.FactoryMBean;

public class ManagedProducer implements EntaxyProducer.EntaxyProducerListener {

	private static final Logger log = LoggerFactory.getLogger(ManagedProducer.class);
	
	protected BundleContext bundleContext;
	
	protected EntaxyProducer entaxyProducer;
	
	protected Map<String, List<ServiceRegistration<FactoryMBean>>> services = new HashMap<>();
	protected Map<String, FactoryMBeanImpl> mbeans = new HashMap<>();
	
	public ManagedProducer(BundleContext bundleContext, EntaxyProducer entaxyProducer) {
		this.bundleContext = bundleContext;
		this.entaxyProducer = entaxyProducer;
		this.entaxyProducer.addListener(this);
		synchronized (this.services) {
			for(EntaxyFactory factory: this.entaxyProducer.getAllFactories())
				registerFactory(factory);
		}
	}

	@Override
	public void factoryAdded(String factoryId, String factoryType, EntaxyProducer producer) {
		factoryRemoved(factoryId, factoryType, producer);
		synchronized (this.services) {
			registerFactory(this.entaxyProducer.getFactoryById(factoryId));
		}
	}

	@Override
	public void factoryRemoved(String factoryId, String factoryType, EntaxyProducer producer) {
		synchronized (this.services) {
			if (this.services.containsKey(factoryId)) {
				for (ServiceRegistration<FactoryMBean> sr: this.services.get(factoryId))
					try {
						sr.unregister();
					} catch (Exception e) {
					}
				this.services.remove(factoryId);
				this.mbeans.remove(factoryId);
			}
		}
	}

	protected void registerFactory(EntaxyFactory factory) {
		try {
			FactoryMBeanImpl impl = new FactoryMBeanImpl(factory);
			
			Hashtable<String, String> props = new Hashtable<>();
			props.put(ManagementCore.JMX_OBJECTNAME
				, FactoryMBean.getQualifier(factory.getId(), factory.getType()).getValue());
			
			this.services.put(factory.getId()
					, FactoryMBeanPublisher.getInstance().publish(impl, factory));
			this.mbeans.put(factory.getId(), impl);
			
		} catch (NotCompliantMBeanException e) {
			log.error("Error creating FactoryMBeanImpl", e);
		}
	}
	
	public void beforeRemove() {
		synchronized (services) {
			for (List<ServiceRegistration<FactoryMBean>> lsr: services.values())
				for (ServiceRegistration<FactoryMBean> sr: lsr)
					try {
						sr.unregister();
					} catch (Exception e) {
						
					}
			services.clear();
			mbeans.clear();
		}
	}
	
}
