/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-resources-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.resources.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.core.producer.resources.EntaxyResourceStringFormatter;
import ru.entaxy.platform.core.producer.resources.StringFormatterService;

@Component (service = StringFormatterService.class, immediate = true)
public class StringFormatterServiceImpl implements StringFormatterService {

	protected static StringFormatterService _INSTANCE;
		
	public static StringFormatterService getInstance() {
		return _INSTANCE;
	}
	
	protected Map<String, EntaxyResourceStringFormatter> formattersMap = new HashMap<>(); 
	
	@Activate
	public void activate() {
		_INSTANCE = this;
	}
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, collectionType = CollectionType.SERVICE, policy = ReferencePolicy.DYNAMIC
			, policyOption = ReferencePolicyOption.GREEDY, unbind = "removeFormatter")
	public void addFormatter(EntaxyResourceStringFormatter formatter) {
		synchronized (formattersMap) {
			this.formattersMap.put(formatter.getName(), formatter);
		}
	}

	public void removeFormatter(EntaxyResourceStringFormatter formatter) {
		synchronized (formattersMap) {
			this.formattersMap.remove(formatter.getName());
		}
	}
	
	@Override
	public String format(String value, String[] formatters) {
		String current = value;
		for (int i=0; i<formatters.length; i++) {
			String formatter = formatters[i].trim();
			if (this.formattersMap.containsKey(formatter))
				current = this.formattersMap.get(formatter).format(current);
		}
		return current;
	}

	@Override
	public String unformat(String value, String[] formatters) {
		String current = value;
		for (int i=0; i<formatters.length; i++) {
			String formatter = formatters[i].trim();
			if (this.formattersMap.containsKey(formatter))
				current = this.formattersMap.get(formatter).unformat(current);
		}
		return current;
	}
}
