/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.shell;

import java.util.List;

import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.Option;
import org.apache.karaf.shell.api.action.lifecycle.Service;
import org.apache.karaf.shell.support.table.ShellTable;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory.FieldInfo;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory.RefFieldInfo;
import ru.entaxy.platform.base.support.CommonUtils;

@Service
@Command(scope = EntaxyProducerServiceSupport.SCOPE, name = "factory-fields")
public class FactoryFields extends FactoryAwareCommand {

	@Option(name = "-o", aliases = {"--output"}, required = false)
	String outputType;
	
	@Override
	protected Object doExecute() throws Exception {
		
		ShellTable table = new ShellTable();
		table.column("Name");
		table.column("Type");
		table.column("DisplayName");
		table.column("DefaultValue").maxSize(50).wrap(false);
		table.column("Required");
		table.column("Immutable");
		table.column("Is ref");
		table.column("Is back ref");
		table.column("Is ref by value");
		
		List<FieldInfo> fields;
		if (CommonUtils.isValid(outputType)) {
			System.out.println("FOR outputType = " + outputType);
			fields = entaxyFactory.getFields(outputType);
		} else {
			System.out.println("FOR default outputType = ");
			fields = entaxyFactory.getFields();
		}
		
		for (EntaxyFactory.FieldInfo fi: fields)
			table.addRow().addContent(
					fi.getName(),
					fi.getType(),
					fi.getDisplayName(),
					fi.getDefaultValue(),
					(fi.isRequired()?"*":""),
					(fi.isImmutable()?"*":""),
					(fi.isRef()?"*":""),
					(fi.isRef()
							?(((RefFieldInfo)fi).isBackRef()
									?"*"
									:"")
							:""),
					(fi.isRef()
							?(((RefFieldInfo)fi).isRefByValueOnly()
									?"*"
									:"")
							:"")
				);
		
		table.print(System.out);
		
		return null;
	}

}
