/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.shell;

import java.util.Map;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Argument;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.Option;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import com.google.gson.JsonElement;
import com.google.gson.JsonParser;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.base.support.JSONUtils;

@Service
@Command (scope = EntaxyProducerServiceSupport.SCOPE, name = "factory-generate")
public class FactoryGenerate extends FactoryAwareCommand implements Action {

	@Option(name = "-ot", aliases = {"--outputType"}, multiValued = false, required = false)
	String outputType;

	@Option(name = "-s", aliases = {"--scope"}, multiValued = false, required = false)
	String scope;
	
	@Argument(index = 1, name = "properties", required = false)
	String properties;
	
	@Override
	protected Object doExecute() throws Exception {
		if (!CommonUtils.isValid(properties))
			this.properties = "{}";
		
		JsonElement je = (new JsonParser()).parse(properties);
		
		Map<String, Object> propertiesMap = JSONUtils.element2map(je);
		
		Generated result = null;
		
		if (!CommonUtils.isValid(outputType))
			result = entaxyFactory.generate(propertiesMap);
		else if (!CommonUtils.isValid(scope))
			result = entaxyFactory.generate(outputType, propertiesMap);
		else
			result = entaxyFactory.generate(outputType, scope, propertiesMap);
		
		if (result == null)
			System.out.println("NOTHING GENERATED");
		else {
			System.out.println("GENERATED :: [" + result.getType() + "]");
			System.out.println("===");
			System.out.println(result.getObject()==null?"->nothing":result.getObject().toString());
			System.out.println("===");
		}
		
		return null;
	}

}
