/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.shell;

import java.util.ArrayList;
import java.util.List;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Argument;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;
import org.apache.karaf.shell.support.table.Col;
import org.apache.karaf.shell.support.table.ShellTable;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.core.producer.api.EntaxyProducer;

@Service
@Command (scope = "producer", name = "factory-list")
public class ListFactories extends EntaxyProducerServiceSupport implements Action {

	@Argument (index = 0, multiValued = false, required = false)
	String targetType;
	
	@Override
	public Object execute() throws Exception {
		List<EntaxyProducer> producers = new ArrayList<>();
		if (null != targetType) {
			EntaxyProducer producer = producerService.getProducerForType(targetType);
			if (producer != null)
				producers.add(producer);
		} else {
			producers.addAll(producerService.getAllProducers());
		}
		
		ShellTable table = new ShellTable();
		table.column("Id");
		table.column("Type");
		table.column("Display Name");
		table.column("Category");
		table.column("Label");

		Col descriptionColumn = new Col("Description")
				.maxSize(40).wrap(true);
		
		table.column(descriptionColumn);
				
		
		for (EntaxyProducer p: producers) {
			List<EntaxyFactory> factories = targetType==null?p.getAllFactories():p.getFactoriesForType(targetType);
			for (EntaxyFactory factory: factories) {
				table.addRow().addContent(
						factory.getId(), factory.getType()
						, factory.getDisplayName()
						, factory.getCategory(), factory.getLabel()
						, factory.getDescription());
			}
		}
		
		table.print(System.out);
		return null;
	}
	
}
