/*-
 * ~~~~~~licensing~~~~~~
 * object-runtime-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.objects.runtime;

import java.util.Map;

public interface EntaxyRuntimeObjectManager {

	public enum ENTAXY_RUNTIME_OBJECT_OPERATION {
		START,
		STOP,
		UNINSTALL
	}
	
	public enum ENTAXY_RUNTIME_OBJECT_OPERATION_RESULT {
		DONE,
		FAILED,
		BLOCKED,
		REQUEST
	}
	
	public interface OperationConfig {
		String getObjectFullId();
		boolean isForced();
		boolean isSoft();
		Map<String, ENTAXY_RUNTIME_OBJECT_OPERATION> getAffected();
	}
	
	public interface OperationResult {
		ENTAXY_RUNTIME_OBJECT_OPERATION_RESULT getResult();
		Map<String, ENTAXY_RUNTIME_OBJECT_OPERATION> getAffected();
		String getMessage();
	}
	
	OperationResult execute(ENTAXY_RUNTIME_OBJECT_OPERATION operation, OperationConfig config);
	
	OperationResult start(String objectFullId);
	OperationResult startForced(String objectFullId);
	OperationResult startSoft(String objectFullId);
	
	OperationResult start(OperationConfig config);

	OperationResult stop(String objectFullId);
	OperationResult stopForced(String objectFullId);
	OperationResult stopSoft(String objectFullId);
	
	OperationResult stop(OperationConfig config);

	OperationResult uninstall(String objectFullId);
	OperationResult uninstallForced(String objectFullId);
	OperationResult uninstallSoft(String objectFullId);
	
	OperationResult uninstall(OperationConfig config);
	
}
