/*-
 * ~~~~~~licensing~~~~~~
 * object-runtime-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.objects.runtime.healthcheck;

import java.util.List;
import java.util.stream.Collectors;

import org.apache.felix.hc.annotation.HealthCheckMBean;
import org.apache.felix.hc.annotation.HealthCheckService;
import org.apache.felix.hc.api.HealthCheck;
import org.apache.felix.hc.api.Result;
import org.apache.felix.hc.api.Result.Status;
import org.apache.felix.hc.api.ResultLog;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;

import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObjectService;

@Component
@HealthCheckService(name = "Ghost Objects HealthCheck", tags = {"runtime", "objects"})
@HealthCheckMBean(name = "Entaxy.Objects.Ghosts")
public class GhostObjectsHealthCheck implements HealthCheck {

	@Reference
	EntaxyRuntimeObjectService objectService;
	
	@Override
	public Result execute() {
		List<EntaxyRuntimeObject> ghosts = objectService.getObjects().stream()
				.filter(obj -> obj.isGhost())
				.collect(Collectors.toList());
		if (ghosts.isEmpty()) {
			return new Result(Status.OK, "");
		}
		ResultLog resultLog = new ResultLog();
		for (EntaxyRuntimeObject obj: ghosts)
			resultLog.add(new ResultLog.Entry(Status.WARN, obj.getObjectFullId()));
		Result result = new Result(resultLog);
		return result;
	}

}
