/*-
 * ~~~~~~licensing~~~~~~
 * object-runtime-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.objects.runtime.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.osgi.framework.Bundle;
import org.osgi.framework.FrameworkUtil;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObjectContainer;

public abstract class EntaxyRuntimeObjectContainerImpl implements EntaxyRuntimeObjectContainer {

	String id;
	String name;
	
	Map<String, EntaxyRuntimeObjectImpl> objects = new HashMap<>();

	List<EntaxyRuntimeObject> objectsCache = new ArrayList<>();
	
	public void bundle(Bundle bundle) {
		setId(bundle.getBundleId() + "");
		setName(bundle.getSymbolicName());
	}

	public void bundle(long bundleId) {
		if (bundleId > 0) {
			Bundle b = FrameworkUtil.getBundle(this.getClass()).getBundleContext().getBundle(bundleId);
			if (b != null)
				bundle(b);
		}
	}
	
	public void add(EntaxyRuntimeObjectImpl object) {
		objects.put(object.getObjectFullId(), object);
		object.setContainerImpl(this);
		objectsCache.add((EntaxyRuntimeObject)object);
	}
	
	public void remove(EntaxyRuntimeObjectImpl object) {
		objects.remove(object.getObjectFullId());
		object.setContainerImpl(null);
		objectsCache.remove((EntaxyRuntimeObject)object);
	}

	@Override
	public String getId() {
		return id;
	}

	@Override
	public String getName() {
		return name;
	}

	public void setId(String id) {
		this.id = id;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Override
	public List<EntaxyRuntimeObject> getObjects() {
		return objectsCache;
	}

	@Override
	public List<EntaxyRuntimeObjectContainer> getParentContainers() {
		final Set<EntaxyRuntimeObjectContainer> containers  = new HashSet<>();
		for (EntaxyRuntimeObjectImpl roi: this.objects.values()) {
			roi.getIncomingRelations().stream()
				.filter(r -> r.isSubordinating())
				.map(r -> r.getSource())
				.map(obj -> EntaxyRuntimeObjectServiceImpl.getInstance().getContainerForObject(obj.getObjectFullId()))
				.filter(cont -> (cont != null))
				.filter(cnt -> !this.id.equals(cnt.getId()))
				.forEach(cont -> containers.add(cont));
		}
		return new ArrayList<>(containers);
	}

	@Override
	public List<EntaxyRuntimeObjectContainer> getChildContainers() {
		final Set<EntaxyRuntimeObjectContainer> containers  = new HashSet<>();
		for (EntaxyRuntimeObjectImpl roi: this.objects.values()) {
			roi.getRelations().stream()
				.filter(r -> r.isSubordinating())
				.map(r -> r.getTarget())
				.map(obj -> EntaxyRuntimeObjectServiceImpl.getInstance().getContainerForObject(obj.getObjectFullId()))
				.filter(cont -> (cont != null))
				.filter(cnt -> !this.id.equals(cnt.getId()))
				.forEach(cont -> containers.add(cont));
		}
		return new ArrayList<>(containers);
	}
	
	@Override
	public List<EntaxyRuntimeObjectContainer> getDependencies() {
		final Set<EntaxyRuntimeObjectContainer> containers  = new HashSet<>();
		for (EntaxyRuntimeObjectImpl roi: this.objects.values()) {
			roi.getRelations().stream()
				.filter(r -> EntaxyObject.OBJECT_RELATION.DEPENDENCY.equals(r.getType()))
				.map(r -> r.getTarget())
				.map(obj -> EntaxyRuntimeObjectServiceImpl.getInstance().getContainerForObject(obj.getObjectFullId()))
				.filter(cont -> (cont != null))
				.filter(cnt -> !this.id.equals(cnt.getId()))
				.forEach(cont -> containers.add(cont));
		}
		return new ArrayList<>(containers);
	}

	@Override
	public List<EntaxyRuntimeObjectContainer> getDependent() {
		final Set<EntaxyRuntimeObjectContainer> containers  = new HashSet<>();
		for (EntaxyRuntimeObjectImpl roi: this.objects.values()) {
			roi.getIncomingRelations().stream()
				.filter(r -> EntaxyObject.OBJECT_RELATION.DEPENDENCY.equals(r.getType()))
				.map(r -> r.getSource())
				.map(obj -> EntaxyRuntimeObjectServiceImpl.getInstance().getContainerForObject(obj.getObjectFullId()))
				.filter(cont -> (cont != null))
				.filter(cnt -> !this.id.equals(cnt.getId()))
				.forEach(cont -> containers.add(cont));
		}
		return new ArrayList<>(containers);
	}

	@Override
	public List<EntaxyRuntimeObjectContainer> getRelatedContainers() {
		final Set<EntaxyRuntimeObjectContainer> containers  = new HashSet<>();
		for (EntaxyRuntimeObjectImpl roi: this.objects.values()) {
			roi.getRelations().stream()
				.filter(r -> EntaxyObject.OBJECT_RELATION.DEPENDENCY.equals(r.getType()))
				.map(r -> r.getTarget())
				.map(obj -> EntaxyRuntimeObjectServiceImpl.getInstance().getContainerForObject(obj.getObjectFullId()))
				.filter(cont -> (cont != null))
				.filter(cnt -> !this.id.equals(cnt.getId()))
				.forEach(cont -> containers.add(cont));
		}
		return new ArrayList<>(containers);
	}

	@Override
	public List<EntaxyRuntimeObjectContainer> getIncomingRelatedContainers() {
		final Set<EntaxyRuntimeObjectContainer> containers  = new HashSet<>();
		for (EntaxyRuntimeObjectImpl roi: this.objects.values()) {
			roi.getIncomingRelations().stream()
				.filter(r -> EntaxyObject.OBJECT_RELATION.DEPENDENCY.equals(r.getType()))
				.map(r -> r.getSource())
				.map(obj -> EntaxyRuntimeObjectServiceImpl.getInstance().getContainerForObject(obj.getObjectFullId()))
				.filter(cont -> (cont != null))
				.filter(cnt -> !this.id.equals(cnt.getId()))
				.forEach(cont -> containers.add(cont));
		}
		return new ArrayList<>(containers);
	}
	
}
