/*-
 * ~~~~~~licensing~~~~~~
 * object-runtime-shell
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.objects.runtime.shell;

import java.util.List;
import java.util.stream.Collectors;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObjectContainer;

@Service
@Command(name = "container-info", scope = EntaxyRuntimeObjectServiceSupport.OBJECTS_SCOPE, description = "Get container info")
public class ContainerInfo extends ContainerAwareCommand implements Action {

	
	@Override
	protected Object doExecute(EntaxyRuntimeObjectContainer container) {
		String message = "Container Info for ".concat(containerName)
			.concat("\n----------------------------------------")
			.concat("\nID: ".concat(container.getId()))
			.concat("\nTYPE: ".concat(container.getType()))
			.concat("\nNAME: ".concat(container.getName()))
			.concat("\nOBJECTS: ")
			.concat(
					container.getObjects().stream()
						.map(obj -> ("\n\t"+obj.getObjectFullId()))
						.collect(Collectors.joining())
			.concat("\nPARENT CONTAINERS: ")
			.concat(
					RuntimeObjectShellUtils.getContainerListPrintString(container.getParentContainers(), "\n\t")
					)
			.concat("\nDEPENDENCIES: ")
			.concat(
					RuntimeObjectShellUtils.getContainerListPrintString(container.getDependencies(), "\n\t")
					)
			.concat("\nCHILD CONTAINERS: ")
			.concat(
					RuntimeObjectShellUtils.getContainerListPrintString(container.getChildContainers(), "\n\t")
					)
			.concat("\nDEPENDENTS: ")
			.concat(
					RuntimeObjectShellUtils.getContainerListPrintString(container.getDependent(), "\n\t")
					)
			
			);
		
		
		// output to console
		System.out.println(message);
		
		return null;
	}

}
