/*-
 * ~~~~~~licensing~~~~~~
 * connection-producing
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.connection.producing.adapters;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.base.generator.template.impl.ProvidedTemplate;
import ru.entaxy.base.generator.template.impl.TemplateMetadata;
import ru.entaxy.base.generator.template.impl.TemplateProvider;
import ru.entaxy.platform.base.support.JSONUtils;
import ru.entaxy.platform.base.support.osgi.tracker.UniformBundleTrackerCustomizer;

public class TrackedAdapterCustomizer extends UniformBundleTrackerCustomizer<List<TrackedAdapter>> {

	private static final Logger log = LoggerFactory.getLogger(TrackedAdapterCustomizer.class);
	
	@Override
	protected List<TrackedAdapter> createManagedObject(Bundle bundle, BundleEvent event,
			Map<String, List<?>> filterResults) {

		URL metadataUrl = bundle.getEntry("/ru/entaxy/adapter/metadata.json");
		log.debug("Json URL is {}", metadataUrl.toString());
		
		try {
		
			String metadata = JSONUtils.getJsonRootObjectString(metadataUrl); 
					
			TrackedAdapter trackedAdapter = new TrackedAdapter();
			trackedAdapter.setMetadata(metadata);
			trackedAdapter.setBundleId(bundle.getBundleId());
			
			String configuration = ConfigConverter.convertFromAdapter(metadata);
			trackedAdapter.setConfiguration(configuration);

			log.debug("TrackedAdapter metadata: \n" + metadata);
			log.debug("TrackedAdapter configuration: \n" + configuration);

			Enumeration<URL> foundEntries = bundle.findEntries("/template/", "*.*", true);
			while (foundEntries.hasMoreElements()) {
				URL entry = foundEntries.nextElement();
				log.debug("FOUND :: " + entry.toString());
				if (entry.toString().endsWith("/")) {
					log.debug(":: .. is folder");
					continue;
				}
				trackedAdapter.getTemplates().add(entry);
			}
			
			return Collections.<TrackedAdapter>singletonList(trackedAdapter);
			
			
		} catch (Exception e) {
			log.error("Failed to read adapter metadata from bundle [" + bundle.getBundleId() + "]", e);
		}
		
		return null;
	}



}
