<?xml version="1.0" encoding="UTF-8"?>
<!--
  ~~~~~~licensing~~~~~~
  route-producing
  ==========
  Copyright (C) 2020 - 2023 EmDev LLC
  ==========
  You may not use this file except in accordance with the License Terms of the Copyright
  Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
  rights to the Software and any copies are the property of the Copyright Holder. Unless
  it is explicitly allowed the Copyright Holder, the User is prohibited from using the
  Software for commercial purposes to provide services to third parties.
  
  The Copyright Holder hereby declares that the Software is provided on an "AS IS".
  Under no circumstances does the Copyright Holder guarantee or promise that the
  Software provided by him will be suitable or not suitable for the specific purposes
  of the User, that the Software will meet all commercial and personal subjective
  expectations of the User, that the Software will work properly, without technical
  errors, quickly and uninterruptedly.
  
  Under no circumstances shall the Copyright Holder or its Affiliates is not liable
  to the User for any direct or indirect losses of the User, his expenses or actual
  damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
  or damage to data, property, etc.
  ~~~~~~/licensing~~~~~~
  -->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
    <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>

    <!-- xsl:param name="profileName" select="'common'" / -->

    <xsl:template match="/">
        <xsl:apply-templates select="child::node()"/>
    </xsl:template>

    <xsl:template match="//set-destination" exclude-result-prefixes="xsl fn xs">
        <xsl:variable name="full" select="./text()"/>
        <xsl:variable name="name">
            <xsl:choose>
                <xsl:when test="contains($full, ':')">
                    <xsl:value-of select="substring-before($full, ':')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$full"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="type" select="substring-after($full, ':')"/>
        <xsl:comment>generated from 'set-destination'</xsl:comment>
        <setHeader name="ENTAXY_Destination">
            <constant>
                <xsl:value-of select="$name"/>
            </constant>
        </setHeader>
        <xsl:if test="string-length($type) > 0">
            <setHeader name="ENTAXY_DestinationType">
                <constant>
                    <xsl:value-of select="$type"/>
                </constant>
            </setHeader>
        </xsl:if>
    </xsl:template>

    <xsl:template match="//entaxy-trace" exclude-result-prefixes="xsl fn xs">
        <xsl:choose>
            <xsl:when test="@reset eq 'true'">
                <setHeader name="ENTAXY_TRACE">
                    <constant></constant>
                </setHeader>
            </xsl:when>
        </xsl:choose>
        <xsl:choose>
            <xsl:when test="child::element()[1]">
                <choice>
                    <when>
                        <simple>${headers.ENTAXY_TRACE} != null</simple>
                        <setHeader name="ENTAXY_TRACE">
                            <simple>${headers.ENTAXY_TRACE} -&gt;</simple>
                        </setHeader>
                    </when>
                </choice>
                <setHeader name="NTX_TRACE_RECORD">
                    <xsl:apply-templates select="child::element()[1]"/>
                </setHeader>
                <choice>
                    <when>
                        <simple>${headers.ENTAXY_TRACE} == null</simple>
                        <setHeader name="ENTAXY_TRACE">
                            <simple>[${headers.NTX_TRACE_RECORD}]</simple>
                        </setHeader>
                    </when>
                    <otherwise>
                        <setHeader name="ENTAXY_TRACE">
                            <simple>${headers.ENTAXY_TRACE}[${headers.NTX_TRACE_RECORD}]</simple>
                        </setHeader>
                    </otherwise>
                </choice>
                <removeHeader headerName="NTX_TRACE_RECORD"/>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="//respond" exclude-result-prefixes="xsl fn xs">
        <xsl:variable name="respondNow">
            <xsl:choose>
                <xsl:when test="@now">
                    <xsl:value-of select="@now"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>true</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="respondContinue">
            <xsl:choose>
                <xsl:when test="@continue">
                    <xsl:value-of select="@continue"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>true</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="respondContinueAsync">
            <xsl:choose>
                <xsl:when test="@continueAsync">
                    <xsl:value-of select="@continueAsync"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>true</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <setHeader name="NTX_RESPOND_exec">
            <constant>true</constant>
        </setHeader>
        <setHeader name="NTX_RESPOND_now">
            <constant>
                <xsl:value-of select="$respondNow"/>
            </constant>
        </setHeader>
        <setHeader name="NTX_RESPOND_continue">
            <constant>
                <xsl:value-of select="$respondContinue"/>
            </constant>
        </setHeader>
        <setHeader name="NTX_RESPOND_continueAsync">
            <constant>
                <xsl:value-of select="$respondContinueAsync"/>
            </constant>
        </setHeader>
    </xsl:template>

    <xsl:template match="//route//to-connection" exclude-result-prefixes="xsl fn xs">
        <reference interface="org.apache.camel.Component">
            <xsl:attribute name="id" select="@name"/>
            <xsl:attribute name="filter" select="concat('(connection.name=', @name, ')')"/>
        </reference>
        <xsl:variable name="dir">
            <xsl:choose>
                <xsl:when test="@directory">
                    <xsl:value-of select="@directory"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text></xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <to>
            <xsl:variable name="queryString">
                <xsl:for-each select="./element()">
                    <xsl:call-template name="convert-to-query"/>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="resultQS">
                <xsl:choose>
                    <xsl:when test="string-length($queryString) > 0">
                        <xsl:value-of select="concat('?', $queryString)"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:attribute name="uri" select="concat(@name,':' ,$dir , $resultQS)"/>
        </to>
    </xsl:template>

    <!--	<prefer-connector>-->
    <!--		<protocol>file</protocol>-->
    <!--		<classifier></classifier>-->
    <!--		<type></type>-->
    <!--	</prefer-connector>-->
    <xsl:template match="//route//prefer-connector" exclude-result-prefixes="xsl fn xs">
        <xsl:variable name="nodes" select="child::element()"/>
        <setHeader name="NTX_preferred_connector">
            <constant>
                {
                <xsl:for-each select="$nodes">
                    "<xsl:value-of select="node-name(.)"/>": "<xsl:value-of select="."/>"
                    <xsl:variable name="i" select="position()"/>
                    <xsl:choose>
                        <xsl:when test="$nodes[$i + 1]!=''">
                            ,
                        </xsl:when>
                    </xsl:choose>
                </xsl:for-each>
                }
            </constant>
        </setHeader>
    </xsl:template>

    <xsl:template match="//route//xslt" exclude-result-prefixes="xsl fn xs">
        <reference id="xslUrlResolver" interface="javax.xml.transform.URIResolver" filter="(type=xslt)"/>
        <to>
            <xsl:variable name="resource">
                <xsl:value-of select="@resource"/>
            </xsl:variable>
            <xsl:variable name="params">
                <xsl:value-of select="@params"/>
            </xsl:variable>
            <xsl:variable name="resultResource">
                <xsl:choose>
                    <xsl:when test="contains($resource, ':')">
                        <xsl:value-of select="translate($resource,':','/')"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$resource"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="resultParams">
                <xsl:choose>
                    <xsl:when test="string-length($params) > 0">
                        <xsl:value-of select="concat('&amp;', $params)"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:attribute name="uri"
                           select="concat('xslt:entaxy-resource://', $resultResource, '?uriResolver=#xslUrlResolver', $resultParams)"/>
        </to>
    </xsl:template>

    <xsl:template name="convert-to-query" exclude-result-prefixes="xsl fn xs">
        <xsl:value-of select="concat(node-name(.), '=', text())"/>
        <xsl:if test="./following-sibling::element()">
            <xsl:text>&amp;</xsl:text>
        </xsl:if>
    </xsl:template>

    <xsl:template match="//default-destination" exclude-result-prefixes="xsl fn xs">
        <xsl:variable name="full" select="./text()"/>
        <xsl:variable name="name">
            <xsl:choose>
                <xsl:when test="contains($full, ':')">
                    <xsl:value-of select="substring-before($full, ':')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$full"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="type" select="substring-after($full, ':')"/>
        <xsl:comment>generated from 'default-destination'</xsl:comment>
        <choice>
            <when>
                <simple>${headers.ENTAXY_Destination} == null</simple>
                <removeHeader headerName="ENTAXY_DestinationType"/>
                <setHeader name="ENTAXY_Destination">
                    <constant>
                        <xsl:value-of select="$name"/>
                    </constant>
                </setHeader>
                <xsl:if test="string-length($type) > 0">
                    <setHeader name="ENTAXY_DestinationType">
                        <constant>
                            <xsl:value-of select="$type"/>
                        </constant>
                    </setHeader>
                </xsl:if>
            </when>
        </choice>
    </xsl:template>

    <xsl:template match="//next-route" exclude-result-prefixes="xsl fn xs">
        <xsl:variable name="local">
            <xsl:choose>
                <xsl:when test="./@local">
                    <xsl:value-of select="./@local"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>false</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="async">
            <xsl:choose>
                <xsl:when test="./@async">
                    <xsl:value-of select="./@async"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>false</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="component">
            <xsl:choose>
                <xsl:when test="$async = 'false'">
                    <xsl:text>direct</xsl:text>
                    <xsl:if test="$local = 'false'">
                        <xsl:text>-vm</xsl:text>
                    </xsl:if>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:choose>
                        <xsl:when test="$local = 'true'">
                            <xsl:text>seda</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>vm</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="name">
            <xsl:value-of select="./text()"/>
        </xsl:variable>
        <xsl:comment>generated from 'next-route'</xsl:comment>
        <setHeader name="NTX_NextPoint">
            <constant>
                <xsl:value-of select="$component"/><xsl:text>:</xsl:text><xsl:value-of select="$name"/>
            </constant>
        </setHeader>
    </xsl:template>


    <xsl:template name="default-copy" match="node()|@*">
        <xsl:copy>
            <xsl:apply-templates select="node()|@*"/>
        </xsl:copy>
    </xsl:template>

</xsl:stylesheet>
