/*-
 * ~~~~~~licensing~~~~~~
 * connector-producing
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.objects.service.producing;

import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.core.producer.api.EntaxyProducer;
import ru.entaxy.platform.core.producer.impl.CommonObjectProducer;
import ru.entaxy.platform.core.producer.impl.EntaxyProducerInfo;
import ru.entaxy.platform.core.producer.wrapper.AbstractFactoryWrapper;
import ru.entaxy.platform.core.producer.wrapper.DefaultFactoryWrapper;

@Component(service = {EntaxyProducer.class}, immediate = true)
@EntaxyProducerInfo(supportedTypes = {ServiceProducer.OBJECT_TYPES_SERVICE, ServiceProducer.OBJECT_TYPES_SERVICE_SUBTYPES})
public class ServiceProducer extends CommonObjectProducer implements EntaxyProducer {

	public static final String OBJECT_TYPES_SERVICE = "entaxy.runtime.service";
	public static final String OBJECT_TYPES_SERVICE_RESOURCES = "entaxy.runtime.service.support";
	public static final String OBJECT_TYPES_SERVICE_SUBTYPES = "entaxy.runtime.service.*";
	
	@Reference(bind = "addFactory", unbind = "removeFactory", cardinality = ReferenceCardinality.MULTIPLE
			, collectionType = CollectionType.SERVICE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void addFactory(EntaxyFactory factory) {
		doAddFactory(factory);
	}
	
	// WE MUST DECLARE IT for @Reference annotation processor
	@Override
	public void removeFactory(EntaxyFactory factory) {
		super.removeFactory(factory);
	}

	@Override
	protected AbstractFactoryWrapper doAddFactory(EntaxyFactory factory) {
		AbstractFactoryWrapper wrapper = super.doAddFactory(factory, DefaultFactoryWrapper.class);
		if (wrapper != null)
			wrapper.setGenerationProcessor(this);
		return wrapper;
	}
	
}
