<?xml version="1.0" encoding="UTF-8"?>
<!--
  ~~~~~~licensing~~~~~~
  profile-producing
  ==========
  Copyright (C) 2020 - 2023 EmDev LLC
  ==========
  You may not use this file except in accordance with the License Terms of the Copyright
  Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
  rights to the Software and any copies are the property of the Copyright Holder. Unless
  it is explicitly allowed the Copyright Holder, the User is prohibited from using the
  Software for commercial purposes to provide services to third parties.
  
  The Copyright Holder hereby declares that the Software is provided on an "AS IS".
  Under no circumstances does the Copyright Holder guarantee or promise that the
  Software provided by him will be suitable or not suitable for the specific purposes
  of the User, that the Software will meet all commercial and personal subjective
  expectations of the User, that the Software will work properly, without technical
  errors, quickly and uninterruptedly.
  
  Under no circumstances shall the Copyright Holder or its Affiliates is not liable
  to the User for any direct or indirect losses of the User, his expenses or actual
  damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
  or damage to data, property, etc.
  ~~~~~~/licensing~~~~~~
  -->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions" xmlns:local="http://ru.entaxy">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" />
	
	<xsl:param name="serviceId" select="'service-2'" />
	
	<xsl:template match="/">
		<xsl:apply-templates select="child::node()" />
	</xsl:template>

	<xsl:template match="//route/operation-router/operation/connector" exclude-result-prefixes="xsl fn xs local">
		<log loggingLevel="INFO">
			<xsl:attribute name="message" select="concat('Sending to connector: ', @direction)"></xsl:attribute>
		</log>
		<to>
			<xsl:attribute name="uri" select="concat('bean:', $serviceId, '-connector-registry?method=', @direction, 'ConnectorFind')"></xsl:attribute>
		</to>
		<choice>
        	<when>
				<simple>${headers.NTX_targetConnector} != null</simple>
				<setProperty name="NTX_TMP_targetConnector">
					<simple>${headers.NTX_targetConnector}</simple>
				</setProperty>
				<removeHeader headerName="NTX_targetConnector" />
				<toD>
					<xsl:attribute name="uri" select="concat('${exchangeProperty.NTX_TMP_targetConnector}', '')" />
				</toD>
			</when>
			<otherwise>
				<throwException exceptionType="ru.entaxy.platform.services.runtime.exceptions.ConnectorNotFound">
					<xsl:attribute name="message" 
						select="concat('Connector not found: service=', $serviceId, '; profile=${headers[X-SystemName]}; direction=', @direction)">
					</xsl:attribute>
				</throwException>
			</otherwise>
		</choice>	
	</xsl:template>

	<xsl:template match="//route//to-connection" exclude-result-prefixes="xsl fn xs local">
		<reference interface="org.apache.camel.Component">
			<xsl:attribute name="id" select="@name" />
			<xsl:attribute name="filter" select="concat('(connection.name=', @name, ')')" />
		</reference>
		<xsl:variable name="dir">
			<xsl:choose>
				<xsl:when test="@directory"><xsl:value-of select="@directory" /></xsl:when>
				<xsl:otherwise><xsl:text></xsl:text></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<to>
			<xsl:variable name="queryString">
				<xsl:for-each select="./element()">
					<xsl:call-template name="convert-to-query" />
				</xsl:for-each>
			</xsl:variable>
			<xsl:variable name="resultQS">
				<xsl:choose>
					<xsl:when test="string-length($queryString) > 0">
						<xsl:value-of select="concat('?', $queryString)" />
					</xsl:when>
					<xsl:otherwise><xsl:text /></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:attribute name="uri" select="concat(@name,':' ,$dir , $resultQS)" />
		</to>
	</xsl:template>
	
	<xsl:template name="convert-to-query" exclude-result-prefixes="xsl fn xs local">	
		<xsl:value-of select="concat(node-name(.), '=', ./text())" />
		<xsl:if test="./following-sibling::element()"><xsl:text>&amp;</xsl:text></xsl:if>
	</xsl:template>
	
	<xsl:template name="default-copy" match="node()|@*">
		<xsl:copy>
			<xsl:apply-templates select="node()|@*" />
		</xsl:copy>
	</xsl:template>	
		
</xsl:stylesheet>
