/*-
 * ~~~~~~licensing~~~~~~
 * service-runtime
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.services.runtime.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.camel.Exchange;

import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.base.support.osgi.service.CommonNamedReferenceListener;
import ru.entaxy.platform.services.runtime.EntaxyServiceConnectorInfo;

public class EntaxyServiceConnectorRegistry extends CommonNamedReferenceListener<EntaxyServiceConnectorInfo> {

	protected List<String> profileNameHeaders = new ArrayList<>();
	
	@Override
	protected String getObjectName(EntaxyServiceConnectorInfo connectorInfo) {
		return String.format("%s-%s", connectorInfo.getDirection(), connectorInfo.getProfileId());
	}

	public void inConnectorFind(Exchange exchange) {
		String profileName = getProfileNameFromExchange(exchange);
		if (!CommonUtils.isValid(profileName))
			return;
		String key = "in-" + profileName;
		if (!isRegistered(key))
			return;
		EntaxyServiceConnectorInfo connectorInfo = getReference(key);
		exchange.getIn().setHeader("NTX_targetConnector", connectorInfo.getEndpoint());
	}

	protected String getProfileNameFromExchange(Exchange exchange) {
		String result = null;
		Map<String, Object> headers = exchange.getIn().getHeaders();
		for (String headerName: profileNameHeaders) {
			if (!headers.containsKey(headerName))
				continue;
			Object obj = headers.get(headerName);
			if (obj == null)
				continue;
			result = obj.toString();
			break;
		}
		return result;
	}
	
	public List<String> getProfileNameHeaders() {
		return profileNameHeaders;
	}

	public void setProfileNameHeaders(List<String> profileNameHeaders) {
		this.profileNameHeaders = profileNameHeaders;
	}
	
	
}
