<!--
  ~~~~~~licensing~~~~~~
  support
  ==========
  Copyright (C) 2020 - 2023 EmDev LLC
  ==========
  You may not use this file except in accordance with the License Terms of the Copyright
  Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
  rights to the Software and any copies are the property of the Copyright Holder. Unless
  it is explicitly allowed the Copyright Holder, the User is prohibited from using the
  Software for commercial purposes to provide services to third parties.
  
  The Copyright Holder hereby declares that the Software is provided on an "AS IS".
  Under no circumstances does the Copyright Holder guarantee or promise that the
  Software provided by him will be suitable or not suitable for the specific purposes
  of the User, that the Software will meet all commercial and personal subjective
  expectations of the User, that the Software will work properly, without technical
  errors, quickly and uninterruptedly.
  
  Under no circumstances shall the Copyright Holder or its Affiliates is not liable
  to the User for any direct or indirect losses of the User, his expenses or actual
  damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
  or damage to data, property, etc.
  ~~~~~~/licensing~~~~~~
  -->
<xsl:stylesheet version="2.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:soap="http://www.entaxy.ru/ExchangeTypes/1.0"
                xmlns="http://www.entaxy.ru/ExchangeTypes/1.0"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:esb_internal="http://www.entaxy.ru/connector/universal/internal"
                exclude-result-prefixes="esb_internal">
    <xsl:import href="address.xsl"/>
    <xsl:output omit-xml-declaration="yes" indent="yes"/>

    <xsl:param name="ENTAXY_AcknowledgeMsgID"/>
    <xsl:param name="ENTAXY_MessageType"/>
    <xsl:param name="ENTAXY_MessageUUID" select="'00000000-0000-0000-0000-000000000000'"/>
    <xsl:param name="ENTAXY_Source"/>
    <xsl:param name="ENTAXY_SourceType"/>
    <xsl:param name="ENTAXY_Destination"/>
    <xsl:param name="ENTAXY_DestinationType"/>
    <xsl:param name="NTX_CustomHeaders" select="node()"/>
    <xsl:param name="ENTAXY_Priority" select="'4'"/>
    <xsl:param name="ENTAXY_ExchangePattern"/>
    <xsl:param name="ENTAXY_ContentType" select="'application/xml'"/>
    <xsl:param name="CamelJms_IntContentXsiType"/>

    <xsl:template match="@* | node()">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template name="packet">
        <xsl:if test="$ENTAXY_MessageUUID">
            <packet>
                <header>
                    <xsl:if test="$ENTAXY_AcknowledgeMsgID">
                        <transportUUID>
                            <xsl:value-of select="$ENTAXY_AcknowledgeMsgID"/>
                        </transportUUID>
                    </xsl:if>
                    <xsl:if test="$ENTAXY_Destination">
                        <xsl:call-template name="address">
                            <xsl:with-param name="id" select="$ENTAXY_Destination"/>
                            <xsl:with-param name="type" select="$ENTAXY_DestinationType"/>
                        </xsl:call-template>
                    </xsl:if>
                </header>
                <message>
                    <messageUUID>
                        <xsl:value-of select="$ENTAXY_MessageUUID"/>
                    </messageUUID>
                    <xsl:if test="$ENTAXY_MessageType">
                        <messageType>
                            <xsl:value-of select="$ENTAXY_MessageType"/>
                        </messageType>
                    </xsl:if>
                    <xsl:if test="$ENTAXY_Priority">
                        <priority>
                            <xsl:value-of select="$ENTAXY_Priority"/>
                        </priority>
                    </xsl:if>
                    <xsl:if test="$ENTAXY_ExchangePattern">
                        <exchangePattern>
                            <xsl:value-of select="$ENTAXY_ExchangePattern"/>
                        </exchangePattern>
                    </xsl:if>
                    <xsl:if test="$ENTAXY_Source">
                        <xsl:call-template name="address">
                            <xsl:with-param name="wrapper" select="'source'"/>
                            <xsl:with-param name="id" select="$ENTAXY_Source"/>
                            <xsl:with-param name="type" select="$ENTAXY_SourceType"/>
                        </xsl:call-template>
                    </xsl:if>
                    <xsl:if test="$NTX_CustomHeaders">
                        <xsl:copy>
                            <xsl:apply-templates select="$NTX_CustomHeaders/soap:customHeader"/>
                        </xsl:copy>
                    </xsl:if>
                    <xsl:if test="$ENTAXY_ContentType">
                        <contentType>
                            <xsl:value-of select="$ENTAXY_ContentType"/>
                        </contentType>
                    </xsl:if>
                    <xsl:if test="/body and not(/body/esb_internal:empty)">
                        <content>
                            <xsl:if test="$CamelJms_IntContentXsiType">
                                <xsl:attribute name="xsi:type">
                                    <xsl:value-of select="$CamelJms_IntContentXsiType"/>
                                </xsl:attribute>
                            </xsl:if>
                            <xsl:if test="not(/body/esb_internal:emptyContent)">
                                <xsl:copy>
                                    <xsl:apply-templates select="/body/node()"/>
                                </xsl:copy>
                            </xsl:if>
                        </content>
                    </xsl:if>
                </message>
            </packet>
        </xsl:if>
    </xsl:template>
</xsl:stylesheet>
