/*-
 * ~~~~~~licensing~~~~~~
 * htpasswd
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.auth.basic.htpasswd;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import ru.entaxy.esb.system.auth.basic.htpasswd.entity.Htpasswd;
import ru.entaxy.esb.system.auth.basic.jpa.api.entity.BasicAuthAccount;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.security.NoSuchAlgorithmException;
import java.util.List;

public class HtpasswdGenerator {

    private static final Log LOG = LogFactory.getLog(HtpasswdGenerator.class);

    private String checkSumFileName;

    public Htpasswd htpasswd;

    public void generateHtpasswd(List<BasicAuthAccount> accounts, String salt) throws IOException, NoSuchAlgorithmException {
        htpasswd.setMasterSalt(salt);
        htpasswd.prepare(accounts);
        createFile();
    }

    private void createFile() throws IOException {
        String content = htpasswd.toString();
        LOG.trace("HTTPASSWD " + content);
        String storeFolder = htpasswd.getDirectory();
        File folder = new File(storeFolder);
        folder.mkdirs();

        File htpasswdFile = new File(folder.getAbsolutePath() + File.separator + htpasswd.getFileName());

        Path path = Paths.get(htpasswdFile.getAbsolutePath());
        Files.write(path, content.getBytes());

        String checkSum = calculateCheckSum(path);
        File checkSumFile = new File(folder.getAbsolutePath() + File.separator + checkSumFileName);
        path = Paths.get(checkSumFile.getAbsolutePath());
        Files.write(path, checkSum.getBytes());
    }

    private String calculateCheckSum(Path path) throws IOException {
        String md5;
        try (InputStream is = Files.newInputStream(path)) {
            md5 = DigestUtils.md5Hex(is);
        }
        return md5;
    }

    public Htpasswd getHtpasswd() {
        return htpasswd;
    }

    public void setHtpasswd(Htpasswd htpasswd) {
        this.htpasswd = htpasswd;
    }

    public String getCheckSumFileName() {
        return checkSumFileName;
    }

    public void setCheckSumFileName(String checkSumFileName) {
        this.checkSumFileName = checkSumFileName;
    }

}
