/*-
 * ~~~~~~licensing~~~~~~
 * htpasswd
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.auth.basic.htpasswd.entity;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import ru.entaxy.esb.system.auth.basic.jpa.api.entity.BasicAuthAccount;

import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

public class Htpasswd {

    private static final Log LOG = LogFactory.getLog(Htpasswd.class);
    private String directory;
    private String fileName;
    private String masterSalt = null;
    private final List<HtpasswdEntry> entries = new ArrayList<>();

    public Htpasswd() {
    }

    public void prepare(List<BasicAuthAccount> accounts) throws NoSuchAlgorithmException {
        if (masterSalt == null || masterSalt.isEmpty()) {
            throw new IllegalArgumentException("masterSalt not setted!");
        }
        if (accounts != null && accounts.size() > 0) {
            entries.clear();
            for (BasicAuthAccount account : accounts) {
                entries.add(new HtpasswdEntry(
                        account.getLogin(),
                        account.getPasswordHash(),
                        masterSalt,
                        account.getEncryptionAlgorithm().getAlgorithmName()));
            }
        }
    }

    public void addString(String login, String passwordHash, String encryptionAlgorithm) throws NoSuchAlgorithmException {
        entries.add(new HtpasswdEntry(
                login,
                passwordHash,
                masterSalt,
                encryptionAlgorithm));
    }

    public String getDirectory() {
        return directory;
    }

    public void setDirectory(String directory) {
        this.directory = directory;
    }

    public String getFileName() {
        return fileName;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public String getMasterSalt() {
        return masterSalt;
    }

    public void setMasterSalt(String masterSalt) {
        this.masterSalt = masterSalt;
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        for (HtpasswdEntry entry : entries) {
            builder.append(entry.toString());
        }
        return builder.toString();
    }

}
