/*-
 * ~~~~~~licensing~~~~~~
 * system-commons
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.common.aggregation;

import org.apache.camel.AggregationStrategy;
import org.apache.camel.Exchange;

public class TimeoutAwareAggregationStrategyImpl implements AggregationStrategy {

    private static final String ACK_MESSAGE_HEADER = "NTX_AckMessage";

    private final String nameHeaderAcknowledge;

    public TimeoutAwareAggregationStrategyImpl(String nameHeaderAcknowledge) {
        this.nameHeaderAcknowledge = nameHeaderAcknowledge;
    }

    @Override
    public Exchange aggregate(Exchange oldExchange, Exchange newExchange) {
        if (oldExchange == null) {
            return newExchange;
        } else {
            oldExchange.getMessage().setHeader(nameHeaderAcknowledge, checkOnCorrectPair(oldExchange, newExchange));
            return oldExchange;
        }
    }

    private boolean checkOnCorrectPair(Exchange oldExchange, Exchange newExchange) {
        return (oldExchange != null && oldExchange.getMessage().getHeader(ACK_MESSAGE_HEADER) == null &&
                (newExchange == null || newExchange.getMessage().getHeader(ACK_MESSAGE_HEADER) == null));
    }

    @Override
    public void timeout(Exchange oldExchange, int index, int total, long timeout) {
        oldExchange.getMessage().setHeader(nameHeaderAcknowledge,
                (oldExchange.getMessage().getHeader(ACK_MESSAGE_HEADER) == null));
    }
}
