/*-
 * ~~~~~~licensing~~~~~~
 * system-commons
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.common.interceptor;

import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.binding.soap.interceptor.AbstractSoapInterceptor;
import org.apache.cxf.headers.Header;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.jaxb.JAXBDataBinding;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.Phase;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import java.util.List;
import java.util.Map;

public class SoapHeaderInterceptor extends AbstractSoapInterceptor {

    private static final Logger log = LoggerFactory.getLogger(SoapHeaderInterceptor.class);

    private static final String HEADER_USER_LOGIN = "X-ForwardedUser";
    private static final String HEADER_CREATED_BY = "createdBy";
    private String namespaceUri = "";

    public SoapHeaderInterceptor() {
        super(Phase.READ);
    }

    public void handleMessage(SoapMessage message) throws Fault {
        try {
            Map<String, List<String>> headers = (Map<String, List<String>>) message.get(Message.PROTOCOL_HEADERS);
            List<Header> soapHeaders = message.getHeaders();
            soapHeaders.add(new Header(new QName(namespaceUri, HEADER_CREATED_BY),
                    getSystemName(headers), new JAXBDataBinding(String.class)));
            message.put(Header.HEADER_LIST, soapHeaders);
        } catch (JAXBException e) {
            log.error("Error", e);
            throw new Fault(e);
        }
    }

    private String getSystemName(Map<String, List<String>> headers) {
        List<String> list = headers.get(HEADER_USER_LOGIN);
        return list != null && !list.isEmpty() ? headers.get(HEADER_USER_LOGIN).get(0) : null;
    }

    public void setNamespaceUri(String namespaceUri) {
        this.namespaceUri = namespaceUri;
    }
}
