/*-
 * ~~~~~~licensing~~~~~~
 * system-commons
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.common.osgi.impl;

import org.osgi.framework.ServiceReference;
import ru.entaxy.esb.system.common.osgi.NamedReferenceListener;

import java.util.*;

public abstract class CommonNamedReferenceListener<T> implements NamedReferenceListener<T> {

    protected Map<String, T> registeredReferences = new HashMap<String, T>();

    public void register(T service) throws Exception {
        if (service instanceof ServiceReference)
            return;
        String key = getObjectName(service);
        registeredReferences.put(key, service);
        doAfterRegister(service);
    }

    protected void doAfterRegister(T service) throws Exception {

    }

    protected void doBeforeUnregister(T service) throws Exception {

    }

    public void unregister(T service) throws Exception {
        if (service == null || service instanceof ServiceReference)
            return;
        doBeforeUnregister(service);
        registeredReferences.remove(getObjectName(service));
    }

    @Override
    public List<String> getReferenceNames() {
        List<String> result = new ArrayList<String>(registeredReferences.keySet());
        Collections.sort(result);
        return result;
    }

    @Override
    public T getReference(String referenceName) {
        return registeredReferences.get(referenceName);
    }

    public boolean isRegistered(String referenceName) {
        return registeredReferences.containsKey(referenceName);
    }

    protected abstract String getObjectName(T object);
}
