/*-
 * ~~~~~~licensing~~~~~~
 * events-handler
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.core.events.handler.util;

import com.google.gson.ExclusionStrategy;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import org.apache.camel.component.gson.GsonDataFormat;
import ru.entaxy.esb.system.core.events.jpa.entity.SystemSubscription;

import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.ArrayList;

public class GsonDataFormatInitializer extends GsonDataFormat {

    private Gson gson;

    public GsonDataFormatInitializer() {
        super();
        Type type = new TypeToken<ArrayList<SystemSubscription>>() {
        }.getType();
        setUnmarshalGenericType(type);
    }

    @Override
    protected void doStart() throws Exception {
        if (gson == null) {
            GsonBuilder builder = new GsonBuilder();
            if (getExclusionStrategies() != null && !getExclusionStrategies().isEmpty()) {
                ExclusionStrategy[] strategies = getExclusionStrategies().toArray(new ExclusionStrategy[getExclusionStrategies().size()]);
                builder.setExclusionStrategies(strategies);
            }
//            if (longSerializationPolicy != null) {
//                builder.setLongSerializationPolicy(longSerializationPolicy);
//            }
            if (getFieldNamingPolicy() != null) {
                builder.setFieldNamingPolicy(getFieldNamingPolicy());
            }
//            if (fieldNamingStrategy != null) {
//                builder.setFieldNamingStrategy(fieldNamingStrategy);
//            }
//            if (serializeNulls) {
//                builder.serializeNulls();
//            }
//            if (prettyPrint) {
//                builder.setPrettyPrinting();
//            }
//            if (dateFormatPattern != null) {
//                builder.setDateFormat(dateFormatPattern);
//            }

            builder.registerTypeAdapterFactory(HibernateProxyTypeAdapter.FACTORY);

            gson = builder.create();

            Field f1 = this.getClass().getSuperclass().getDeclaredField("gson");
            f1.setAccessible(true);
            f1.set(this, gson);
        }
    }

}
