/*-
 * ~~~~~~licensing~~~~~~
 * permission-component
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.core.permission.component.operation;

import org.apache.camel.Exchange;
import ru.entaxy.esb.system.core.permission.common.PermissionConstants;
import ru.entaxy.esb.system.core.permission.component.PermissionEndpoint;
import ru.entaxy.esb.system.jpa.entity.System;

import javax.persistence.NoResultException;

public class CheckSystemAccess extends BaseOperation {

    @Override
    protected void doProcess(Exchange exchange, PermissionEndpoint endpoint) throws Exception {
        validateParams(String.valueOf(endpoint.getObjectId()), endpoint.getSubjectId());
        exchange.getIn().setBody(checkSystemAccessProc(endpoint), Boolean.class);
    }

    protected boolean checkSystemAccessProc(PermissionEndpoint endpoint) {
        System system = getSystem(endpoint.getSubjectId());
        boolean result = false;
        if (system != null) {
            result = getPermissionService().existByAllParameters(endpoint.getObjectId(), PermissionConstants.TYPE_SYSTEM,
                    String.valueOf(system.getId()), PermissionConstants.TYPE_SYSTEM, endpoint.getAction());
        }
        return result;
    }

    protected System getSystem(String subject) {
        try {
            long systemId = Long.valueOf(subject);
            return getSystemService().get(systemId);
        } catch (Exception e) {
            /*keep silence*/
        }
        System system = null;
        try {
            system = getSystemService().getByName(subject);
            return system;
        } catch (NoResultException e) {
            /*keep silence*/
        }

        try {
            system = getSystemService().getByUuid(subject);
        } catch (NoResultException e) {
            /*keep silence*/
        }

        return system;
    }

}
