/*-
 * ~~~~~~licensing~~~~~~
 * connector-api
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.connector.entity;

import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;

import javax.persistence.*;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@NamedEntityGraph(
        name = "connector-entity-graph",
        attributeNodes = {
                @NamedAttributeNode("connectorParams"),
                @NamedAttributeNode("bundleEntity")
        }
)
@Entity
@Table(name = "connector")
public class Connector {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private long id;
    @Column(name = "name")
    private String name;
    @Column(name = "type")
    private String type;
    @Column(name = "version")
    private String version;
    @Column(name = "create_date")
    private Date createDate;
    @Column(name = "edit_date")
    private Date editDate;
    @Column(name = "created_by")
    private String createdBy;
    @Column(name = "edited_by")
    private String editedBy;
    @Column(name = "template_name")
    private String templateName;
    @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.LAZY)
    @JoinColumn(name = "connector_id")
    private List<ConnectorParam> connectorParams;
    @OneToOne(cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.LAZY)
    @JoinColumn(name = "bundle_id")
    private BundleEntity bundleEntity;
    @Column(name = "system_id")
    private long systemId;

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public Date getCreateDate() {
        return createDate;
    }

    public void setCreateDate(Date createDate) {
        this.createDate = createDate;
    }

    public Date getEditDate() {
        return editDate;
    }

    public void setEditDate(Date editDate) {
        this.editDate = editDate;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    public String getEditedBy() {
        return editedBy;
    }

    public void setEditedBy(String editedBy) {
        this.editedBy = editedBy;
    }

    public String getTemplateName() {
        return templateName;
    }

    public void setTemplateName(String templateName) {
        this.templateName = templateName;
    }

    public List<ConnectorParam> getConnectorParams() {
        return connectorParams;
    }

    public void setConnectorParams(List<ConnectorParam> connectorParams) {
        this.connectorParams = connectorParams;
    }

    public BundleEntity getBundleEntity() {
        return bundleEntity;
    }

    public void setBundleEntity(BundleEntity bundleEntity) {
        this.bundleEntity = bundleEntity;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public long getSystemId() {
		return systemId;
	}

	public void setSystemId(long systemId) {
		this.systemId = systemId;
	}

	@Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Connector connector = (Connector) o;
        return id == connector.id &&
                Objects.equals(name, connector.name) &&
                Objects.equals(type, connector.type) &&
                Objects.equals(createDate, connector.createDate) &&
                Objects.equals(editDate, connector.editDate) &&
                Objects.equals(createdBy, connector.createdBy) &&
                Objects.equals(editedBy, connector.editedBy) &&
                Objects.equals(templateName, connector.templateName) &&
                Objects.equals(connectorParams, connector.connectorParams);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name, type, createDate, editDate, createdBy, editedBy,
                templateName, connectorParams);
    }

    @Override
    public String toString() {
        return "Connector{" +
                "id=" + id +
                ", name='" + name + '\'' +
                ", type='" + type + '\'' +
                ", createDate=" + createDate +
                ", editDate=" + editDate +
                ", createdBy='" + createdBy + '\'' +
                ", editedBy='" + editedBy + '\'' +
                ", templateName='" + templateName +
                '}';
    }
}
