/*-
 * ~~~~~~licensing~~~~~~
 * system-group-component
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.groups.component;

import org.apache.camel.Exchange;
import org.apache.camel.support.DefaultProducer;
import org.osgi.framework.BundleContext;
import org.osgi.framework.FrameworkUtil;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.system.common.exception.ProfileNotFound;
import ru.entaxy.esb.system.groups.registry.system.groups.profile.SystemGroupCollectorListener;
import ru.entaxy.esb.system.groups.registry.system.groups.profile.SystemGroupProfile;


public class SystemGroupProducer extends DefaultProducer {

    private static final Logger LOG = LoggerFactory.getLogger(SystemGroupProducer.class);
    private final SystemGroupEndpoint endpoint;

    private SystemGroupCollectorListener systemGroupCollectorListener;

    public SystemGroupProducer(SystemGroupEndpoint endpoint) {
        super(endpoint);
        this.endpoint = endpoint;
    }

    public void process(Exchange exchange) throws Exception {
        LOG.debug("In SystemGroupProducer " + exchange.getIn().getBody());

        //get System Group Profile from systemRegistry endpoint.getSystemName()
        SystemGroupProfile systemGroupProfile = null;

        systemGroupProfile = getSystemGroupProfile(endpoint.getSystemName());

        LOG.debug("Called system group profile " + (systemGroupProfile != null ? systemGroupProfile.getProfileName() : "NULL"));

        systemGroupProfile.send(exchange);
    }

    public SystemGroupProfile getSystemGroupProfile(String nameSystem) throws ProfileNotFound {
        systemGroupCollectorListener = getSystemGroupCollectorListener();
        LOG.info("Registry SystemGroupProfile {}; services count {}", nameSystem, systemGroupCollectorListener.getReferenceNames().size());
        SystemGroupProfile systemGroupProfile = null;
        if (systemGroupCollectorListener.isRegistered(nameSystem)) {
            systemGroupProfile = (SystemGroupProfile) systemGroupCollectorListener.getReference(nameSystem);
        } else {
            throw new ProfileNotFound("Profile for group system " + nameSystem + " not found");
        }
        return systemGroupProfile;
    }


    public SystemGroupCollectorListener getSystemGroupCollectorListener() {
        if (systemGroupCollectorListener == null) {
            BundleContext bundleContext = FrameworkUtil.getBundle(SystemGroupProducer.class).getBundleContext();
            ServiceReference<SystemGroupCollectorListener> systemGroupCollectorListenerServiceReference =
                    bundleContext.getServiceReference(SystemGroupCollectorListener.class);
            systemGroupCollectorListener = bundleContext.getService(systemGroupCollectorListenerServiceReference);
        }
        return systemGroupCollectorListener;
    }

}
