/*-
 * ~~~~~~licensing~~~~~~
 * system-group-impl
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.groups.registry.jpa.impl;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import ru.entaxy.esb.system.groups.registry.jpa.SystemGroupRegistryService;
import ru.entaxy.esb.system.groups.registry.jpa.entity.SystemGroupRegistry;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import javax.transaction.Transactional;
import java.util.Date;
import java.util.List;

/**
 * Implementation of the SystemGroupRegistry service using the JPA entity manager service (provided by Karaf).
 */
@Transactional
public class SystemGroupRegistryServiceImpl implements SystemGroupRegistryService {

    private SessionFactory sessionFactory;

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public Session getSession() {
        return sessionFactory.openSession();
    }

    @Override
    public List<SystemGroupRegistry> list() {
        List<SystemGroupRegistry> list;
        try (Session s = getSession()) {
            s.getTransaction().begin();
            CriteriaQuery<SystemGroupRegistry> cq = s.getCriteriaBuilder().createQuery(SystemGroupRegistry.class);
            cq.from(SystemGroupRegistry.class);
            list = s.createQuery(cq).getResultList();
            s.getTransaction().commit();
            s.close();
        }

        return list;
    }

    @Override
    public SystemGroupRegistry get(long id) {
        SystemGroupRegistry systemGroupRegistry;
        try (Session s = getSession()) {
            s.getTransaction().begin();

            CriteriaBuilder builder = s.getCriteriaBuilder();
            CriteriaQuery<SystemGroupRegistry> criteriaQuery = builder.createQuery(SystemGroupRegistry.class);
            Root<SystemGroupRegistry> root = criteriaQuery.from(SystemGroupRegistry.class);
            criteriaQuery.where(builder.equal(root.get("id"), id));
            systemGroupRegistry = s.createQuery(criteriaQuery).getSingleResult();

            s.getTransaction().commit();
            s.close();
        }
        return systemGroupRegistry;
    }

    @Override
    public SystemGroupRegistry getByUuid(String uuid) {
        SystemGroupRegistry systemGroupRegistry;
        try (Session s = getSession()) {
            s.getTransaction().begin();

            CriteriaBuilder builder = s.getCriteriaBuilder();
            CriteriaQuery<SystemGroupRegistry> criteriaQuery = builder.createQuery(SystemGroupRegistry.class);
            Root<SystemGroupRegistry> root = criteriaQuery.from(SystemGroupRegistry.class);
            criteriaQuery.where(builder.equal(root.get("uuid"), uuid));
            systemGroupRegistry = s.createQuery(criteriaQuery).getSingleResult();

            s.getTransaction().commit();
            s.close();
        }
        return systemGroupRegistry;
    }

    @Override
    public SystemGroupRegistry add(SystemGroupRegistry systemGroupRegistry) {
        try (Session s = getSession()) {
            s.getTransaction().begin();
            s.persist(systemGroupRegistry);
            s.getTransaction().commit();
            s.close();
        }
        return systemGroupRegistry;
    }

    @Override
    public SystemGroupRegistry add(String uuid, String name, String description, Date createDate, Date editDate,
                                   String createdBy, String editedBy, double version, String type, String connectorId,
                                   String descriptionFileUrl, String endpointName, String address, String responseType) {
        SystemGroupRegistry systemGroupRegistry = new SystemGroupRegistry();
        systemGroupRegistry.setUuid(uuid);
        systemGroupRegistry.setName(name);
        systemGroupRegistry.setDescription(description);
        systemGroupRegistry.setCreateDate(createDate);
        systemGroupRegistry.setEditDate(editDate);
        systemGroupRegistry.setCreatedBy(createdBy);
        systemGroupRegistry.setVersion(version);
        systemGroupRegistry.setType(type);
        systemGroupRegistry.setConnectorId(connectorId);
        systemGroupRegistry.setDescriptionFileUrl(descriptionFileUrl);
        systemGroupRegistry.setEndpointName(endpointName);
        systemGroupRegistry.setAddress(address);
        systemGroupRegistry.setResponseType(responseType);

        add(systemGroupRegistry);

        return systemGroupRegistry;
    }

    @Override
    public void remove(long id) {
        try (Session s = getSession()) {
            s.getTransaction().begin();

            SystemGroupRegistry systemGroupRegistry = get(id);
            s.delete(SystemGroupRegistry.class.getName(), systemGroupRegistry);

            s.getTransaction().commit();
            s.close();
        }
    }

}
