/*-
 * ~~~~~~licensing~~~~~~
 * bundle-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.management.bundle.jpa;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleType;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import java.util.List;

public class BundleServiceImpl implements BundleService {

    private SessionFactory sessionFactory;

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public Session getSession() {
        return sessionFactory.openSession();
    }

    @Override
    public List<BundleEntity> list() {
        List<BundleEntity> list;
        try (Session s = getSession()) {
            s.getTransaction().begin();
            CriteriaQuery<BundleEntity> cq = s.getCriteriaBuilder().createQuery(BundleEntity.class);
            cq.from(BundleEntity.class);
            list = s.createQuery(cq).getResultList();
            s.getTransaction().commit();
            s.close();
        }

        return list;
    }

    @Override
    public BundleEntity add(BundleEntity bundleEntity) {
        try (Session s = getSession()) {
            s.getTransaction().begin();
            s.saveOrUpdate(bundleEntity);
            s.getTransaction().commit();
            s.close();
        }
        return bundleEntity;
    }

    @Override
    public BundleEntity add(String name, String bundleUrl, String status) {
        BundleEntity bundleEntity = new BundleEntity();
        bundleEntity.setName(name);
        bundleEntity.setUrl(bundleUrl);
        bundleEntity.setStatus(status);
        return add(bundleEntity);
    }

    @Override
    public BundleEntity add(String name, BundleType type, String bundleUrl, String status) {
    	BundleEntity bundleEntity = new BundleEntity();
        bundleEntity.setName(name);
        bundleEntity.setType(type);
        bundleEntity.setUrl(bundleUrl);
        bundleEntity.setStatus(status);
        return add(bundleEntity);
    }
    
    @Override
    public BundleEntity getByName(String name) {
        BundleEntity bundleEntity;
        try (Session s = getSession()) {
            s.getTransaction().begin();

            CriteriaBuilder builder = s.getCriteriaBuilder();
            CriteriaQuery<BundleEntity> criteriaQuery = builder.createQuery(BundleEntity.class);
            Root<BundleEntity> root = criteriaQuery.from(BundleEntity.class);
            criteriaQuery.where(builder.equal(root.get("name"), name));
            bundleEntity = s.createQuery(criteriaQuery).uniqueResult();

            s.getTransaction().commit();
            s.close();
        }
        return bundleEntity;
    }

    @Override
    public BundleEntity update(String name, String status) {
        BundleEntity bundleEntity = getByName(name);
        bundleEntity.setStatus(status);
        return add(bundleEntity);
    }

}

