/*-
 * ~~~~~~licensing~~~~~~
 * connector-manager
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.management.connector.manager.mapper;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Mappings;
import org.mapstruct.Named;
import ru.entaxy.esb.system.connector.entity.Connector;
import ru.entaxy.esb.system.connector.entity.ConnectorParam;
import ru.entaxy.esb.system.management.bundle.jpa.dto.BundleDto;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;
import ru.entaxy.esb.system.management.connector.manager.dto.ConnectorDto;

import java.text.SimpleDateFormat;
import java.util.*;

@Mapper(componentModel = "spring")
public interface ConnectorMapper {

    @Mappings({
            @Mapping(target = "createDate", expression = "java(new java.util.Date())"),
            @Mapping(target = "version", source = "name", qualifiedByName = "generateVersion"),
            @Mapping(source = "connectorParams", target = "connectorParams", qualifiedByName = "toConnectorParams")
    })
    Connector toConnectorEntity(ConnectorDto connectorDto);

    @Mappings({
            @Mapping(source = "bundleEntity.status", target = "status"),
            @Mapping(source = "connectorParams", target = "connectorParams", qualifiedByName = "toConnectorParams")
    })
    ConnectorDto toConnectorDto(Connector connector);

    List<ConnectorDto> toConnectorDtos(List<Connector> connectorList);

    @Mappings({
            @Mapping(source = "bundleName", target = "name"),
            @Mapping(source = "bundleUrl", target = "url"),
    })
    BundleEntity toBundleEntity(BundleDto bundle);
    
    @Mappings({
        @Mapping(target = "bundleName", source = "name"),
        @Mapping(target = "bundleUrl", source = "url"),
    })
    BundleDto toBundleDto(BundleEntity bundle);

    default List<ConnectorParam> toConnectorParams(Map<String, String> connectorParams) {
        List<ConnectorParam> connectorParamList = new ArrayList<>();
        for (Map.Entry<String, String> entry : connectorParams.entrySet()) {
            ConnectorParam connectorParam = new ConnectorParam();
            connectorParam.setName(entry.getKey());
            connectorParam.setValue(entry.getValue());
            connectorParamList.add(connectorParam);
        }
        return connectorParamList;
    }

    default Map<String, String> toConnectorParams(List<ConnectorParam> connectorParams) {
        Map<String, String> connectorParamMap = new HashMap<>();
        for (ConnectorParam connectorParam : connectorParams) {
            connectorParamMap.put(connectorParam.getName(), connectorParam.getValue());
        }
        return connectorParamMap;
    }

    @Named("generateVersion")
    default String generateVersion(String version) {
        SimpleDateFormat format = new SimpleDateFormat("1.yyMMdd.HHmmss");
        return format.format(new Date());
    }
}
