/*-
 * ~~~~~~licensing~~~~~~
 * file-upload
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.management.file.upload;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.cxf.jaxrs.ext.multipart.Attachment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.esb.resources.EntaxyResourceService;

import javax.activation.DataHandler;
import javax.ws.rs.core.MultivaluedMap;
import java.io.InputStream;

public class FileUploadProcessor implements Processor {

    private static final Logger log = LoggerFactory.getLogger(FileUploadProcessor.class);

    private EntaxyResourceService entaxyResourceService;

    public void setEntaxyResourceService(EntaxyResourceService entaxyResourceService) {
        this.entaxyResourceService = entaxyResourceService;
    }

    @Override
    public void process(Exchange exchange) throws Exception {
        Attachment attachment = exchange.getIn().getBody(Attachment.class);
        DataHandler dataHandler = attachment.getDataHandler();

        InputStream inputStream = null;
        EntaxyResource entaxyResource = null;

        try {
            MultivaluedMap<String, String> map = attachment.getHeaders();

            inputStream = dataHandler.getInputStream();

            String protocol = (String) exchange.getIn().getHeader("protocol");
            String path = (String) exchange.getIn().getHeader("path");
            String location = protocol + ":" + path;

            entaxyResource = entaxyResourceService.getResource(location);

            /* For now we allow resource reloading */
            // if (!entaxyResource.exists()) {
                entaxyResource.save(inputStream);
            // }
        } catch (Exception e) {
            log.error("Error while uploading file: " + e.getMessage());
        } finally {
            inputStream.close();
        }
        exchange.getMessage().setBody(entaxyResource.getName());
    }
}
