/*-
 * ~~~~~~licensing~~~~~~
 * artemis-plugin-entaxy
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.activemq.hawtio.plugin;

import io.hawt.web.plugin.HawtioPlugin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

/**
 * The Plugin Context Listener used to load in the plugin
 **/
public class PluginContextListener implements ServletContextListener {

   private static final Logger LOG = LoggerFactory.getLogger(PluginContextListener.class);

   HawtioPlugin plugin = null;

   @Override
   public void contextInitialized(ServletContextEvent servletContextEvent) {

      ServletContext context = servletContextEvent.getServletContext();

      plugin = new HawtioPlugin();
      plugin.setContext(context.getContextPath());
      plugin.setName(context.getInitParameter("plugin-name"));
      plugin.setScripts(context.getInitParameter("plugin-scripts"));
      plugin.setDomain(null);

      try {
         plugin.init();
      } catch (Exception e) {
         throw createServletException(e);
      }

      LOG.info("Initialized {} plugin", plugin.getName());
   }

   @Override
   public void contextDestroyed(ServletContextEvent servletContextEvent) {
      try {
         plugin.destroy();
      } catch (Exception e) {
         throw createServletException(e);
      }

      LOG.info("Destroyed {} plugin", plugin.getName());
   }

   protected RuntimeException createServletException(Exception e) {
      return new RuntimeException(e);
   }

}
