/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module.component('entaxyAddFolderModal', {
        bindings: {
            modalInstance: '<',
            resolve: '<'
        },
        template:
        `
          <div class="entaxy-simple-modal-container">
            <div class="modal-header">
              <button type="button" class="close" aria-label="Close" ng-click="$ctrl.cancel()">
                <span class="pficon pficon-close" aria-hidden="true"></span>
              </button>
              <h4 class="modal-title">Set folder name</h4>
            </div>
            <div class="modal-body-small">
              <div class="form-group" ng-class="{'has-error': $ctrl.error}">
                <div class="col-sm-12">
                  <input type="text" class="form-control" ng-model="$ctrl.name">
                  <span class="help-block" ng-show="$ctrl.error">{{$ctrl.error}}</span>
                </div>
              </div>
            </div>
            <div class="modal-footer">
              <button type="submit" class="btn btn-primary" ng-click="$ctrl.save()">Save</button>
            </div>
          </div>
        `,
        controller: entaxyAddFolderModalController
    })
    .name;

    function entaxyAddFolderModalController($uibModal) {
        'ngInject';

        let ctrl = this;

        ctrl.$onInit = function() {
            ctrl.names = ctrl.resolve.names;
        }

        ctrl.cancel = function(reason) {
            ctrl.modalInstance.dismiss(reason);
        }

        ctrl.save = function() {
            ctrl.error = validate();
            if (!ctrl.error) {
                ctrl.modalInstance.close(ctrl.name);
            }
        }

        function validate() {
            let error;
            if (!ctrl.name || ctrl.name.length == 0 || ctrl.name.trim().length == 0) {
                return 'Folder name must not be empty';
            }
            ['/', '\\', ':', '*', '?', '<', '>', '|'].forEach((symbol) => {
                if (ctrl.name.indexOf(symbol) >= 0) {
                    error = 'Folder name must not include symbols /\\:*?<>|';
                }
            });
            ctrl.names.forEach((name) => {
                if (name === ctrl.name) {
                    error = 'Folder with this name already exists';
                }
            });
            return error;
        }
    }
    entaxyAddFolderModalController.$inject = ['$uibModal'];

})(Entaxy || (Entaxy = {}));
