/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module
    .component('entaxyConnectors', {
        bindings: {
            pageTitle: '@'
        },
        template:
        `
          <h2>
            {{$ctrl.pageTitle}}
          </h2>

          <entaxy-table runtime-type="entaxy.runtime.connector"
                status-name="BundleState" object-name="systemName"
                extra-primary-actions-config="$ctrl.config"
                is-parent-dependent="true"></entaxy-table>

        `,
        controller: EntaxyConnectorsController
    })
    .name;

    function EntaxyConnectorsController(workspace, $scope, $uibModal, operationsService, jolokia, entaxyLegacyConnectorService) {
        'ngInject';

        var ctrl = this;
        ctrl.workspace = workspace;

        let primaryActions = [ { name: 'Add Legacy Connector', actionFn: () => ctrl.showAddConnectorModal() } ];
        ctrl.config = {
            primaryActions: primaryActions,
            isTableUpdateNeeded: true
        }

        ctrl.showAddConnectorModal = function() {
            let selectedMBeanName = workspace.getSelectedMBeanName();

            let resolve = {
                profileName: () => jolokia.getAttribute(selectedMBeanName, 'Name')
            };

            entaxyLegacyConnectorService.openAddLegacyConnectorModal(resolve).then(connectorArgs => {
                addConnector(connectorArgs);
            });
        }

        function addConnector(connectorArgs) {

            let args = entaxyLegacyConnectorService.getArguments(connectorArgs);
            let mbeanName = ctrl.workspace.getSelectedMBeanName();

            operationsService.executeOperation(mbeanName, { name: 'addConnector' }, args)
            .then(result => {
                Core.notification('success', result, 3000);
            }).catch(error => {
                Core.notification('danger', error, 5000);
            });
        }

        ctrl.dismissAlert = () => ctrl.alert = null;

    }
    EntaxyConnectorsController.$inject = ['workspace', '$scope', '$uibModal', 'operationsService', 'jolokia', 'entaxyLegacyConnectorService'];

})(Entaxy || (Entaxy = {}));
