/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module
    .directive('entaxyFileDrop', ['$parse', function ($parse) {
        return {
            restrict: 'A',
            replace: false,
            require: 'ngModel',
            link: function (scope, element, attrs, ngModel) {

                let enterTarget = null;

                element.on('dragenter', function (e) {
                    e.preventDefault();
                    e.stopPropagation();

                    enterTarget = e.target;
                    element.addClass('highlighted');
                    createNotification();
                });

                element.on('dragover', function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                });

                element.on('dragleave', function (e) {
                    if (e.target === enterTarget) {
                        e.preventDefault();
                        e.stopPropagation();

                        element.removeClass('highlighted');
                        removeNotification();
                    }
                });

                let notificationContainer = null;

                function createNotification() {
                    if (notificationContainer === null) {
                        notificationContainer = document.createElement('div');
                        notificationContainer.className = 'upload-notification';
                        let span = document.createElement('span');
                        span.className = 'fa fa-upload';
                        notificationContainer.appendChild(span);
                        notificationContainer.appendChild(document.createTextNode('Drop file to upload'));

                        let elementWidth = element.width();
                        let offset = 150 + elementWidth/2 + 20;
                        notificationContainer.style.left = 'calc(100vw - ' + offset + 'px)';

                        element.append(notificationContainer);
                    }
                }

                function removeNotification() {
                    if (notificationContainer !== null) {
                        notificationContainer.parentNode.removeChild(notificationContainer);
                        notificationContainer = null;
                    }
                }

                element.on('drop', function (e) {
                    e.preventDefault();
                    e.stopPropagation();

                    element.removeClass('highlighted');
                    removeNotification();

                    if (e.originalEvent.dataTransfer && e.originalEvent.dataTransfer.files.length > 0) {
                        ngModel.$setViewValue(e.originalEvent.dataTransfer.files[0]);

                        let processFn = $parse(attrs.processFn);
                        if (processFn) {
                            processFn(scope);
                        }
                    }
                    return false;
                });
            }
        };
    }])
    .name;
})(Entaxy || (Entaxy = {}));
