/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module.component('entaxyModal', {
        bindings: {
            modalInstance: '<',
            resolve: '<'
        },
        template:
        `
          <div class="entaxy-modal-container">
            <div class="modal-header">
              <button type="button" class="close" aria-label="Close" ng-click="$ctrl.cancel()">
                <span class="pficon pficon-close" aria-hidden="true"></span>
              </button>
              <h4 class="modal-title">{{$ctrl.modalTitle}}</h4>
            </div>

            <div ng-if="$ctrl.step1">
              <div class="modal-body-header">
                <h2>{{$ctrl.firstStepTitle}}</h2>
              </div>
              <div class="modal-body">
                <entaxy-modal-list items="$ctrl.profiles" selected="$ctrl.selectedProfile"
                      change-selection="$ctrl.changeProfileSelection" enable-dbl-click="true"></entaxy-modal-list>
              </div>
              <div class="modal-footer">
                <button type="button" class="btn btn-default" ng-click="$ctrl.next()" ng-disabled="!$ctrl.selectedProfile">Next</button>
              </div>
            </div>

            <div ng-if="$ctrl.step2">
              <div class="modal-body-header">
                <h2>{{$ctrl.secondStepTitle}}</h2>
              </div>
              <div class="modal-body">
                <entaxy-modal-list-with-description items="$ctrl.factories" selected-item="$ctrl.selectedFactory"
                    change-selection="$ctrl.changeFactorySelection"></entaxy-modal-list-with-description>
              </div>
              <div class="modal-footer">
                <button type="button" class="btn btn-default" ng-if="$ctrl.profiles" ng-click="$ctrl.back()">Back</button>
                <button type="button" class="btn btn-default" ng-click="$ctrl.next()" ng-disabled="!$ctrl.selectedFactory">Next</button>
              </div>
            </div>

            <div ng-if="$ctrl.step3">
              <div class="modal-body-header">
                <h2>{{$ctrl.thirdStepTitle}}</h2>
              </div>
              <div class="modal-body">
                <entaxy-modal-group-fields groups="$ctrl.groups" fields="$ctrl.formFields"
                      use-form-dirty="true" is-form-dirty="$ctrl.isFormDirty"
                      errors="$ctrl.errors" mode="$ctrl.mode"></entaxy-modal-group-fields>
              </div>
              <div class="modal-footer">
                <button type="button" class="btn btn-default" ng-if="$ctrl.factories.length > 1 || $ctrl.profiles" ng-click="$ctrl.back()">Back</button>
                <button type="submit" class="btn btn-primary" ng-click="$ctrl.save()">{{$ctrl.submitBtnTitle}}</button>
              </div>
            <div>
          </div>
        `,
        controller: entaxyModalController
    })
    .name;

    function entaxyModalController($scope, operationsService, entaxyService) {
        'ngInject';

        var ctrl = this;

        ctrl.$onInit = function() {

            ctrl.mode = ctrl.resolve.mode;

            if (!ctrl.mode) {
                ctrl.cancel('Mode is not defined');
            }

            ctrl.itemType = ctrl.resolve.itemType ? ctrl.resolve.itemType : 'Item';

            let extraToTitle = ctrl.resolve.parentName ? (' to ' + ctrl.resolve.parentName) : '';

            ctrl.modalTitle = ctrl.mode + ' ' + ctrl.itemType + extraToTitle;

            ctrl.submitBtnTitle = Entaxy.getButtonTitleByMode(ctrl.mode);

            let itemName = ctrl.resolve.itemName;

            ctrl.firstStepTitle = 'Choose profile to attach connector to';

            ctrl.secondStepTitle = ctrl.resolve.secondStepTitle ? ctrl.resolve.secondStepTitle : 'Choose factory';

            ctrl.thirdStepTitle = itemName ? ctrl.mode + ' properties for ' + itemName
                : ctrl.resolve.thirdStepTitle ? ctrl.resolve.thirdStepTitle : 'Fill in fields';

            if (ctrl.mode !== Entaxy.MODAL_MODES.ADD) {
                let properties = Object.entries(ctrl.resolve.info.properties);
                ctrl.properties = properties.map(property => { return { name: property[0], value: property[1] }; });
            }

            ctrl.profiles = ctrl.resolve.profiles;

            ctrl.factories = ctrl.resolve.factories ? ctrl.resolve.factories : null;

            if (!ctrl.factories || ctrl.factories.length === 0) {
                ctrl.cancel('Factories are not found');
            }

            ctrl.factories.sort(Entaxy.compareBy('displayName'));

            if (ctrl.profiles) {
                ctrl.step1 = true;
                ctrl.step2 = false;
                ctrl.step3 = false
            } else {
                if (ctrl.factories.length > 1) {
                    ctrl.step1 = false;
                    ctrl.step2 = true;
                    ctrl.step3 = false;
                } else if (ctrl.factories.length === 1) {
                    ctrl.next();
                }
            }
        }

        ctrl.next = function() {

            if (ctrl.step1 === true) {
                if (ctrl.factories.length > 1) {
                    ctrl.step1 = false;
                    ctrl.step2 = true;
                } else if (ctrl.factories.length === 1) {
                    defineFormFields();
                }
            } else {
                defineFormFields();
            }
        }

        function defineFormFields() {
            let container = ctrl.formFields ? ctrl.formFields.find(field => field.name === '__entaxyContainerId') : undefined;

            if (!ctrl.formFields || ctrl.selectedFactory.name !== ctrl.formFields[0].value
                                || (ctrl.profiles && container.value !== ctrl.selectedProfile.name)) {

                ctrl.formFields = [];

                ctrl.formFields.push({
                    label: 'Factory Id',
                    name: 'factoryId',
                    type: 'java.lang.String',
                    helpText: null,
                    value: ctrl.factories.length > 1 ? ctrl.selectedFactory.name : ctrl.factories[0].name,
                    readOnly: true,
                    required: ctrl.mode === Entaxy.MODAL_MODES.VIEW ? false : true,
                    group: 'general'
                });

                let mbeanName = ctrl.factories.length > 1 ? ctrl.selectedFactory.mbeanName : ctrl.factories[0].mbeanName;

                operationsService.executeOperation(mbeanName, { name: 'getFields' }, [ 'init' ])
                .then((response) => {

                    let groups = new Set();
                    groups.add('general');

                    _.forEach(JSON.parse(response), (field) => {

                        let objectId = undefined;
                        if (ctrl.mode !== Entaxy.MODAL_MODES.ADD) {
                            objectId = ctrl.resolve.info['objectId'];
                        }
                        let profileName = ctrl.selectedProfile ? ctrl.selectedProfile.name : undefined;
                        let formField = entaxyService.makeFormField(field, objectId, ctrl.properties, ctrl.mode, profileName);
                        if (formField) {
                            ctrl.formFields.push(formField);
                            groups.add(formField.group);
                        }
                    });

                    ctrl.formFields.forEach(formField => formField.type = Entaxy.convertToHtmlInputType(formField.type));

                    processDependencies();

                    ctrl.groups = Array.from(groups).map((group) => { return { name: group, displayName: group }; });

                    ctrl.step1 = false;
                    ctrl.step2 = false;
                    ctrl.step3 = true;
                });
            } else {
                ctrl.step1 = false;
                ctrl.step2 = false;
                ctrl.step3 = true;
            }
        }

        function processDependencies() {
            let dependentFormFields = entaxyService.getDependentFormFields(ctrl.formFields);
            ctrl.definingFormFields = entaxyService.getDefiningFormFields(dependentFormFields, ctrl.formFields);

            for (let i = 0; i < ctrl.definingFormFields.length; i++) {
                if (ctrl.definingFormFields[i].value) {
                    let dependentFormFieldsWithListType = dependentFormFields
                            .filter(field => (field.typeInfo && field.typeInfo.type === 'list'));
                    entaxyService.processDependencies(dependentFormFieldsWithListType,
                                                      ctrl.definingFormFields,
                                                      ctrl.definingFormFields[i].name,
                                                      ctrl.definingFormFields[i].value,
                                                      undefined);
                }
                $scope.$watch('$ctrl.definingFormFields[' + i + '].value', function(newValue, oldValue) {
                    if (newValue !== oldValue) {
                        entaxyService.processDependencies(dependentFormFields,
                                                          ctrl.definingFormFields,
                                                          ctrl.definingFormFields[i].name,
                                                          newValue,
                                                          oldValue);
                    }
                });
            }
        }

        ctrl.back = function() {
            if (ctrl.step2 === true) {
                ctrl.step1 = true;
                ctrl.step2 = false;
            } else if (ctrl.step3 === true) {
                ctrl.errors = undefined;
                ctrl.step2 = true;
                ctrl.step3 = false;
            }
        }

        ctrl.cancel = function(reason) {
            ctrl.modalInstance.dismiss(reason);
        }

        ctrl.changeProfileSelection = function(item, isDblClicked) {
            ctrl.selectedProfile = item;
            if (isDblClicked) {
                ctrl.next();
            }
        }

        ctrl.changeFactorySelection = function(item, isDblClicked) {
            ctrl.selectedFactory = item;
            if (isDblClicked) {
                ctrl.next();
            }
        }

        ctrl.save = function() {

            if (ctrl.mode === Entaxy.MODAL_MODES.VIEW) {
                ctrl.cancel();
            }

            let isConfirmationNeeded = ctrl.mode === Entaxy.MODAL_MODES.EDIT && !ctrl.isFormDirty;
            entaxyService.requestConfirmationForSavingIfNeededAndProceed(isConfirmationNeeded, save);
        }

        function save () {
            let selectedFactory = ctrl.selectedFactory ? ctrl.selectedFactory : ctrl.factories[0];
            let profileMBeanName = ctrl.selectedProfile ? ctrl.selectedProfile.mbeanName : undefined;

            entaxyService.validateFields(ctrl.formFields, selectedFactory, ctrl.itemType, profileMBeanName)
            .then(errors => {
                ctrl.errors = errors;

                if (Object.keys(ctrl.errors).length === 0) {
                    let args = entaxyService.getArguments(ctrl.formFields, ctrl.factories);
                    ctrl.modalInstance.close(args);
                }
            });
        }
    }
    entaxyModalController.$inject = ['$scope', 'operationsService', 'entaxyService'];

})(Entaxy || (Entaxy = {}));
